@@ -186,6 +186,10 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * The key for the color(s) of light sources in a light (mandatory if light source abscissa is defined).
      */
     LIGHT_SOURCE_COLOR("lightSourceColor"),
+    /**
+     * The key for the color(s) of light sources in a light (optional).
+     */
+    LIGHT_SOURCE_DIAMETER("lightSourceDiameter"),
     /**
      * The key for the elevation in centimeters of a piece of furniture (optional).
      */
@@ -203,11 +207,23 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      */
     CREATOR("creator"),
     /**
-     * The key for the resizability of a piece of furniture (optional).
+     * The key for the resizability of a piece of furniture (optional, <code>true</code> by default).
      * If the value of this key is <code>false</code>, the piece of furniture
      * will be considered as a piece with a fixed size. 
      */
     RESIZABLE("resizable"),
+    /**
+     * The key for the deformability of a piece of furniture (optional, <code>true</code> by default).
+     * If the value of this key is <code>false</code>, the piece of furniture
+     * will be considered as a piece that should always keep its proportions when resized. 
+     */
+    DEFORMABLE("deformable"),
+    /**
+     * The key for the texturable capability of a piece of furniture (optional, <code>true</code> by default).
+     * If the value of this key is <code>false</code>, the piece of furniture
+     * will be considered as a piece that will always keep the same color or texture. 
+     */
+    TEXTURABLE("texturable"),
     /**
      * The key for the price of a piece of furniture (optional).
      */
@@ -260,8 +276,17 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
    * Creates a default furniture catalog read from resources and   
    * furniture plugin folder if <code>furniturePluginFolder</code> isn't <code>null</code>.
    */
-;  public DefaultFurnitureCatalog(final UserPreferences preferences, 
+  public DefaultFurnitureCatalog(final UserPreferences preferences, 
                                  File furniturePluginFolder) {
+    this(preferences, furniturePluginFolder == null ? null : new File [] {furniturePluginFolder});
+  }
+  
+  /**
+   * Creates a default furniture catalog read from resources and   
+   * furniture plugin folders if <code>furniturePluginFolders</code> isn't <code>null</code>.
+   */
+  public DefaultFurnitureCatalog(final UserPreferences preferences, 
+                                 File [] furniturePluginFolders) {
     Map<FurnitureCategory, Map<CatalogPieceOfFurniture, Integer>> furnitureHomonymsCounter = 
         new HashMap<FurnitureCategory, Map<CatalogPieceOfFurniture,Integer>>();
     List<String> identifiedFurniture = new ArrayList<String>();
@@ -280,21 +305,23 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     readFurnitureCatalog(classPackage + "." + ADDITIONAL_FURNITURE_CATALOG_FAMILY, 
         preferences, furnitureHomonymsCounter, identifiedFurniture);
     
-    if (furniturePluginFolder != null) {
-      // Try to load sh3f files from furniture plugin folder
-      File [] pluginFurnitureCatalogFiles = furniturePluginFolder.listFiles(new FileFilter () {
-        public boolean accept(File pathname) {
-          return pathname.isFile();
-        }
-      });
-      
-      if (pluginFurnitureCatalogFiles != null) {
-        // Treat furniture catalog files in reverse order so file named with a date will be taken into account 
-        // from most recent to least recent
-        Arrays.sort(pluginFurnitureCatalogFiles, Collections.reverseOrder());
-        for (File pluginFurnitureCatalogFile : pluginFurnitureCatalogFiles) {
-          // Try to load the properties file describing furniture catalog from current file  
-          readPluginFurnitureCatalog(pluginFurnitureCatalogFile, furnitureHomonymsCounter, identifiedFurniture);
+    if (furniturePluginFolders != null) {
+      for (File furniturePluginFolder : furniturePluginFolders) {
+        // Try to load sh3f files from furniture plugin folder
+        File [] pluginFurnitureCatalogFiles = furniturePluginFolder.listFiles(new FileFilter () {
+          public boolean accept(File pathname) {
+            return pathname.isFile();
+          }
+        });
+        
+        if (pluginFurnitureCatalogFiles != null) {
+          // Treat furniture catalog files in reverse order so file named with a date will be taken into account 
+          // from most recent to least recent
+          Arrays.sort(pluginFurnitureCatalogFiles, Collections.reverseOrder());
+          for (File pluginFurnitureCatalogFile : pluginFurnitureCatalogFiles) {
+            // Try to load the properties file describing furniture catalog from current file  
+            readPluginFurnitureCatalog(pluginFurnitureCatalogFile, furnitureHomonymsCounter, identifiedFurniture);
+          }
         }
       }
     }
@@ -461,12 +488,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         furnitureCatalogUrl, furnitureResourcesUrlBase, false, false);
     Content planIcon = getContent(resource, PropertyKey.PLAN_ICON.getKey(index), 
         furnitureCatalogUrl, furnitureResourcesUrlBase, false, true);
-    boolean multiPartModel = false;
-    try {
-      multiPartModel = Boolean.parseBoolean(resource.getString(PropertyKey.MULTI_PART_MODEL.getKey(index)));
-    } catch (MissingResourceException ex) {
-      // By default inDirectory is false
-    }
+    boolean multiPartModel = getOptionalBoolean(resource, PropertyKey.MULTI_PART_MODEL.getKey(index), false);
     Content model = getContent(resource, PropertyKey.MODEL.getKey(index), 
         furnitureCatalogUrl, furnitureResourcesUrlBase, multiPartModel, false);
     float width = Float.parseFloat(resource.getString(PropertyKey.WIDTH.getKey(index)));
@@ -479,12 +501,9 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     // By default creator is eTeks
     String creator = getOptionalString(resource, PropertyKey.CREATOR.getKey(index), null);
     String id = getOptionalString(resource, PropertyKey.ID.getKey(index), null);
-    boolean resizable = true;
-    try {
-      resizable = Boolean.parseBoolean(resource.getString(PropertyKey.RESIZABLE.getKey(index)));
-    } catch (MissingResourceException ex) {
-      // By default piece is resizable
-    }
+    boolean resizable = getOptionalBoolean(resource, PropertyKey.RESIZABLE.getKey(index), true);
+    boolean deformable = getOptionalBoolean(resource, PropertyKey.DEFORMABLE.getKey(index), true);
+    boolean texturable = getOptionalBoolean(resource, PropertyKey.TEXTURABLE.getKey(index), true);
     BigDecimal price = null;
     try {
       price = new BigDecimal(resource.getString(PropertyKey.PRICE.getKey(index)));
@@ -507,17 +526,17 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       return new CatalogDoorOrWindow(id, name, description, icon, planIcon, model,
           width, depth, height, elevation, movable, 
           wallThicknessPercentage, wallDistancePercentage, sashes, modelRotation, creator, 
-          resizable, price, valueAddedTaxPercentage);
+          resizable, deformable, texturable, price, valueAddedTaxPercentage);
     } else {
       LightSource [] lightSources = getLightSources(resource, index, width, depth, height);
       if (lightSources != null) {
         return new CatalogLight(id, name, description, icon, planIcon, model,
             width, depth, height, elevation, movable, lightSources, modelRotation, creator, 
-            resizable, price, valueAddedTaxPercentage);
+            resizable, deformable, texturable, price, valueAddedTaxPercentage);
       } else {
         return new CatalogPieceOfFurniture(id, name, description, icon, planIcon, model,
             width, depth, height, elevation, movable, modelRotation, creator, 
-            resizable, price, valueAddedTaxPercentage);
+            resizable, deformable, texturable, price, valueAddedTaxPercentage);
       }
     }
   }
@@ -564,7 +583,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
             doorOrWindow.isMovable(), doorOrWindow.getWallThickness(), 
             doorOrWindow.getWallDistance(), doorOrWindow.getSashes(), 
             doorOrWindow.getModelRotation(), doorOrWindow.getCreator(),
-            doorOrWindow.isResizable(), doorOrWindow.getPrice(), doorOrWindow.getValueAddedTaxPercentage());
+            doorOrWindow.isResizable(), doorOrWindow.isDeformable(), doorOrWindow.isTexturable(), 
+            doorOrWindow.getPrice(), doorOrWindow.getValueAddedTaxPercentage());
       } else if (piece instanceof CatalogLight) {
         CatalogLight light = (CatalogLight)piece;
         piece = new CatalogLight(light.getId(), suffixedName,
@@ -572,13 +592,15 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
             light.getWidth(), light.getDepth(), light.getHeight(), light.getElevation(), 
             light.isMovable(), light.getLightSources(), 
             light.getModelRotation(), light.getCreator(),
-            light.isResizable(), light.getPrice(), light.getValueAddedTaxPercentage());
+            light.isResizable(), light.isDeformable(), light.isTexturable(),
+            light.getPrice(), light.getValueAddedTaxPercentage());
       } else {
         piece = new CatalogPieceOfFurniture(piece.getId(), suffixedName,
             piece.getDescription(), piece.getIcon(), piece.getPlanIcon(), piece.getModel(),
             piece.getWidth(), piece.getDepth(), piece.getHeight(), piece.getElevation(), 
             piece.isMovable(), piece.getModelRotation(), piece.getCreator(),
-            piece.isResizable(), piece.getPrice(), piece.getValueAddedTaxPercentage());
+            piece.isResizable(), piece.isDeformable(), piece.isTexturable(),
+            piece.getPrice(), piece.getValueAddedTaxPercentage());
       }
       add(pieceCategory, piece, furnitureHomonymsCounter);
     }
@@ -738,6 +760,17 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         throw new IllegalArgumentException(
             "Expected " + lightSourceX.length + " values in " + PropertyKey.LIGHT_SOURCE_COLOR.getKey(index) + " key");
       }
+      String lightSourceDiametersString = getOptionalString(resource, PropertyKey.LIGHT_SOURCE_DIAMETER.getKey(index), null);
+      String [] lightSourceDiameters;
+      if (lightSourceDiametersString != null) {
+        lightSourceDiameters = lightSourceDiametersString.split(" ");
+        if (lightSourceDiameters.length != lightSourceX.length) {
+          throw new IllegalArgumentException(
+              "Expected " + lightSourceX.length + " values in " + PropertyKey.LIGHT_SOURCE_DIAMETER.getKey(index) + " key");
+        }
+      } else {
+        lightSourceDiameters = null;
+      }
       
       lightSources = new LightSource [lightSourceX.length];
       for (int i = 0; i < lightSources.length; i++) {
@@ -748,7 +781,10 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         lightSources [i] = new LightSource(Float.parseFloat(lightSourceX [i]) / lightWidth, 
             Float.parseFloat(lightSourceY [i]) / lightDepth, 
             Float.parseFloat(lightSourceZ [i]) / lightHeight, 
-            color);
+            color,
+            lightSourceDiameters != null
+                ? Float.parseFloat(lightSourceDiameters [i]) / lightWidth
+                : null);
       }
     }     
     return lightSources;
@@ -781,5 +817,19 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       return defaultValue;
     }
   }
+
+  /**
+   * Returns the boolean value of <code>propertyKey</code> in <code>resource</code>, 
+   * or <code>defaultValue</code> if the property doesn't exist.
+   */
+  private boolean getOptionalBoolean(ResourceBundle resource, 
+                                     String propertyKey,
+                                     boolean defaultValue) {
+    try {
+      return Boolean.parseBoolean(resource.getString(propertyKey));
+    } catch (MissingResourceException ex) {
+      return defaultValue;
+    }
+  }
 }
 
