@@ -20,8 +20,10 @@
 package com.eteks.sweethome3d.j3d;
 
 import java.awt.Color;
+import java.lang.ref.WeakReference;
 import java.util.Enumeration;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 
 import javax.media.j3d.Appearance;
@@ -52,22 +54,25 @@ import com.eteks.sweethome3d.model.HomeEnvironment;
 import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.Light;
+import com.eteks.sweethome3d.model.Selectable;
+import com.eteks.sweethome3d.model.SelectionEvent;
+import com.eteks.sweethome3d.model.SelectionListener;
 import com.sun.j3d.utils.geometry.Box;
 
 /**
  * Root of piece of furniture branch.
  */
 public class HomePieceOfFurniture3D extends Object3DBranch {
-  private static final Material               DEFAULT_TEXTURED_SHAPE_MATERIAL = new Material();
   private static final TransparencyAttributes DEFAULT_TEXTURED_SHAPE_TRANSPARENCY_ATTRIBUTES = 
       new TransparencyAttributes(TransparencyAttributes.NICEST, 0);
+  private static final PolygonAttributes      DEFAULT_TEXTURED_SHAPE_POLYGON_ATTRIBUTES = 
+      new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0);
   private static final TextureAttributes      MODULATE_TEXTURE_ATTRIBUTES = new TextureAttributes();
   
   private final Home home;
   
   static {
-    DEFAULT_TEXTURED_SHAPE_MATERIAL.setShininess(1);
-    DEFAULT_TEXTURED_SHAPE_MATERIAL.setSpecularColor(0, 0, 0);
     MODULATE_TEXTURE_ATTRIBUTES.setTextureMode(TextureAttributes.MODULATE);
   }
   
@@ -198,19 +203,16 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     HomePieceOfFurniture piece = (HomePieceOfFurniture)getUserData();
     Node filledModelNode = getFilledModelNode();
     if (piece.getColor() != null) {
-      Integer color = piece.getColor();
-      Color3f materialColor = new Color3f(((color >>> 16) & 0xFF) / 256f,
-                                           ((color >>> 8) & 0xFF) / 256f,
-                                                   (color & 0xFF) / 256f);
-      Material material = new Material(materialColor, new Color3f(), materialColor, materialColor, 64);
-      setMaterialAndTexture(filledModelNode, material, null, false, null, null, new HashSet<Appearance>());
+      setColorAndTexture(filledModelNode, piece.getColor(), null, piece.getShininess(), 
+          false, null, null, new HashSet<Appearance>());
     } else if (piece.getTexture() != null) {
-      setMaterialAndTexture(filledModelNode, null, piece.getTexture(), waitTextureLoadingEnd, 
-          new Vector3f(piece.getWidth(), piece.getHeight(), piece.getDepth()),
+      setColorAndTexture(filledModelNode, null, piece.getTexture(), piece.getShininess(),
+          waitTextureLoadingEnd, new Vector3f(piece.getWidth(), piece.getHeight(), piece.getDepth()),
           ModelManager.getInstance().getSize(((Group)filledModelNode).getChild(0)), new HashSet<Appearance>());
     } else {
       // Set default material and texture of model
-      setMaterialAndTexture(filledModelNode, null, null, false, null, null, new HashSet<Appearance>());
+      setColorAndTexture(filledModelNode, null, null, piece.getShininess(), 
+          false, null, null, new HashSet<Appearance>());
     }
   }
 
@@ -307,8 +309,37 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     updatePieceOfFurnitureColorAndTexture(waitTextureLoadingEnd);      
     updatePieceOfFurnitureVisibility();
     updatePieceOfFurnitureModelMirrored();
+
+    // Manage light sources visibility 
+    if (this.home != null 
+        && getUserData() instanceof Light) {
+      this.home.addSelectionListener(new LightSelectionListener(this));
+    }
   }
 
+  /**
+   * Selection listener bound to this object with a weak reference to avoid
+   * strong link between home and this tree.  
+   */
+  private static class LightSelectionListener implements SelectionListener {
+    private WeakReference<HomePieceOfFurniture3D>  piece;
+
+    public LightSelectionListener(HomePieceOfFurniture3D piece) {
+      this.piece = new WeakReference<HomePieceOfFurniture3D>(piece);
+    }
+    
+    public void selectionChanged(SelectionEvent ev) {
+      // If piece 3D was garbage collected, remove this listener from home
+      HomePieceOfFurniture3D piece3D = this.piece.get();
+      Home home = (Home)ev.getSource();
+      if (piece3D == null) {
+        home.removeSelectionListener(this);
+      } else {
+        piece3D.updatePieceOfFurnitureVisibility();
+      }
+    }
+  }
+  
   /**
    * Returns a box that may replace model. 
    */
@@ -390,19 +421,20 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
 
   /**
    * Sets the material and texture attribute of all <code>Shape3D</code> children nodes of <code>node</code> 
-   * with a given <code>material</code>. 
+   * from the given <code>color</code> and <code>texture</code>. 
    */
-  private void setMaterialAndTexture(Node node, Material material, HomeTexture texture, boolean waitTextureLoadingEnd,
-                                     Vector3f pieceSize, Vector3f modelSize, Set<Appearance> modifiedAppearances) {
+  private void setColorAndTexture(Node node, Integer color, HomeTexture texture, 
+                                  Float shininess, boolean waitTextureLoadingEnd,
+                                  Vector3f pieceSize, Vector3f modelSize, Set<Appearance> modifiedAppearances) {
     if (node instanceof Group) {
       // Set material and texture of all children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        setMaterialAndTexture((Node)enumeration.nextElement(), material, texture, waitTextureLoadingEnd,
+        setColorAndTexture((Node)enumeration.nextElement(), color, texture, shininess, waitTextureLoadingEnd,
             pieceSize, modelSize, modifiedAppearances);
       }
     } else if (node instanceof Link) {
-      setMaterialAndTexture(((Link)node).getSharedGroup(), material, texture, waitTextureLoadingEnd,
+      setColorAndTexture(((Link)node).getSharedGroup(), color, texture, shininess, waitTextureLoadingEnd,
           pieceSize, modelSize, modifiedAppearances);
     } else if (node instanceof Shape3D) {
       final Shape3D shape = (Shape3D)node;
@@ -424,16 +456,23 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
             defaultMaterialAndTexture = new DefaultMaterialAndTexture(appearance);
             appearance.setUserData(defaultMaterialAndTexture);
           }
-          if (material != null && defaultMaterialAndTexture.getTexture() == null) {
+          float materialShininess = shininess != null
+              ? shininess.floatValue()
+              : (appearance.getMaterial() != null
+                  ? appearance.getMaterial().getShininess() / 128f
+                  : 0);
+          if (color != null && defaultMaterialAndTexture.getTexture() == null) {
             // Change material if no default texture is displayed on the shape
             // (textures always keep the colors of their image file)
-            appearance.setMaterial(material);
+            appearance.setMaterial(getMaterial(color, color, materialShininess));
+            appearance.setTransparencyAttributes(defaultMaterialAndTexture.getTransparencyAttributes());
+            appearance.setPolygonAttributes(defaultMaterialAndTexture.getPolygonAttributes());
             appearance.setTexCoordGeneration(defaultMaterialAndTexture.getTexCoordGeneration());
             appearance.setTextureAttributes(defaultMaterialAndTexture.getTextureAttributes());
             appearance.setTexture(null);
-          } else if (material == null && texture != null) {
+          } else if (color == null && texture != null) {
             // Change material to white then texture
-            appearance.setMaterial((Material)DEFAULT_TEXTURED_SHAPE_MATERIAL);
+            appearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_AMBIENT_COLOR, materialShininess));
             TexCoordGeneration texCoordGeneration = new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR,
                 TexCoordGeneration.TEXTURE_COORDINATE_2,
                 new Vector4f(-pieceSize.x / modelSize.x / texture.getWidth(), 0, 0, 0), 
@@ -443,16 +482,24 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
             TextureManager.getInstance().loadTexture(texture.getImage(), waitTextureLoadingEnd,
                 new TextureManager.TextureObserver() {
                     public void textureUpdated(Texture texture) {
-                      if (texture.getFormat() == Texture.RGBA) {
+                      if (TextureManager.getInstance().isTextureTransparent(texture)) {
                         shape.getAppearance().setTransparencyAttributes(DEFAULT_TEXTURED_SHAPE_TRANSPARENCY_ATTRIBUTES);
+                        shape.getAppearance().setPolygonAttributes(DEFAULT_TEXTURED_SHAPE_POLYGON_ATTRIBUTES);
                       }
                       shape.getAppearance().setTexture(texture);
                     }
                   });
           } else {
             // Restore default material and texture
-            appearance.setMaterial(defaultMaterialAndTexture.getMaterial());
+            Material defaultMaterial = defaultMaterialAndTexture.getMaterial();
+            if (defaultMaterial != null && shininess != null) {
+              defaultMaterial = (Material)defaultMaterial.cloneNodeComponent(true);
+              defaultMaterial.setSpecularColor(new Color3f(shininess, shininess, shininess));
+              defaultMaterial.setShininess(shininess * 128);
+            }
+            appearance.setMaterial(defaultMaterial);
             appearance.setTransparencyAttributes(defaultMaterialAndTexture.getTransparencyAttributes());
+            appearance.setPolygonAttributes(defaultMaterialAndTexture.getPolygonAttributes());
             appearance.setTexCoordGeneration(defaultMaterialAndTexture.getTexCoordGeneration());
             appearance.setTexture(defaultMaterialAndTexture.getTexture());
             appearance.setTextureAttributes(defaultMaterialAndTexture.getTextureAttributes());
@@ -465,7 +512,8 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
   }
 
   /**
-   * Sets the visible attribute of all <code>Shape3D</code> children nodes of <code>node</code>. 
+   * Sets the visible attribute of the <code>Shape3D</code> children nodes of <code>node</code>.
+   * If <code>updateLightSources</code> is <code>true</code>, only light sources will updated. 
    */
   private void setVisible(Node node, boolean visible) {
     if (node instanceof Group) {
@@ -477,7 +525,8 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     } else if (node instanceof Link) {
       setVisible(((Link)node).getSharedGroup(), visible);
     } else if (node instanceof Shape3D) {
-      Appearance appearance = ((Shape3D)node).getAppearance();
+      final Shape3D shape = (Shape3D)node;
+      Appearance appearance = shape.getAppearance();
       if (appearance == null) {
         appearance = createAppearanceWithChangeCapabilities();
         ((Shape3D)node).setAppearance(appearance);
@@ -489,11 +538,36 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
         appearance.setRenderingAttributes(renderingAttributes);
       }
       
+      String shapeName = (String)shape.getUserData();
+      if (visible 
+          && shapeName != null
+          && (getUserData() instanceof Light)
+          && shapeName.startsWith(ModelManager.LIGHT_SHAPE_PREFIX)
+          && this.home != null
+          && !isSelected(this.home.getSelectedItems())) {
+        // Don't display light sources shapes of unselected lights
+        visible = false;
+      }
       // Change visibility
       renderingAttributes.setVisible(visible);
     }
   }
 
+  /**
+   * Returns <code>true</code> if this piece of furniture belongs to <code>selectedItems</code>.
+   */
+  private boolean isSelected(List<? extends Selectable> selectedItems) {
+    Object piece = getUserData();
+    for (Selectable item : selectedItems) {
+      if (item == piece
+          || (item instanceof HomeFurnitureGroup
+              && isSelected(((HomeFurnitureGroup)item).getFurniture()))) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   /**
    * Sets the cull face of all <code>Shape3D</code> children nodes of <code>node</code>.
    * @param cullFace <code>PolygonAttributes.CULL_FRONT</code> or <code>PolygonAttributes.CULL_BACK</code>
@@ -575,6 +649,10 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     // Allow future material and rendering attributes changes
     appearance.setCapability(Appearance.ALLOW_MATERIAL_READ);
     appearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
+    Material material = appearance.getMaterial();
+    if (material != null) {
+      material.setCapability(Material.ALLOW_COMPONENT_READ);
+    }
     appearance.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_READ);
     appearance.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_WRITE);
     appearance.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
@@ -614,6 +692,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
   private static class DefaultMaterialAndTexture {
     private final Material               material;
     private final TransparencyAttributes transparencyAttributes;
+    private final PolygonAttributes      polygonAttributes;
     private final TexCoordGeneration     texCoordGeneration;
     private final Texture                texture;
     private final TextureAttributes      textureAttributes;
@@ -621,9 +700,10 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     public DefaultMaterialAndTexture(Appearance appearance) {
       this.material = appearance.getMaterial();
       this.transparencyAttributes = appearance.getTransparencyAttributes();
+      this.polygonAttributes = appearance.getPolygonAttributes();
       this.texCoordGeneration = appearance.getTexCoordGeneration();
       this.texture = appearance.getTexture();
-      this.textureAttributes = appearance.getTextureAttributes();      
+      this.textureAttributes = appearance.getTextureAttributes();
     }
     
     public Material getMaterial() {
@@ -634,6 +714,10 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
       return this.transparencyAttributes;
     }
     
+    public PolygonAttributes getPolygonAttributes() {
+      return this.polygonAttributes;
+    }
+    
     public TexCoordGeneration getTexCoordGeneration() {
       return this.texCoordGeneration;
     }
