@@ -32,7 +32,8 @@ import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Modifier;
 import java.net.URL;
 import java.util.ArrayList;
-import java.util.Collections;
+import java.util.Arrays;
+import java.util.Comparator;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.List;
@@ -60,6 +61,7 @@ import javax.media.j3d.Link;
 import javax.media.j3d.Material;
 import javax.media.j3d.Node;
 import javax.media.j3d.QuadArray;
+import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.SharedGroup;
 import javax.media.j3d.Texture;
@@ -92,7 +94,8 @@ import com.sun.j3d.loaders.lw3d.Lw3dLoader;
  * This manager supports 3D models with an OBJ, 3DS or LWS format by default. 
  * Additional classes implementing Java 3D <code>Loader</code> interface may be 
  * specified in the <code>com.eteks.sweethome3d.j3d.additionalLoaderClasses</code>
- * (separated by a space or a colon :) to enable the support of other formats. 
+ * (separated by a space or a colon :) to enable the support of other formats.<br> 
+ * Note: this class is compatible with Java 3D 1.3.
  * @author Emmanuel Puybaret
  */
 public class ModelManager {
@@ -104,7 +107,11 @@ public class ModelManager {
    * <code>Shape3D</code> user data prefix for mirror shapes. 
    */
   public static final String MIRROR_SHAPE_PREFIX = "sweethome3d_window_mirror";
-
+  /**
+   * <code>Shape3D</code> user data prefix for lights. 
+   */
+  public static final String LIGHT_SHAPE_PREFIX = "sweethome3d_light";
+  
   private static final TransparencyAttributes WINDOW_PANE_TRANSPARENCY_ATTRIBUTES = 
       new TransparencyAttributes(TransparencyAttributes.NICEST, 0.5f);
 
@@ -127,7 +134,7 @@ public class ModelManager {
   
   private ModelManager() {    
     // This class is a singleton
-    this.loadedModelNodes = Collections.synchronizedMap(new WeakHashMap<Content, BranchGroup>());
+    this.loadedModelNodes = new WeakHashMap<Content, BranchGroup>();
     this.loadingModelObservers = new HashMap<Content, List<ModelObserver>>();
     // Load other optional Loader classes 
     List<Class<Loader>> loaderClasses = new ArrayList<Class<Loader>>();
@@ -191,7 +198,9 @@ public class ModelManager {
       this.modelsLoader.shutdownNow();
       this.modelsLoader = null;
     }
-    this.loadedModelNodes.clear();
+    synchronized (this.loadedModelNodes) {
+      this.loadedModelNodes.clear();
+    }
   }
   
   /**
@@ -322,15 +331,20 @@ public class ModelManager {
   public void loadModel(final Content content,
                         boolean synchronous,
                         ModelObserver modelObserver) {
-    BranchGroup modelRoot = this.loadedModelNodes.get(content);
+    BranchGroup modelRoot;
+    synchronized (this.loadedModelNodes) {
+      modelRoot = this.loadedModelNodes.get(content);
+    }
     if (modelRoot != null) {
       // Notify cached model to observer with a clone of the model
       modelObserver.modelUpdated((BranchGroup)cloneNode(modelRoot));
     } else if (synchronous) {
       try {
         modelRoot = loadModel(content);
-        // Store in cache model node for future copies 
-        this.loadedModelNodes.put(content, (BranchGroup)modelRoot);
+        synchronized (this.loadedModelNodes) {
+          // Store in cache model node for future copies 
+          this.loadedModelNodes.put(content, (BranchGroup)modelRoot);
+        }
         modelObserver.modelUpdated((BranchGroup)cloneNode(modelRoot));
       } catch (IOException ex) {
         modelObserver.modelError(ex);
@@ -357,8 +371,10 @@ public class ModelManager {
           public void run() {
             try {
               final BranchGroup loadedModel = loadModel(content);
-              // Update loaded models cache and notify registered observers
-              loadedModelNodes.put(content, loadedModel);
+              synchronized (loadedModelNodes) {
+                // Update loaded models cache and notify registered observers
+                loadedModelNodes.put(content, loadedModel);
+              }
               EventQueue.invokeLater(new Runnable() {
                   public void run() {
                     for (final ModelObserver observer : loadingModelObservers.remove(content)) {
@@ -387,7 +403,10 @@ public class ModelManager {
    * and the texture images of shapes.
    */
   public Node cloneNode(Node node) {
-    return cloneNode(node, new HashMap<SharedGroup, SharedGroup>());
+    // Clone node in a synchronized block because cloneNodeComponent is not thread safe
+    synchronized (this.loadedModelNodes) {  
+      return cloneNode(node, new HashMap<SharedGroup, SharedGroup>());
+    }
   }
     
   private Node cloneNode(Node node, Map<SharedGroup, SharedGroup> clonedSharedGroups) {
@@ -397,7 +416,7 @@ public class ModelManager {
       Appearance appearance = shape.getAppearance();
       if (appearance != null) {
         // Force only duplication of node's appearance except its texture
-        Appearance clonedAppearance = (Appearance)appearance.cloneNodeComponent(true);
+        Appearance clonedAppearance = (Appearance)appearance.cloneNodeComponent(true);        
         Texture texture = appearance.getTexture();
         if (texture != null) {
           clonedAppearance.setTexture(texture);
@@ -561,14 +580,16 @@ public class ModelManager {
   /**
    * Updates the name of scene shapes and transparency window panes shapes.
    */
+  @SuppressWarnings("unchecked")
   private void updateShapeNamesAndWindowPanesTransparency(Scene scene) {
     Map<String, Object> namedObjects = scene.getNamedObjects();
     for (Map.Entry<String, Object> entry : namedObjects.entrySet()) {
       if (entry.getValue() instanceof Shape3D) {
+        String shapeName = entry.getKey();
         // Assign shape name to its user data
         Shape3D shape = (Shape3D)entry.getValue();
-        shape.setUserData(entry.getKey());
-        if (entry.getKey().startsWith(WINDOW_PANE_SHAPE_PREFIX)) {
+        shape.setUserData(shapeName);
+        if (shapeName.startsWith(WINDOW_PANE_SHAPE_PREFIX)) {
           Appearance appearance = shape.getAppearance();
           if (appearance == null) {
             appearance = new Appearance();
@@ -627,7 +648,7 @@ public class ModelManager {
           }
           
           // If texture image supports transparency
-          if (texture.getFormat() == Texture.RGBA) {
+          if (TextureManager.getInstance().isTextureTransparent(sharedTexture)) {
             if (appearance.getTransparencyAttributes() == null) {
               // Add transparency attributes to ensure transparency works
               appearance.setTransparencyAttributes(
@@ -644,13 +665,133 @@ public class ModelManager {
    * projected on the floor (plan y = 0). 
    */
   public Area getAreaOnFloor(Node node) {
-    Area modelAreaOnFloor = new Area();
-    computeAreaOnFloor(node, modelAreaOnFloor, new Transform3D());
+    Area modelAreaOnFloor;
+    int vertexCount = getVertexCount(node);
+    if (vertexCount < 10000) {
+      modelAreaOnFloor = new Area();
+      computeAreaOnFloor(node, modelAreaOnFloor, new Transform3D());
+    } else {
+      List<float []> vertices = new ArrayList<float[]>(vertexCount); 
+      computeVerticesOnFloor(node, vertices, new Transform3D());
+      float [][] surroundingPolygon = getSurroundingPolygon(vertices.toArray(new float [vertices.size()][]));
+      GeneralPath generalPath = new GeneralPath(GeneralPath.WIND_NON_ZERO, surroundingPolygon.length);
+      generalPath.moveTo(surroundingPolygon [0][0], surroundingPolygon [0][1]);
+      for (int i = 0; i < surroundingPolygon.length; i++) {
+        generalPath.lineTo(surroundingPolygon [i][0], surroundingPolygon [i][1]);
+      }
+      generalPath.closePath();
+      modelAreaOnFloor = new Area(generalPath);
+    }
     return modelAreaOnFloor;
   }
   
   /**
-   * Computes the 2D area on floor of a the 3D shapes children of <code>node</code>.
+   * Returns the total count of vertices in all geometries.
+   */
+  private int getVertexCount(Node node) {
+    int count = 0;
+    if (node instanceof Group) {
+      // Enumerate all children
+      Enumeration<?> enumeration = ((Group)node).getAllChildren();
+      while (enumeration.hasMoreElements()) {
+        count += getVertexCount((Node)enumeration.nextElement());
+      }
+    } else if (node instanceof Link) {
+      count = getVertexCount(((Link)node).getSharedGroup());
+    } else if (node instanceof Shape3D) {
+      Shape3D shape = (Shape3D)node;
+      Appearance appearance = shape.getAppearance();
+      RenderingAttributes renderingAttributes = appearance != null 
+          ? appearance.getRenderingAttributes() : null;
+      if (renderingAttributes == null
+          || renderingAttributes.getVisible()) {
+        for (int i = 0, n = shape.numGeometries(); i < n; i++) {
+          Geometry geometry = shape.getGeometry(i);
+          if (geometry instanceof GeometryArray) {
+            count += ((GeometryArray)geometry).getVertexCount();
+          }
+        }
+      }
+    }    
+    return count;
+  }
+  
+  /**
+   * Computes the vertices coordinates projected on floor of the 3D shapes children of <code>node</code>.
+   */
+  private void computeVerticesOnFloor(Node node, List<float []> vertices, Transform3D parentTransformations) {
+    if (node instanceof Group) {
+      if (node instanceof TransformGroup) {
+        parentTransformations = new Transform3D(parentTransformations);
+        Transform3D transform = new Transform3D();
+        ((TransformGroup)node).getTransform(transform);
+        parentTransformations.mul(transform);
+      }
+      // Compute all children
+      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
+      while (enumeration.hasMoreElements()) {
+        computeVerticesOnFloor((Node)enumeration.nextElement(), vertices, parentTransformations);
+      }
+    } else if (node instanceof Link) {
+      computeVerticesOnFloor(((Link)node).getSharedGroup(), vertices, parentTransformations);
+    } else if (node instanceof Shape3D) {
+      Shape3D shape = (Shape3D)node;
+      Appearance appearance = shape.getAppearance();
+      RenderingAttributes renderingAttributes = appearance != null 
+          ? appearance.getRenderingAttributes() : null;
+      TransparencyAttributes transparencyAttributes = appearance != null 
+          ? appearance.getTransparencyAttributes() : null;
+      if ((renderingAttributes == null
+            || renderingAttributes.getVisible())
+          && (transparencyAttributes == null
+              || transparencyAttributes.getTransparency() < 1)) {
+        // Compute shape geometries area
+        for (int i = 0, n = shape.numGeometries(); i < n; i++) {
+          Geometry geometry = shape.getGeometry(i);
+          if (geometry instanceof GeometryArray) {
+            GeometryArray geometryArray = (GeometryArray)geometry;      
+
+            int vertexCount = geometryArray.getVertexCount();
+            Point3f vertex = new Point3f();
+            if ((geometryArray.getVertexFormat() & GeometryArray.BY_REFERENCE) != 0) {
+              if ((geometryArray.getVertexFormat() & GeometryArray.INTERLEAVED) != 0) {
+                float [] vertexData = geometryArray.getInterleavedVertices();
+                int vertexSize = vertexData.length / vertexCount;
+                // Store vertices coordinates 
+                for (int index = 0, j = vertexSize - 3; index < vertexCount; j += vertexSize, index++) {
+                  vertex.x = vertexData [j];
+                  vertex.y = vertexData [j + 1];
+                  vertex.z = vertexData [j + 2];
+                  parentTransformations.transform(vertex);
+                  vertices.add(new float [] {vertex.x, vertex.z});
+                }
+              } else {
+                // Store vertices coordinates
+                float [] vertexCoordinates = geometryArray.getCoordRefFloat();
+                for (int index = 0, j = 0; index < vertexCount; j += 3, index++) {
+                  vertex.x = vertexCoordinates [j];
+                  vertex.y = vertexCoordinates [j + 1];
+                  vertex.z = vertexCoordinates [j + 2];
+                  parentTransformations.transform(vertex);
+                  vertices.add(new float [] {vertex.x, vertex.z});
+                }
+              }
+            } else {
+              // Store vertices coordinates
+              for (int index = 0, j = 0; index < vertexCount; j++, index++) {
+                geometryArray.getCoordinate(j, vertex);
+                parentTransformations.transform(vertex);
+                vertices.add(new float [] {vertex.x, vertex.z});
+              }
+            }
+          }
+        }
+      }
+    }    
+  }
+  
+  /**
+   * Computes the 2D area on floor of the 3D shapes children of <code>node</code>.
    */
   private void computeAreaOnFloor(Node node, Area nodeArea, Transform3D parentTransformations) {
     if (node instanceof Group) {
@@ -669,9 +810,19 @@ public class ModelManager {
       computeAreaOnFloor(((Link)node).getSharedGroup(), nodeArea, parentTransformations);
     } else if (node instanceof Shape3D) {
       Shape3D shape = (Shape3D)node;
-      // Compute shape geometries area
-      for (int i = 0, n = shape.numGeometries(); i < n; i++) {
-        computeGeometryAreaOnFloor(shape.getGeometry(i), parentTransformations, nodeArea);
+      Appearance appearance = shape.getAppearance();
+      RenderingAttributes renderingAttributes = appearance != null 
+          ? appearance.getRenderingAttributes() : null;
+      TransparencyAttributes transparencyAttributes = appearance != null 
+          ? appearance.getTransparencyAttributes() : null;
+      if ((renderingAttributes == null
+            || renderingAttributes.getVisible())
+          && (transparencyAttributes == null
+              || transparencyAttributes.getTransparency() < 1)) {
+        // Compute shape geometries area
+        for (int i = 0, n = shape.numGeometries(); i < n; i++) {
+          computeGeometryAreaOnFloor(shape.getGeometry(i), parentTransformations, nodeArea);
+        }
       }
     }    
   }
@@ -907,6 +1058,125 @@ public class ModelManager {
     }
   }
 
+  /**
+   * Returns the convex polygon that surrounds the given <code>vertices</code>.
+   * From Andrew's monotone chain 2D convex hull algorithm described at
+   * http://softsurfer.com/Archive/algorithm%5F0109/algorithm%5F0109.htm
+   */
+  private float [][] getSurroundingPolygon(float [][] vertices) {
+    Arrays.sort(vertices, new Comparator<float []> () {
+        public int compare(float [] vertex1, float [] vertex2) {
+          if (vertex1 [0] == vertex2 [0]) {
+            return (int)Math.signum(vertex2 [1] - vertex1 [1]);
+          } else {
+            return (int)Math.signum(vertex2 [0] - vertex1 [0]);
+          }
+        }
+      });
+    float [][] polygon = new float [vertices.length][];
+    // The output array polygon [] will be used as the stack
+    int bottom = 0, top = -1; // indices for bottom and top of the stack
+    int i; // array scan index
+
+    // Get the indices of points with min x-coord and min|max y-coord
+    int minMin = 0, minMax;
+    float xmin = vertices [0][0];
+    for (i = 1; i < vertices.length; i++) {
+      if (vertices [i][0] != xmin) {
+        break;
+      }
+    }
+    minMax = i - 1;
+    if (minMax == vertices.length - 1) { 
+      // Degenerate case: all x-coords == xmin
+      polygon [++top] = vertices [minMin];
+      if (vertices [minMax][1] != vertices [minMin][1]) { 
+        // A nontrivial segment
+        polygon [++top] = vertices [minMax];
+      }
+      // Add polygon end point
+      polygon [++top] = vertices [minMin];
+      float [][] surroundingPolygon = new float [top + 1][];
+      System.arraycopy(polygon, 0, surroundingPolygon, 0, surroundingPolygon.length);
+    }
+
+    // Get the indices of points with max x-coord and min|max y-coord
+    int maxMin, maxMax = vertices.length - 1;
+    float xMax = vertices [vertices.length - 1][0];
+    for (i = vertices.length - 2; i >= 0; i--) {
+      if (vertices [i][0] != xMax) {
+        break;
+      }
+    }
+    maxMin = i + 1;
+
+    // Compute the lower hull on the stack polygon
+    polygon [++top] = vertices [minMin]; // push minmin point onto stack
+    i = minMax;
+    while (++i <= maxMin) {
+      // The lower line joins points [minmin] with points [maxmin]
+      if (isLeft(vertices [minMin], vertices [maxMin], vertices [i]) >= 0 && i < maxMin) {
+        // ignore points [i] above or on the lower line
+        continue; 
+      }
+
+      while (top > 0) // There are at least 2 points on the stack
+      {
+        // Test if points [i] is left of the line at the stack top
+        if (isLeft(polygon [top - 1], polygon [top], vertices [i]) > 0)
+          break; // points [i] is a new hull vertex
+        else
+          top--; // pop top point off stack
+      }
+      polygon [++top] = vertices [i]; // push points [i] onto stack
+    }
+
+    // Next, compute the upper hull on the stack polygon above the bottom hull
+    // If distinct xmax points
+    if (maxMax != maxMin) { 
+      // Push maxmax point onto stack
+      polygon [++top] = vertices [maxMax]; 
+    }
+    // The bottom point of the upper hull stack
+    bottom = top; 
+    i = maxMin;
+    while (--i >= minMax) {
+      // The upper line joins points [maxmax] with points [minmax]
+      if (isLeft(vertices [maxMax], vertices [minMax], vertices [i]) >= 0 && i > minMax) {
+        // Ignore points [i] below or on the upper line
+        continue; 
+      }
+
+      // At least 2 points on the upper stack
+      while (top > bottom) 
+      {
+        // Test if points [i] is left of the line at the stack top
+        if (isLeft(polygon [top - 1], polygon [top], vertices [i]) > 0) {
+          // points [i] is a new hull vertex
+          break; 
+        } else {
+          // Pop top point off stack
+          top--; 
+        }
+      }
+      // Push points [i] onto stack
+      polygon [++top] = vertices [i]; 
+    }
+    if (minMax != minMin) {
+      // Push joining endpoint onto stack
+      polygon [++top] = vertices [minMin]; 
+    }
+
+    float [][] surroundingPolygon = new float [top + 1][];
+    System.arraycopy(polygon, 0, surroundingPolygon, 0, surroundingPolygon.length);
+    return surroundingPolygon;
+  }
+
+  private float isLeft(float [] vertex0, float [] vertex1, float [] vertex2) {
+    return (vertex1 [0] - vertex0 [0]) * (vertex2 [1] - vertex0 [1]) 
+         - (vertex2 [0] - vertex0 [0]) * (vertex1 [1] - vertex0 [1]);
+  }
+
   /**
    * An observer that receives model loading notifications. 
    */
