@@ -66,6 +66,7 @@ import javax.media.j3d.Material;
 import javax.media.j3d.Node;
 import javax.media.j3d.PolygonAttributes;
 import javax.media.j3d.QuadArray;
+import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.TexCoordGeneration;
 import javax.media.j3d.Texture;
@@ -85,7 +86,8 @@ import javax.vecmath.Vector4f;
  * An output stream that writes Java 3D nodes at OBJ + MTL format.
  * <p>Once you wrote nodes, call <code>close</code> method to create the MTL file
  * and the texture images in the same directory as OBJ file. This feature applies
- * only to constructor that takes a file as parameter.
+ * only to constructor that takes a file as parameter.<br>
+ * Note: this class is compatible with Java 3D 1.3.
  * @author Emmanuel Puybaret
  */
 public class OBJWriter extends FilterWriter {
@@ -334,69 +336,73 @@ public class OBJWriter extends FilterWriter {
     } else if (node instanceof Shape3D) {
       Shape3D shape = (Shape3D)node;
       Appearance appearance = shape.getAppearance();
-      
-      // Build a unique human readable object name
-      String objectName = "";
-      if (accept(nodeName)) {
-        objectName = nodeName + "_";
-      }
+      RenderingAttributes renderingAttributes = appearance != null 
+          ? appearance.getRenderingAttributes() : null;
+      if (renderingAttributes == null
+          || renderingAttributes.getVisible()) {
+        // Build a unique human readable object name
+        String objectName = "";
+        if (accept(nodeName)) {
+          objectName = nodeName + "_";
+        }
+          
+        String shapeName = null;
+        if (shape.getUserData() instanceof String) {
+          shapeName = (String)shape.getUserData(); 
+        }
+        if (accept(shapeName)) {
+          objectName += shapeName + "_";
+        } 
         
-      String shapeName = null;
-      if (shape.getUserData() instanceof String) {
-        shapeName = (String)shape.getUserData(); 
-      }
-      if (accept(shapeName)) {
-        objectName += shapeName + "_";
-      } 
-      
-      objectName += String.valueOf(this.vertexOffset);
-      
-      // Start a new object at OBJ format 
-      this.out.write("g " + objectName + "\n");
-      
-      TexCoordGeneration texCoordGeneration = null;
-      if (this.mtlFileName != null) {
-        if (appearance != null) {
-          texCoordGeneration = appearance.getTexCoordGeneration();
-          ComparableAppearance comparableAppearance = new ComparableAppearance(appearance);
-          String appearanceName = this.appearances.get(comparableAppearance);
-          if (appearanceName == null) {
-            // Store appearance
-            appearanceName = objectName;
-            this.appearances.put(comparableAppearance, appearanceName);
-            
-            Texture texture = appearance.getTexture();
-            if (texture != null) {
-              File textureFile = this.textures.get(texture);
-              if (textureFile == null) {
-                // Store texture
-                String fileFormat = texture.getFormat() == Texture.RGBA 
-                    ? "png"
-                    : "jpg";
-                textureFile = new File(this.mtlFileName.substring(0, this.mtlFileName.length() - 4) 
-                    + "_" + appearanceName + "." + fileFormat);
-                this.textures.put(texture, textureFile);
+        objectName += String.valueOf(this.vertexOffset);
+        
+        // Start a new object at OBJ format 
+        this.out.write("g " + objectName + "\n");
+        
+        TexCoordGeneration texCoordGeneration = null;
+        if (this.mtlFileName != null) {
+          if (appearance != null) {
+            texCoordGeneration = appearance.getTexCoordGeneration();
+            ComparableAppearance comparableAppearance = new ComparableAppearance(appearance);
+            String appearanceName = this.appearances.get(comparableAppearance);
+            if (appearanceName == null) {
+              // Store appearance
+              appearanceName = objectName;
+              this.appearances.put(comparableAppearance, appearanceName);
+              
+              Texture texture = appearance.getTexture();
+              if (texture != null) {
+                File textureFile = this.textures.get(texture);
+                if (textureFile == null) {
+                  // Store texture
+                  String fileFormat = texture.getFormat() == Texture.RGBA 
+                      ? "png"
+                      : "jpg";
+                  textureFile = new File(this.mtlFileName.substring(0, this.mtlFileName.length() - 4) 
+                      + "_" + appearanceName + "." + fileFormat);
+                  this.textures.put(texture, textureFile);
+                }
               }
-            }
-          } 
-          this.out.write("usemtl " + appearanceName + "\n");
+            } 
+            this.out.write("usemtl " + appearanceName + "\n");
+          }
         }
-      }
-      
-      int cullFace = PolygonAttributes.CULL_BACK;
-      boolean backFaceNormalFlip = false;
-      if (appearance != null) {
-        PolygonAttributes polygonAttributes = appearance.getPolygonAttributes();
-        if (polygonAttributes != null) {
-          cullFace = polygonAttributes.getCullFace();
-          backFaceNormalFlip = polygonAttributes.getBackFaceNormalFlip();
+        
+        int cullFace = PolygonAttributes.CULL_BACK;
+        boolean backFaceNormalFlip = false;
+        if (appearance != null) {
+          PolygonAttributes polygonAttributes = appearance.getPolygonAttributes();
+          if (polygonAttributes != null) {
+            cullFace = polygonAttributes.getCullFace();
+            backFaceNormalFlip = polygonAttributes.getBackFaceNormalFlip();
+          }
+        }
+        
+        // Write object geometries
+        for (int i = 0, n = shape.numGeometries(); i < n; i++) {
+          writeNodeGeometry(shape.getGeometry(i), parentTransformations, texCoordGeneration, 
+              cullFace, backFaceNormalFlip);
         }
-      }
-      
-      // Write object geometries
-      for (int i = 0, n = shape.numGeometries(); i < n; i++) {
-        writeNodeGeometry(shape.getGeometry(i), parentTransformations, texCoordGeneration, 
-            cullFace, backFaceNormalFlip);
       }
     }    
   }
@@ -460,7 +466,11 @@ public class OBJWriter extends FilterWriter {
       Vector4f planeT = null;
       if (texCoordGeneration != null) {
         textureCoordinatesGenerated = texCoordGeneration.getGenMode() == TexCoordGeneration.OBJECT_LINEAR
-            && texCoordGeneration.getEnable();
+            && texCoordGeneration.getEnable()
+            && !(geometryArray instanceof IndexedLineArray)
+            && !(geometryArray instanceof IndexedLineStripArray)
+            && !(geometryArray instanceof LineArray)
+            && !(geometryArray instanceof LineStripArray);
         if (textureCoordinatesGenerated) {
           planeS = new Vector4f();
           planeT = new Vector4f();
@@ -591,8 +601,7 @@ public class OBJWriter extends FilterWriter {
         if (geometryArray instanceof IndexedLineArray) {
           IndexedLineArray lineArray = (IndexedLineArray)geometryArray;
           for (int i = 0, n = lineArray.getIndexCount(); i < n; i += 2) {
-            writeIndexedLine(lineArray, i, i + 1,  
-                vertexIndexSubstitutes, textureCoordinatesIndexSubstitutes, textureCoordinatesGenerated);
+            writeIndexedLine(lineArray, i, i + 1, vertexIndexSubstitutes, textureCoordinatesIndexSubstitutes);
           }
         } else if (geometryArray instanceof IndexedTriangleArray) {
           IndexedTriangleArray triangleArray = (IndexedTriangleArray)geometryArray;
@@ -618,7 +627,7 @@ public class OBJWriter extends FilterWriter {
             for (int strip = 0; strip < stripIndexCounts.length; strip++) {
               for (int i = initialIndex, n = initialIndex + stripIndexCounts [strip] - 1; i < n; i++) {
                 writeIndexedLine(geometryStripArray, i, i + 1, 
-                    vertexIndexSubstitutes, textureCoordinatesIndexSubstitutes, textureCoordinatesGenerated);
+                    vertexIndexSubstitutes, textureCoordinatesIndexSubstitutes);
               }
               initialIndex += stripIndexCounts [strip];
             }
@@ -652,8 +661,7 @@ public class OBJWriter extends FilterWriter {
         if (geometryArray instanceof LineArray) {
           LineArray lineArray = (LineArray)geometryArray;
           for (int i = 0, n = lineArray.getVertexCount(); i < n; i += 2) {
-            writeLine(lineArray, i, i + 1, vertexIndexSubstitutes,    
-                textureCoordinatesIndexSubstitutes, textureCoordinatesGenerated);
+            writeLine(lineArray, i, i + 1, vertexIndexSubstitutes, textureCoordinatesIndexSubstitutes);
           }
         } else if (geometryArray instanceof TriangleArray) {
           TriangleArray triangleArray = (TriangleArray)geometryArray;
@@ -678,8 +686,7 @@ public class OBJWriter extends FilterWriter {
           if (geometryStripArray instanceof LineStripArray) {
             for (int strip = 0; strip < stripVertexCounts.length; strip++) {
               for (int i = initialIndex, n = initialIndex + stripVertexCounts [strip] - 1; i < n; i++) {
-                writeLine(geometryStripArray, i, i + 1, vertexIndexSubstitutes,   
-                    textureCoordinatesIndexSubstitutes, textureCoordinatesGenerated);
+                writeLine(geometryStripArray, i, i + 1, vertexIndexSubstitutes, textureCoordinatesIndexSubstitutes);
               }
               initialIndex += stripVertexCounts [strip];
             }
@@ -779,7 +786,6 @@ public class OBJWriter extends FilterWriter {
   /**
    * Applies to <code>normal</code> the given transformation, and appends to <code>normalsBuffer</code> 
    * its values in a line vn at OBJ format, if the normal isn't a key of <code>normalIndices</code> yet.  
-   * @param normalsBuffer TODO
    * @return <code>true</code> if the written normal doens't contain any NaN value 
    */
   private boolean writeNormal(StringBuilder normalsBuffer,
@@ -848,10 +854,8 @@ public class OBJWriter extends FilterWriter {
   private void writeIndexedLine(IndexedGeometryArray geometryArray, 
                                 int vertexIndex1, int vertexIndex2, 
                                 int [] vertexIndexSubstitutes, 
-                                int [] textureCoordinatesIndexSubstitutes,
-                                boolean textureCoordinatesGenerated) throws IOException {
-    if (textureCoordinatesGenerated
-        || (geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
+                                int [] textureCoordinatesIndexSubstitutes) throws IOException {
+    if ((geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
       this.out.write("l " + (this.vertexOffset + vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex1)]) 
           + "/" + (this.textureCoordinatesOffset + textureCoordinatesIndexSubstitutes [geometryArray.getTextureCoordinateIndex(0, vertexIndex1)]) 
           + " " + (this.vertexOffset + vertexIndexSubstitutes [geometryArray.getCoordinateIndex(vertexIndex2)]) 
@@ -865,7 +869,6 @@ public class OBJWriter extends FilterWriter {
   /**
    * Writes the triangle indices given at vertexIndex1, vertexIndex2, vertexIndex3, 
    * in a line f at OBJ format. 
-   * @param normalsDefined TODO
    */
   private void writeIndexedTriangle(IndexedGeometryArray geometryArray, 
                                     int vertexIndex1, int vertexIndex2, int vertexIndex3, 
@@ -928,7 +931,6 @@ public class OBJWriter extends FilterWriter {
   /**
    * Writes the quadrilateral indices given at vertexIndex1, vertexIndex2, vertexIndex3, vertexIndex4,
    * in a line f at OBJ format. 
-   * @param normalsDefined TODO
    */
   private void writeIndexedQuadrilateral(IndexedGeometryArray geometryArray, 
                                          int vertexIndex1, int vertexIndex2, int vertexIndex3, int vertexIndex4, 
@@ -1006,10 +1008,8 @@ public class OBJWriter extends FilterWriter {
   private void writeLine(GeometryArray geometryArray, 
                          int vertexIndex1, int vertexIndex2, 
                          int [] vertexIndexSubstitutes,  
-                         int [] textureCoordinatesIndexSubstitutes,
-                         boolean textureCoordinatesGenerated) throws IOException {
-    if (textureCoordinatesGenerated
-        || (geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
+                         int [] textureCoordinatesIndexSubstitutes) throws IOException {
+    if ((geometryArray.getVertexFormat() & GeometryArray.TEXTURE_COORDINATE_2) != 0) {
       this.out.write("l " + (this.vertexOffset + vertexIndexSubstitutes [vertexIndex1]) 
           + "/" + (this.textureCoordinatesOffset + textureCoordinatesIndexSubstitutes [vertexIndex1]) 
           + " " + (this.vertexOffset + vertexIndexSubstitutes [vertexIndex2]) 
@@ -1023,7 +1023,6 @@ public class OBJWriter extends FilterWriter {
   /**
    * Writes the triangle indices given at vertexIndex1, vertexIndex2, vertexIndex3, 
    * in a line f at OBJ format. 
-   * @param normalsDefined TODO
    */
   private void writeTriangle(GeometryArray geometryArray, 
                              int vertexIndex1, int vertexIndex2, int vertexIndex3, 
@@ -1086,7 +1085,6 @@ public class OBJWriter extends FilterWriter {
   /**
    * Writes the quadrilateral indices given at vertexIndex1, vertexIndex2, vertexIndex3, vertexIndex4,
    * in a line f at OBJ format. 
-   * @param normalsDefined TODO
    */
   private void writeQuadrilateral(GeometryArray geometryArray, 
                                   int vertexIndex1, int vertexIndex2, int vertexIndex3, int vertexIndex4, 
