@@ -29,10 +29,12 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.UUID;
@@ -79,7 +81,10 @@ import org.sunflow.PluginRegistry;
 import org.sunflow.SunflowAPI;
 import org.sunflow.core.Display;
 import org.sunflow.core.Instance;
+import org.sunflow.core.ParameterList;
+import org.sunflow.core.ParameterList.InterpolationType;
 import org.sunflow.core.light.SphereLight;
+import org.sunflow.core.light.SunSkyLight;
 import org.sunflow.image.Color;
 import org.sunflow.math.Matrix4;
 import org.sunflow.math.Point3;
@@ -88,9 +93,13 @@ import org.sunflow.system.UI;
 import org.sunflow.system.ui.SilentInterface;
 
 import com.eteks.sweethome3d.model.Camera;
+import com.eteks.sweethome3d.model.Compass;
 import com.eteks.sweethome3d.model.Home;
+import com.eteks.sweethome3d.model.HomeFurnitureGroup;
+import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.LightSource;
 import com.eteks.sweethome3d.model.ObserverCamera;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Wall;
@@ -99,14 +108,20 @@ import com.eteks.sweethome3d.tools.OperatingSystem;
 /**
  * A renderer able to create a photo realistic image of a home.
  * @author Emmanuel Puybaret
+ * @author Frdric Mantegazza (Sun location algorithm)
  */
 public class PhotoRenderer {
   public enum Quality {LOW, HIGH}
   
-  private final static String CAMERA_NAME = "camera";
-  
   private final Quality quality;
+  private final Compass compass;
+  private final int homeLightColor;
+  
   private final SunflowAPI sunflow;
+  private boolean useSunSky;
+  private boolean useAmbientOcclusion;
+  private String sunSkyLightName;
+  private String sunLightName;
   private final Map<Texture, String> textureImagesCache = new HashMap<Texture, String>();
   private Thread renderingThread;
 
@@ -121,22 +136,27 @@ public class PhotoRenderer {
    * @throws IOException if texture image files required in the scene couldn't be created. 
    */
   public PhotoRenderer(Home home, Quality quality) throws IOException {
-    this.sunflow = new SunflowAPI();
+    this.compass = home.getCompass();
     this.quality = quality;
-    int samples = quality == Quality.LOW ? 4 : 8;
+    this.sunflow = new SunflowAPI();
+    
+    this.useAmbientOcclusion = home.getCamera() instanceof ObserverCamera;
+    // SunFlow produce too much white spots when silk shader is used with sun sky
+    // so use this shader only when observer is used 
+    boolean silk = this.useAmbientOcclusion && quality == Quality.HIGH;
     
     // Export to SunFlow the Java 3D shapes and appearance of the ground, the walls, the furniture and the rooms           
     for (Wall wall : home.getWalls()) {
       Wall3D wall3D = new Wall3D(wall, home, true, true);
-      exportNode(wall3D, false);
+      exportNode(wall3D, true, false, silk);
     }
     for (HomePieceOfFurniture piece : home.getFurniture()) {
       HomePieceOfFurniture3D piece3D = new HomePieceOfFurniture3D(piece, home, true, true);
-      exportNode(piece3D, false);
+      exportNode(piece3D, false, false, silk);
     }
     for (Room room : home.getRooms()) {
       Room3D room3D = new Room3D(room, home, home.getCamera() == home.getTopCamera(), true, true);
-      exportNode(room3D, false);
+      exportNode(room3D, true, false, silk);
     } 
     // Create a dummy home to export a ground 3D not cut by rooms and large enough to join the sky at the horizon  
     Home groundHome = new Home();
@@ -147,13 +167,11 @@ public class PhotoRenderer {
     translation.setTranslation(new Vector3f(0, -0.1f, 0));
     TransformGroup groundTransformGroup = new TransformGroup(translation);
     groundTransformGroup.addChild(ground);
-    exportNode(groundTransformGroup, true);
+    exportNode(groundTransformGroup, true, true, silk);
 
-    // Set light settings 
-    boolean observerCamera = home.getCamera() instanceof ObserverCamera;
     HomeTexture skyTexture = home.getEnvironment().getSkyTexture();
-    if (observerCamera 
-        && skyTexture != null) {
+    this.useSunSky = skyTexture == null || !(home.getCamera() instanceof ObserverCamera);
+    if (!this.useSunSky) {
       // If observer camera is used with a sky texture, 
       // create an image base light from sky texture  
       InputStream skyImageStream = skyTexture.getImage().openStream();
@@ -172,20 +190,15 @@ public class PhotoRenderer {
       this.sunflow.parameter("center", new Vector3(-1, 0, 0));
       this.sunflow.parameter("up", new Vector3(0, 1, 0));
       this.sunflow.parameter("fixed", true);
-      this.sunflow.parameter("samples", samples);
+      this.sunflow.parameter("samples", 0); 
       this.sunflow.light(UUID.randomUUID().toString(), "ibl");
-    } else {
-      // Use sun sky light
-      this.sunflow.parameter("up", new Vector3(0, 1, 0));
-      this.sunflow.parameter("east", new Vector3(0, 0, 1));
-      this.sunflow.parameter("sundir", new Vector3(1, 1, 1));
-      this.sunflow.parameter("turbidity", 6f);
-      this.sunflow.parameter("samples", samples * 3 / 2);
-      this.sunflow.light(UUID.randomUUID().toString(), "sunsky");
-    }
+    } 
     
-    // Add lights at the top of each room when observer camera is used
-    if (observerCamera) {
+    // Set light settings 
+    int ceillingLightColor = home.getEnvironment().getCeillingLightColor();
+    this.homeLightColor = home.getEnvironment().getLightColor();
+    if (ceillingLightColor > 0) {
+      // Add lights at the top of each room 
       for (Room room : home.getRooms()) {
         if (room.isCeilingVisible()) {
           float xCenter = room.getXCenter();
@@ -216,19 +229,58 @@ public class PhotoRenderer {
           }
           
           float power = (float)Math.sqrt(room.getArea()) / 3;
-          int lightColor = home.getEnvironment().getLightColor();
           this.sunflow.parameter("radiance", null, 
-              (lightColor >> 16) * power / 255, ((lightColor >> 8) & 0xFF) * power / 255, (lightColor & 0xFF) * power / 255);
+              power * (ceillingLightColor >> 16) / 0xD0 * (this.homeLightColor >> 16) / 255, 
+              power * ((ceillingLightColor >> 8) & 0xFF) / 0xD0 * ((this.homeLightColor >> 8) & 0xFF) / 255, 
+              power * (ceillingLightColor & 0xFF) / 0xD0 * (this.homeLightColor & 0xFF) / 255);
           this.sunflow.parameter("center", new Point3(xCenter, roomHeight - 25, yCenter));                    
           this.sunflow.parameter("radius", 20f);
-          this.sunflow.parameter("samples", samples);
+          this.sunflow.parameter("samples", 4);
           this.sunflow.light(UUID.randomUUID().toString(), "sphere");
         } 
       }
     }
 
-    // Create pinhole camera at default location 
-    this.sunflow.camera(CAMERA_NAME, "pinhole");
+    // Add visible and turned on lights
+    for (HomeLight light : getLights(home.getFurniture())) {
+      float lightPower = light.getPower();
+      if (light.isVisible()
+          && lightPower > 0f) {
+        float angle = light.getAngle();
+        float cos = (float)Math.cos(angle);
+        float sin = (float)Math.sin(angle);
+        for (LightSource lightSource : ((HomeLight)light).getLightSources()) {
+          float lightRadius = lightSource.getDiameter() != null 
+                  ? lightSource.getDiameter() * light.getWidth() / 2 
+                  : 3.25f; // Default radius compatible with most lights available before version 3.0
+          float power = 5 * lightPower * lightPower / (lightRadius * lightRadius);
+          int lightColor = lightSource.getColor();
+          this.sunflow.parameter("radiance", null,
+              power * (lightColor >> 16) * (this.homeLightColor >> 16),
+              power * ((lightColor >> 8) & 0xFF) * ((this.homeLightColor >> 8) & 0xFF),
+              power * (lightColor & 0xFF) * (this.homeLightColor & 0xFF));
+          float xLightSourceInLight = -light.getWidth() / 2 + (lightSource.getX() * light.getWidth());
+          float yLightSourceInLight = light.getDepth() / 2 - (lightSource.getY() * light.getDepth());
+          this.sunflow.parameter("center",
+              new Point3(light.getX() + xLightSourceInLight * cos - yLightSourceInLight * sin,
+                  light.getElevation() + (lightSource.getZ() * light.getHeight()),
+                  light.getY() + xLightSourceInLight * sin + yLightSourceInLight * cos));                    
+          this.sunflow.parameter("radius", lightRadius);
+          this.sunflow.parameter("samples", 4);
+          this.sunflow.light(UUID.randomUUID().toString(), "sphere");
+        }
+      }
+    }
+
+    this.sunflow.parameter("depths.diffuse", 1);
+    this.sunflow.parameter("depths.reflection", 4);
+    this.sunflow.parameter("depths.refraction", 16);
+    this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
+    
+    // Use a spiral computing
+    this.sunflow.parameter("bucket.size", 32);
+    this.sunflow.parameter("bucket.order", "spiral");
+    this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
   }
 
   /**
@@ -240,12 +292,93 @@ public class PhotoRenderer {
                      Camera camera, 
                      final ImageObserver observer) {
     this.renderingThread = Thread.currentThread();
+
+    if (this.sunSkyLightName != null) {
+      this.sunflow.remove(this.sunSkyLightName);
+    }
+    if (this.sunLightName != null) {
+      this.sunflow.remove(this.sunLightName);
+    }
+    float [] sunDirection = getSunDirection(this.compass, Camera.convertTimeToTimeZone(camera.getTime(), this.compass.getTimeZone()));
+    // Update Sun direction during daytime
+    if (sunDirection [1] > -0.075f) {
+      if (this.useSunSky) {
+        this.sunflow.parameter("up", new Vector3(0, 1, 0));
+        this.sunflow.parameter("east", 
+            new Vector3((float)Math.sin(compass.getNorthDirection()), 0, (float)Math.cos(compass.getNorthDirection())));
+        this.sunflow.parameter("sundir", new Vector3(sunDirection [0], sunDirection [1], sunDirection [2]));
+        this.sunflow.parameter("turbidity", 6f);
+        this.sunflow.parameter("samples", this.useAmbientOcclusion ? 0 : 12); 
+        this.sunSkyLightName = UUID.randomUUID().toString();
+        this.sunflow.light(this.sunSkyLightName, "sunsky");
+      }
+
+      // Retrieve sun color
+      SunSkyLight sunSkyLight = new SunSkyLight();
+      ParameterList parameterList = new ParameterList();
+      parameterList.addVectors("up", InterpolationType.NONE, new float [] {0, 1, 0});
+      parameterList.addVectors("east", InterpolationType.NONE, 
+          new float [] {(float)Math.sin(compass.getNorthDirection()), 0, (float)Math.cos(compass.getNorthDirection())});
+      parameterList.addVectors("sundir", InterpolationType.NONE, 
+          new float [] {sunDirection [0], sunDirection [1], sunDirection [2]});
+      sunSkyLight.update(parameterList, this.sunflow);
+      float [] sunColor = sunSkyLight.getSunColor().getRGB();
+      
+      // Simulate additional Sun with a faraway sphere light of a color depending of the hour of the day
+      int sunPower = this.useAmbientOcclusion ? 40 : 10; 
+      this.sunflow.parameter("radiance", null,
+          (this.homeLightColor >> 16) * sunPower * (float)Math.sqrt(sunColor [0]), 
+          ((this.homeLightColor >> 8) & 0xFF) * sunPower * (float)Math.sqrt(sunColor [1]), 
+          (this.homeLightColor & 0xFF) * sunPower * (float)Math.sqrt(sunColor [2]));
+      this.sunflow.parameter("center", new Point3(1000000 * sunDirection [0], 1000000 * sunDirection [1], 1000000 * sunDirection [2])); 
+      this.sunflow.parameter("radius", 10000f);  
+      this.sunflow.parameter("samples", 4);
+      this.sunLightName = UUID.randomUUID().toString();
+      this.sunflow.light(this.sunLightName, "sphere");
+
+      if (this.useAmbientOcclusion) {
+        this.sunflow.parameter("gi.engine", "ambocc");
+        this.sunflow.parameter("gi.ambocc.bright", null, new float [] {1, 1, 1});
+        // Use complementary color
+        this.sunflow.parameter("gi.ambocc.dark", null, 
+            new float [] {(sunColor [1] + sunColor [2]) / 200, 
+                          (sunColor [0] + sunColor [2]) / 200,
+                          (sunColor [0] + sunColor [1]) / 200});
+        this.sunflow.parameter("gi.ambocc.samples", 1);
+        this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
+      }
+    }
+
+    // Update camera lens 
+    final String CAMERA_NAME = "camera";    
+    switch (camera.getLens()) {
+      case SPHERICAL:
+        this.sunflow.camera(CAMERA_NAME, "spherical");
+        break;
+      case FISHEYE:
+        this.sunflow.camera(CAMERA_NAME, "fisheye");
+        break;
+      case NORMAL:
+        this.sunflow.parameter("focus.distance", 250f);
+        this.sunflow.parameter("lens.radius", 1f);
+        this.sunflow.camera(CAMERA_NAME, "thinlens");
+        break;
+      case PINHOLE:
+      default: 
+        this.sunflow.camera(CAMERA_NAME, "pinhole");
+        break;
+    }
     
-    // Update pinhole camera lens from camera location in parameter
+    // Update camera location 
     Point3 eye = new Point3(camera.getX(), camera.getZ(), camera.getY());
     Matrix4 transform;
     float yaw = camera.getYaw();
-    float pitch = camera.getPitch();
+    float pitch;
+    if (camera.getLens() == Camera.Lens.SPHERICAL) {
+      pitch = 0;
+    } else {
+      pitch = camera.getPitch();
+    }
     double pitchCos = Math.cos(pitch);
     if (Math.abs(pitchCos) > 1E-6) {
       // Set the point the camera is pointed to 
@@ -270,17 +403,19 @@ public class PhotoRenderer {
     // Set image size and quality
     this.sunflow.parameter("resolutionX", image.getWidth());
     this.sunflow.parameter("resolutionY", image.getHeight());
-    this.sunflow.parameter("filter", "gaussian"); // box, gaussian, blackman-harris, sinc, mitchell or triangle
     
     if (this.quality == Quality.HIGH) {
+      this.sunflow.parameter("filter", "blackman-harris"); // box, gaussian, blackman-harris, sinc, mitchell or triangle
       // The bigger aa.max is, the cleanest rendering you get
       this.sunflow.parameter("aa.min", 1);
       this.sunflow.parameter("aa.max",  2);
     } else {
+      this.sunflow.parameter("filter", "box");
       this.sunflow.parameter("aa.min", 0);
-      this.sunflow.parameter("aa.max",  1); 
-      this.sunflow.parameter("sampler", "fast"); // ipr, fast or bucket 
+      this.sunflow.parameter("aa.max", 0); 
     }
+    this.sunflow.parameter("sampler", "bucket"); // ipr, fast or bucket 
+
     // Render image with default camera
     this.sunflow.parameter("camera", CAMERA_NAME);
     this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
@@ -299,18 +434,49 @@ public class PhotoRenderer {
     }
   }
 
+  /**
+   * Returns all the light children of the given <code>furniture</code>.  
+   */
+  private List<HomeLight> getLights(List<HomePieceOfFurniture> furniture) {
+    List<HomeLight> lights = new ArrayList<HomeLight>();
+    for (HomePieceOfFurniture piece : furniture) {
+      if (piece instanceof HomeLight) {
+        lights.add((HomeLight)piece);
+      } else if (piece instanceof HomeFurnitureGroup) {
+        lights.addAll(getLights(((HomeFurnitureGroup)piece).getFurniture()));
+      } 
+    }
+    return lights;
+  }
+
+  /**
+   * Returns sun direction at a given <code>time</code>. 
+   * @author Frdric Mantegazza
+   */
+  private float [] getSunDirection(Compass compass, long time) {
+    float elevation = compass.getSunElevation(time);
+    float azimuth = compass.getSunAzimuth(time);
+    azimuth += compass.getNorthDirection() - Math.PI / 2f;
+    return new float [] {(float)(Math.cos(azimuth) * Math.cos(elevation)),
+                         (float)Math.sin(elevation),
+                         (float)(Math.sin(azimuth) * Math.cos(elevation))};
+  }
+
   /**
    * Exports the given Java 3D <code>node</code> and its children to Sunflow API.  
    */
-  private void exportNode(Node node, boolean noConstantShader) throws IOException {
-    exportNode(node, noConstantShader, new Transform3D());
+  private void exportNode(Node node, boolean ignoreTransparency, 
+                          boolean ignoreConstantShader, boolean silk) throws IOException {
+    exportNode(node, ignoreTransparency, ignoreConstantShader, silk, new Transform3D());
   }
 
   /**
    * Exports all the 3D shapes children of <code>node</code> at OBJ format.
    */ 
   private void exportNode(Node node, 
-                          boolean noConstantShader,
+                          boolean ignoreTransparency,
+                          boolean ignoreConstantShader, 
+                          boolean silk,
                           Transform3D parentTransformations) throws IOException {
     if (node instanceof Group) {
       if (node instanceof TransformGroup) {
@@ -322,18 +488,23 @@ public class PhotoRenderer {
       // Export all children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        exportNode((Node)enumeration.nextElement(), noConstantShader, parentTransformations);
+        exportNode((Node)enumeration.nextElement(), ignoreTransparency, ignoreConstantShader, silk, parentTransformations);
       }
     } else if (node instanceof Link) {
-      exportNode(((Link)node).getSharedGroup(), noConstantShader, parentTransformations);
+      exportNode(((Link)node).getSharedGroup(), ignoreTransparency, ignoreConstantShader, silk, parentTransformations);
     } else if (node instanceof Shape3D) {
       Shape3D shape = (Shape3D)node;
       Appearance appearance = shape.getAppearance();
-      RenderingAttributes renderingAttributes = appearance.getRenderingAttributes();
-      if (renderingAttributes == null
-          || renderingAttributes.getVisible()) {
+      RenderingAttributes renderingAttributes = appearance != null 
+          ? appearance.getRenderingAttributes() : null;
+      TransparencyAttributes transparencyAttributes = appearance != null 
+          ? appearance.getTransparencyAttributes() : null;
+      // Ignore invisible shapes and fully transparent shapes without a texture 
+      if ((renderingAttributes == null
+              || renderingAttributes.getVisible())
+          && (transparencyAttributes == null
+              || transparencyAttributes.getTransparency() != 1)) {
         String shapeName = (String)shape.getUserData();
-        
         // Build a unique object name
         String uuid = UUID.randomUUID().toString();
   
@@ -344,7 +515,7 @@ public class PhotoRenderer {
           appearanceName = "shader" + uuid;
           boolean mirror = shapeName != null
               && shapeName.startsWith(ModelManager.MIRROR_SHAPE_PREFIX);
-          exportAppearance(appearance, appearanceName, mirror, noConstantShader);
+          exportAppearance(appearance, appearanceName, mirror, ignoreTransparency, ignoreConstantShader, silk);
         }
 
         // Export object geometries
@@ -930,7 +1101,9 @@ public class PhotoRenderer {
   private void exportAppearance(Appearance appearance,
                                 String appearanceName, 
                                 boolean mirror,
-                                boolean noConstantShader) throws IOException {
+                                boolean ignoreTransparency,
+                                boolean ignoreConstantShader,
+                                boolean silk) throws IOException {
     Texture texture = appearance.getTexture();    
     if (mirror) {
       Material material = appearance.getMaterial();
@@ -955,14 +1128,27 @@ public class PhotoRenderer {
         imagePath = imageFile.getAbsolutePath();
         this.textureImagesCache.put(texture, imagePath);
       }
-      this.sunflow.parameter("texture", imagePath);
-      
       Material material = appearance.getMaterial();
+      float shininess;
       if (material != null
-          && material.getShininess() > 64) {
-        this.sunflow.parameter("shiny", material.getShininess() / 512f);
-        this.sunflow.shader(appearanceName, "textured_shiny_diffuse");
+          && (shininess = material.getShininess()) > 1) {
+        if (silk) {
+          this.sunflow.parameter("diffuse.texture", imagePath);
+          Color3f color = new Color3f();
+          material.getSpecularColor(color);
+          float [] specularColor = new float [] {
+              (float)Math.sqrt(color.x) / 2, (float)Math.sqrt(color.y) / 2, (float)Math.sqrt(color.z) / 2};
+          this.sunflow.parameter("specular", null, specularColor);
+          this.sunflow.parameter("glossyness", (float)Math.pow(10, -Math.log(shininess) / Math.log(5)));
+          this.sunflow.parameter("samples", 1);
+          this.sunflow.shader(appearanceName, "uber");
+        } else {
+          this.sunflow.parameter("texture", imagePath);
+          this.sunflow.parameter("shiny", shininess / 512f);
+          this.sunflow.shader(appearanceName, "textured_shiny_diffuse");
+        }
       } else {
+        this.sunflow.parameter("texture", imagePath);
         this.sunflow.shader(appearanceName, "textured_diffuse");
       }
     } else {
@@ -974,7 +1160,8 @@ public class PhotoRenderer {
 
         TransparencyAttributes transparencyAttributes = appearance.getTransparencyAttributes();
         if (transparencyAttributes != null
-            && transparencyAttributes.getTransparency() > 0) {
+            && transparencyAttributes.getTransparency() > 0
+            && !ignoreTransparency) {
           if (material instanceof OBJMaterial
               && ((OBJMaterial)material).isOpticalDensitySet()) {
             this.sunflow.parameter("eta", ((OBJMaterial)material).getOpticalDensity());
@@ -982,29 +1169,43 @@ public class PhotoRenderer {
             // Use glass ETA as default
             this.sunflow.parameter("eta", 1.55f);
           }
-          // Increase color to render better transparent objects
-          this.sunflow.parameter("color", null,
-              new float [] {Math.min(diffuseColor [0] * 2f, 1f), Math.min(diffuseColor [1] * 2f, 1f), Math.min(diffuseColor [2] * 2f, 1f)});
-          this.sunflow.parameter("absorbtion.distance", 0f);          
-          float transparency = transparencyAttributes.getTransparency();
-          this.sunflow.parameter("absorbtion.color", null, new float [] {transparency, transparency, transparency}); 
+          float transparency = 1 - transparencyAttributes.getTransparency();
+          this.sunflow.parameter("color", null, 
+              new float [] {transparency + (1 - transparency) * (float)Math.sqrt(diffuseColor [0]), 
+                            transparency + (1 - transparency) * (float)Math.sqrt(diffuseColor [1]), 
+                            transparency + (1 - transparency) * (float)Math.sqrt(diffuseColor [2])});
+          this.sunflow.parameter("transparency", transparency);
           this.sunflow.shader(appearanceName, "glass");
-        } else {  
+        } else if (material.getLightingEnable()
+                   || ignoreConstantShader) {  
           this.sunflow.parameter("diffuse", null, diffuseColor);
           float shininess = material.getShininess();
-          if (shininess > 64) {
-            this.sunflow.parameter("shiny", shininess / 512f);
-            this.sunflow.shader(appearanceName, "shiny_diffuse");
+          if (shininess > 1) {
+            if (silk) {
+              material.getSpecularColor(color);
+              float [] specularColor = new float [] {
+                   (float)Math.sqrt(color.x) / 2, (float)Math.sqrt(color.y) / 2, (float)Math.sqrt(color.z) / 2};
+              this.sunflow.parameter("specular", null, specularColor);
+              this.sunflow.parameter("glossyness", (float)Math.pow(10, -Math.log(shininess) / Math.log(5)));
+              this.sunflow.parameter("samples", 1);
+              this.sunflow.shader(appearanceName, "uber");
+            } else { 
+              this.sunflow.parameter("shiny", shininess / 512f);
+              this.sunflow.shader(appearanceName, "shiny_diffuse");
+            }
           } else {
             this.sunflow.shader(appearanceName, "diffuse");
           }
+        } else {
+          this.sunflow.parameter("color", null, diffuseColor);
+          this.sunflow.shader(appearanceName, "constant");
         }
       } else {
         ColoringAttributes coloringAttributes = appearance.getColoringAttributes();
         if (coloringAttributes != null) {
           Color3f color = new Color3f();
           coloringAttributes.getColor(color);
-          if (noConstantShader) {
+          if (ignoreConstantShader) {
             this.sunflow.parameter("diffuse", null, new float [] {color.x, color.y, color.z});
             this.sunflow.shader(appearanceName, "diffuse");
           } else {
@@ -1088,13 +1289,13 @@ public class PhotoRenderer {
     }
 
     private void notifyObserver(final int flags, final int x, final int y, final int width, final int height) {
-      EventQueue.invokeLater(new Runnable() {
-          public void run() {
-            if (observer != null) {
+      if (observer != null) {
+        EventQueue.invokeLater(new Runnable() {
+            public void run() {
               observer.imageUpdate(image, flags, x, y, width, height);
             }
-          }
-        });
+          });
+      }
     }
   }
 
