@@ -20,7 +20,6 @@
 package com.eteks.sweethome3d.j3d;
 
 import java.awt.geom.Area;
-import java.awt.geom.GeneralPath;
 import java.awt.geom.PathIterator;
 import java.awt.geom.Point2D;
 import java.util.ArrayList;
@@ -33,13 +32,11 @@ import java.util.Map;
 import javax.media.j3d.Appearance;
 import javax.media.j3d.BranchGroup;
 import javax.media.j3d.Geometry;
-import javax.media.j3d.Material;
 import javax.media.j3d.Node;
 import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.Texture;
 import javax.media.j3d.TextureAttributes;
-import javax.vecmath.Color3f;
 import javax.vecmath.Point3f;
 import javax.vecmath.TexCoord2f;
 
@@ -54,15 +51,9 @@ import com.sun.j3d.utils.geometry.NormalGenerator;
  * Root of room branch.
  */
 public class Room3D extends Object3DBranch {
-  private static final Material          DEFAULT_MATERIAL            = new Material();
   private static final TextureAttributes MODULATE_TEXTURE_ATTRIBUTES = new TextureAttributes();
   
-  private static final Map<Integer, Material> materials = new HashMap<Integer, Material>();
-  
   static {
-    DEFAULT_MATERIAL.setCapability(Material.ALLOW_COMPONENT_READ);
-    DEFAULT_MATERIAL.setShininess(1);
-    DEFAULT_MATERIAL.setSpecularColor(0, 0, 0);
     MODULATE_TEXTURE_ATTRIBUTES.setTextureMode(TextureAttributes.MODULATE);
   }
   
@@ -168,77 +159,146 @@ public class Room3D extends Object3DBranch {
   private Geometry [] createRoomGeometries(int roomPart, HomeTexture texture) {
     Room room = (Room)getUserData();
     float [][] points = room.getPoints();
-    if (points.length > 2) {
-      boolean clockwise = room.isClockwise();
-      if (clockwise && roomPart == FLOOR_PART
-          || !clockwise && roomPart == CEILING_PART) {
-        // Reverse points order if they are in the good order
-        points = getReversedArray(points);
-      }
-      
+    if ((roomPart == FLOOR_PART && room.isFloorVisible()
+         || roomPart == CEILING_PART && room.isCeilingVisible())
+        && points.length > 2) {
       // If room isn't singular retrieve all the points of its different polygons 
       List<float [][]> roomPoints = new ArrayList<float[][]>();
-      if (!room.isSingular()) {        
-        GeneralPath roomShape = new GeneralPath();
-        roomShape.moveTo(points [0][0], points [0][1]);
-        for (int i = 1; i < points.length; i++) {
-          roomShape.lineTo(points [i][0], points [i][1]);
-        }
-        roomShape.closePath();
+      Map<Integer, List<float [][]>> roomHoles = new HashMap<Integer, List<float [][]>>();
+      List<Room> rooms = this.home.getRooms();
+      if (!room.isSingular() || rooms.get(rooms.size() - 1) != room) {        
+        Area roomArea = new Area(getShape(points));
+        // Remove other rooms surface that may overlap the current room
+        for (int i = rooms.size() - 1; i > 0 && rooms.get(i) != room; i--) {
+          Room otherRoom = rooms.get(i);
+          if (roomPart == FLOOR_PART && otherRoom.isFloorVisible()
+              || roomPart == CEILING_PART && otherRoom.isCeilingVisible()) {
+            roomArea.subtract(new Area(getShape(otherRoom.getPoints())));
+          }
+        }        
         // Retrieve the points of the different polygons 
         // and reverse their points order if necessary
-        Area roomArea = new Area(roomShape);
         List<float []> currentPathPoints = new ArrayList<float[]>();
         roomPoints = new ArrayList<float[][]>();
+        float [] previousRoomPoint = null;
+        int i = 0;
         for (PathIterator it = roomArea.getPathIterator(null); !it.isDone(); ) {
           float [] roomPoint = new float[2];
           switch (it.currentSegment(roomPoint)) {
-            case PathIterator.SEG_MOVETO : 
-              currentPathPoints.add(roomPoint);
+            case PathIterator.SEG_MOVETO :
+              if (previousRoomPoint == null
+                  || roomPoint [0] != previousRoomPoint [0] 
+                  || roomPoint [1] != previousRoomPoint [1]) {
+                currentPathPoints.add(roomPoint);
+              }
+              previousRoomPoint = roomPoint;
               break;
             case PathIterator.SEG_LINETO : 
-              currentPathPoints.add(roomPoint);
+              if (previousRoomPoint == null
+                  || roomPoint [0] != previousRoomPoint [0] 
+                  || roomPoint [1] != previousRoomPoint [1]) {
+                currentPathPoints.add(roomPoint);
+              }
+              previousRoomPoint = roomPoint;
               break;
             case PathIterator.SEG_CLOSE :
-              float [][] pathPoints = 
-                  currentPathPoints.toArray(new float [currentPathPoints.size()][]);
-              boolean pathPointsClockwise = new Room(pathPoints).isClockwise();
-              if (pathPointsClockwise && roomPart == FLOOR_PART
-                  || !pathPointsClockwise && roomPart == CEILING_PART) {
-                pathPoints = getReversedArray(pathPoints);
+              if (currentPathPoints.get(0) [0] == previousRoomPoint [0] 
+                  && currentPathPoints.get(0) [1] == previousRoomPoint [1]) {
+                currentPathPoints.remove(currentPathPoints.size() - 1);
+              }
+              if (currentPathPoints.size() > 2) {
+                float [][] pathPoints = 
+                    currentPathPoints.toArray(new float [currentPathPoints.size()][]);
+                Room subRoom = new Room(pathPoints);
+                if (subRoom.getArea() > 0) {
+                  boolean pathPointsClockwise = subRoom.isClockwise();
+                  if (pathPointsClockwise) {
+                    // Store counter clockwise points as holes
+                    if (roomPart != CEILING_PART) {
+                      pathPoints = getReversedArray(pathPoints);
+                    }
+                    List<float [][]> holes = roomHoles.get(i);
+                    if (holes == null) {
+                      holes = new ArrayList<float [][]>(1);
+                      roomHoles.put(i, holes);
+                    }
+                    holes.add(pathPoints);
+                  } else {
+                    if (roomPart == CEILING_PART) {
+                      pathPoints = getReversedArray(pathPoints);
+                    }
+                    roomPoints.add(pathPoints);
+                    i++;
+                  }
+                }
               }
-              roomPoints.add(pathPoints);
               currentPathPoints.clear();
+              previousRoomPoint = null;
               break;
           }
           it.next();        
         }
       } else {
+        boolean clockwise = room.isClockwise();
+        if (clockwise && roomPart == FLOOR_PART
+            || !clockwise && roomPart == CEILING_PART) {
+          // Reverse points order if they are in the good order
+          points = getReversedArray(points);
+        }
         roomPoints = Arrays.asList(new float [][][] {points});
       }
       
       Geometry [] geometries = new Geometry [roomPoints.size()];
       for (int i = 0; i < geometries.length; i++) {
         float [][] geometryPoints = roomPoints.get(i);
+        List<float [][]> geometryHoles = roomHoles.get(i);
+        if (geometryHoles == null) {
+          geometryHoles = Collections.emptyList();
+        }
+        int vertexCount = geometryPoints.length;
+        int [] stripCounts = new int [1 + geometryHoles.size()];
+        int j = 0;
+        stripCounts [j++] = geometryPoints.length;
+        for (float [][] geometryHole : geometryHoles) {
+          vertexCount += geometryHole.length;
+          stripCounts [j++] = geometryHole.length;
+        }
         // Compute room coordinates
-        Point3f [] coords = new Point3f [geometryPoints.length];
-        for (int j = 0; j < geometryPoints.length; j++) {
+        Point3f [] coords = new Point3f [vertexCount];
+        for (j = 0; j < geometryPoints.length; j++) {
           float y = roomPart == FLOOR_PART 
               ? 0 
               : getRoomHeightAt(geometryPoints [j][0], geometryPoints [j][1]);
           coords [j] = new Point3f(geometryPoints [j][0], y, geometryPoints [j][1]);
         }
+        for (float [][] geometryHole : geometryHoles) {
+          for (int k = 0; k < geometryHole.length; k++, j++) {
+            float y = roomPart == FLOOR_PART 
+                ? 0 
+                : getRoomHeightAt(geometryHole [k][0], geometryHole [k][1]);
+            coords [j] = new Point3f(geometryHole [k][0], y, geometryHole [k][1]);
+          }
+        }
         GeometryInfo geometryInfo = new GeometryInfo(GeometryInfo.POLYGON_ARRAY);
         geometryInfo.setCoordinates (coords);
-        geometryInfo.setStripCounts(new int [] {coords.length});
+        geometryInfo.setStripCounts(stripCounts);
+        geometryInfo.setContourCounts(new int [] {stripCounts.length});
         // Compute room texture coordinates
         if (texture != null) {
-          TexCoord2f [] textureCoords = new TexCoord2f [geometryPoints.length];
-          for (int j = 0; j < geometryPoints.length; j++) {
-            textureCoords [j] = new TexCoord2f(geometryPoints[j][0] / texture.getWidth(), 
+          TexCoord2f [] textureCoords = new TexCoord2f [vertexCount];
+          for (j = 0; j < geometryPoints.length; j++) {
+            textureCoords [j] = new TexCoord2f(geometryPoints [j][0] / texture.getWidth(), 
                 roomPart == FLOOR_PART 
-                    ? -geometryPoints[j][1] / texture.getHeight()
-                    : geometryPoints[j][1] / texture.getHeight());
+                    ? -geometryPoints [j][1] / texture.getHeight()
+                    : geometryPoints [j][1] / texture.getHeight());
+          }
+          for (float [][] geometryHole : geometryHoles) {
+            for (int k = 0; k < geometryHole.length; k++, j++) {
+              textureCoords [j] = new TexCoord2f(geometryHole [k][0] / texture.getWidth(), 
+                  roomPart == FLOOR_PART 
+                      ? -geometryHole [k][1] / texture.getHeight()
+                      : geometryHole [k][1] / texture.getHeight());
+            }
           }
           geometryInfo.setTextureCoordinateParams(1, 2);
           geometryInfo.setTextureCoordinates(0, textureCoords);
@@ -299,9 +359,9 @@ public class Room3D extends Object3DBranch {
   private void updateRoomAppearance(boolean waitTextureLoadingEnd) {
     Room room = (Room)getUserData();
     updateRoomPartAppearance(((Shape3D)getChild(FLOOR_PART)).getAppearance(), 
-        room.getFloorTexture(), waitTextureLoadingEnd, room.getFloorColor(), room.isFloorVisible());
+        room.getFloorTexture(), waitTextureLoadingEnd, room.getFloorColor(), room.getFloorShininess(), room.isFloorVisible());
     updateRoomPartAppearance(((Shape3D)getChild(CEILING_PART)).getAppearance(), 
-        room.getCeilingTexture(), waitTextureLoadingEnd, room.getCeilingColor(), room.isCeilingVisible());
+        room.getCeilingTexture(), waitTextureLoadingEnd, room.getCeilingColor(), room.getCeilingShininess(), room.isCeilingVisible());
   }
   
   /**
@@ -311,13 +371,14 @@ public class Room3D extends Object3DBranch {
                                         final HomeTexture roomPartTexture,
                                         boolean waitTextureLoadingEnd,
                                         Integer roomPartColor,
+                                        float shininess,
                                         boolean visible) {
     if (roomPartTexture == null) {
-      roomPartAppearance.setMaterial(getMaterial(roomPartColor));
+      roomPartAppearance.setMaterial(getMaterial(roomPartColor, roomPartColor, shininess));
       roomPartAppearance.setTexture(null);
     } else {
       // Update material and texture of room part
-      roomPartAppearance.setMaterial(DEFAULT_MATERIAL);
+      roomPartAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_AMBIENT_COLOR, shininess));
       final TextureManager textureManager = TextureManager.getInstance();
       textureManager.loadTexture(roomPartTexture.getImage(), waitTextureLoadingEnd,
           new TextureManager.TextureObserver() {
@@ -330,22 +391,4 @@ public class Room3D extends Object3DBranch {
     RenderingAttributes renderingAttributes = roomPartAppearance.getRenderingAttributes();
     renderingAttributes.setVisible(visible);
   }
-  
-  private Material getMaterial(Integer color) {
-    if (color != null) {
-      Material material = materials.get(color); 
-      if (material == null) {
-        Color3f materialColor = new Color3f(((color >>> 16) & 0xFF) / 256f,
-                                             ((color >>> 8) & 0xFF) / 256f,
-                                                     (color & 0xFF) / 256f);
-        material = new Material(materialColor, new Color3f(), materialColor, new Color3f(), 1);
-        material.setCapability(Material.ALLOW_COMPONENT_READ);
-        // Store created materials in cache
-        materials.put(color, material);
-      }
-      return material;
-    } else {
-      return DEFAULT_MATERIAL;
-    }
-  }
 }
\ No newline at end of file
