@@ -28,21 +28,17 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 
 import javax.media.j3d.Appearance;
 import javax.media.j3d.BranchGroup;
 import javax.media.j3d.Geometry;
-import javax.media.j3d.Material;
 import javax.media.j3d.Node;
 import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.Texture;
 import javax.media.j3d.TextureAttributes;
 import javax.media.j3d.TransparencyAttributes;
-import javax.vecmath.Color3f;
 import javax.vecmath.Point3f;
 import javax.vecmath.TexCoord2f;
 
@@ -59,15 +55,9 @@ import com.sun.j3d.utils.geometry.NormalGenerator;
  * Root of wall branch.
  */
 public class Wall3D extends Object3DBranch {
-  private static final Material          DEFAULT_MATERIAL            = new Material();
   private static final TextureAttributes MODULATE_TEXTURE_ATTRIBUTES = new TextureAttributes();
   
-  private static final Map<Integer, Material> materials = new HashMap<Integer, Material>();
-  
   static {
-    DEFAULT_MATERIAL.setCapability(Material.ALLOW_COMPONENT_READ);
-    DEFAULT_MATERIAL.setShininess(1);
-    DEFAULT_MATERIAL.setSpecularColor(0, 0, 0);
     MODULATE_TEXTURE_ATTRIBUTES.setTextureMode(TextureAttributes.MODULATE);
   }
   
@@ -191,7 +181,7 @@ public class Wall3D extends Object3DBranch {
     float [][] wallSidePoints = getWallSidePoints(wallSide);
     float [] textureReferencePoint = wallSide == LEFT_WALL_SIDE
         ? wallSidePoints [0]
-        : wallSidePoints [3];
+        : wallSidePoints [wallSidePoints.length - 1];
     Shape wallShape = getShape(wallSidePoints);
     Area wallArea = new Area(wallShape);
     float wallHeightAtStart = getWallHeightAtStart();
@@ -205,6 +195,7 @@ public class Wall3D extends Object3DBranch {
     double sinWallYawAngle = Math.sin(wallYawAngle);
     double wallXStartWithZeroYaw = cosWallYawAngle * wall.getXStart() + sinWallYawAngle * wall.getYStart();
     double wallXEndWithZeroYaw = cosWallYawAngle * wall.getXEnd() + sinWallYawAngle * wall.getYEnd();
+    boolean roundWall = wall.getArcExtent() != null && wall.getArcExtent() != 0; 
     double topLineAlpha;
     double topLineBeta;
     if (wallHeightAtStart == wallHeightAtEnd) {
@@ -274,12 +265,13 @@ public class Wall3D extends Object3DBranch {
             float [][] wallPartPoints = wallPoints.toArray(new float[wallPoints.size()][]);
             // Compute geometry for vertical part
             wallGeometries.add(createWallVerticalPartGeometry(wall, wallPartPoints, 0, 
-                cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta, texture, textureReferencePoint));
+                cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta, texture, 
+                textureReferencePoint, wallSide));
             // Compute geometry for bottom part
-            wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, 0));
+            wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, 0, true, roundWall));
             // Compute geometry for top part
             wallGeometries.add(createWallTopPartGeometry(wallPartPoints, 
-                cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta));
+                cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta, roundWall));
           }
         }
         wallPoints.clear();
@@ -326,9 +318,11 @@ public class Wall3D extends Object3DBranch {
               // Generate geometry for wall part below window
               if (lowestDoorOrWindow.getElevation() > 0) {
                 wallGeometries.add(createWallVerticalPartGeometry(wall, wallPartPoints, 0, 
-                    cosWallYawAngle, sinWallYawAngle, 0, lowestDoorOrWindow.getElevation(), texture, textureReferencePoint));
-                wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, 0));
-                wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, lowestDoorOrWindow.getElevation()));
+                    cosWallYawAngle, sinWallYawAngle, 0, lowestDoorOrWindow.getElevation(), texture, 
+                    textureReferencePoint, wallSide));
+                wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, 0, true, roundWall));
+                wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, 
+                    lowestDoorOrWindow.getElevation(), false, roundWall));
               }
               
               // Generate geometry for wall parts between superimposed windows
@@ -343,9 +337,10 @@ public class Wall3D extends Object3DBranch {
                 if (i < doorsOrWindows.size()
                     && lowerDoorOrWindow.getElevation() + lowerDoorOrWindow.getHeight() < higherDoorOrWindow.getElevation()) {
                   wallGeometries.add(createWallVerticalPartGeometry(wall, wallPartPoints, lowerDoorOrWindow.getElevation() + lowerDoorOrWindow.getHeight(), 
-                      cosWallYawAngle, sinWallYawAngle, 0, higherDoorOrWindow.getElevation(), texture, textureReferencePoint));
-                  wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, lowerDoorOrWindow.getElevation() + lowerDoorOrWindow.getHeight()));
-                  wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, higherDoorOrWindow.getElevation()));
+                      cosWallYawAngle, sinWallYawAngle, 0, higherDoorOrWindow.getElevation(), texture, textureReferencePoint, wallSide));
+                  wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, 
+                      lowerDoorOrWindow.getElevation() + lowerDoorOrWindow.getHeight(), true, roundWall));
+                  wallGeometries.add(createWallHorizontalPartGeometry(wallPartPoints, higherDoorOrWindow.getElevation(), false, roundWall));
                 }
               }
                 
@@ -366,11 +361,11 @@ public class Wall3D extends Object3DBranch {
               // Generate geometry for wall part above window
               if (doorOrWindowTop < minTopY) {
                 wallGeometries.add(createWallVerticalPartGeometry(wall, wallPartPoints, doorOrWindowTop, 
-                    cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta, texture, textureReferencePoint));
+                    cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta, texture, textureReferencePoint, wallSide));
                 wallGeometries.add(createWallHorizontalPartGeometry(
-                    wallPartPoints, doorOrWindowTop));
+                    wallPartPoints, doorOrWindowTop, true, roundWall));
                 wallGeometries.add(createWallTopPartGeometry(wallPartPoints, 
-                    cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta));
+                    cosWallYawAngle, sinWallYawAngle, topLineAlpha, topLineBeta, roundWall));
               }
             }
             wallPoints.clear();
@@ -412,15 +407,15 @@ public class Wall3D extends Object3DBranch {
     float [][] wallPoints = wall.getPoints();
     
     if (wallSide == LEFT_WALL_SIDE) {
-      wallPoints [2][0] = wall.getXEnd();
-      wallPoints [2][1] = wall.getYEnd();
-      wallPoints [3][0] = wall.getXStart();
-      wallPoints [3][1] = wall.getYStart();
+      for (int i = wallPoints.length / 2; i < wallPoints.length; i++) {
+        wallPoints [i][0] = (wallPoints [i][0] + wallPoints [wallPoints.length - i - 1][0]) / 2;
+        wallPoints [i][1] = (wallPoints [i][1] + wallPoints [wallPoints.length - i - 1][1]) / 2;
+      }
     } else { // RIGHT_WALL_SIDE
-      wallPoints [1][0] = wall.getXEnd();
-      wallPoints [1][1] = wall.getYEnd();
-      wallPoints [0][0] = wall.getXStart();
-      wallPoints [0][1] = wall.getYStart();
+      for (int i = 0, n = wallPoints.length / 2; i < n; i++) {
+        wallPoints [i][0] = (wallPoints [i][0] + wallPoints [wallPoints.length - i - 1][0]) / 2;
+        wallPoints [i][1] = (wallPoints [i][1] + wallPoints [wallPoints.length - i - 1][1]) / 2;
+      }
     }
     return wallPoints;
   }
@@ -436,35 +431,54 @@ public class Wall3D extends Object3DBranch {
                                                   double cosWallYawAngle, double sinWallYawAngle, 
                                                   double topLineAlpha, double topLineBeta, 
                                                   HomeTexture texture,
-                                                  float [] textureReferencePoint) {
+                                                  float [] textureReferencePoint,
+                                                  int wallSide) {
     // Compute wall coordinates
     Point3f [] bottom = new Point3f [points.length];
     Point3f [] top    = new Point3f [points.length];
     double  [] distanceSqToWallMiddle = new double [points.length];
+    Float   [] pointUCoordinates = new Float [points.length];
     float xStart = wall.getXStart();
     float yStart = wall.getYStart();
     float xEnd = wall.getXEnd();
     float yEnd = wall.getYEnd();
+    Float arcExtent = wall.getArcExtent();
+    float [] arcCircleCenter = null;
+    float arcCircleRadius = 0;
+    float referencePointAngle = 0;
+    if (arcExtent != null && arcExtent != 0) {
+      arcCircleCenter = new float [] {wall.getXArcCircleCenter(), wall.getYArcCircleCenter()};
+      arcCircleRadius = (float)Point2D.distance(arcCircleCenter [0], arcCircleCenter [1], 
+          xStart, yStart);
+      referencePointAngle = (float)Math.atan2(textureReferencePoint [1] - arcCircleCenter [1], 
+          textureReferencePoint [0] - arcCircleCenter [0]);
+    }
     for (int i = 0; i < points.length; i++) {
-      bottom [i] = new Point3f(points[i][0], yMin, points[i][1]);
-      distanceSqToWallMiddle [i] = Line2D.ptLineDistSq(xStart, yStart, xEnd, yEnd, bottom [i].x, bottom [i].z);
+      bottom [i] = new Point3f(points [i][0], yMin, points [i][1]);
+      if (arcCircleCenter == null) {
+        distanceSqToWallMiddle [i] = Line2D.ptLineDistSq(xStart, yStart, xEnd, yEnd, bottom [i].x, bottom [i].z);
+      } else {
+        distanceSqToWallMiddle [i] = arcCircleRadius 
+            - Point2D.distance(arcCircleCenter [0], arcCircleCenter [1], bottom [i].x, bottom [i].z);
+        distanceSqToWallMiddle [i] *= distanceSqToWallMiddle [i];
+      }
       // Compute vertical top point 
-      double xTopPointWithZeroYaw = cosWallYawAngle * points[i][0] + sinWallYawAngle * points[i][1];
+      double xTopPointWithZeroYaw = cosWallYawAngle * points [i][0] + sinWallYawAngle * points [i][1];
       float topY = (float)(topLineAlpha * xTopPointWithZeroYaw + topLineBeta);
-      top [i] = new Point3f(points[i][0], topY, points[i][1]);
+      top [i] = new Point3f(points [i][0], topY, points [i][1]);
     }
     // Search which rectangles should be ignored
     int rectanglesCount = 0;
     boolean [] usedRectangle = new boolean [points.length]; 
     for (int i = 0; i < points.length - 1; i++) {
-      usedRectangle [i] = distanceSqToWallMiddle [i] > 0.0001
-          || distanceSqToWallMiddle [i + 1] > 0.0001;
+      usedRectangle [i] = distanceSqToWallMiddle [i] > 0.001f
+          || distanceSqToWallMiddle [i + 1] > 0.001f;
       if (usedRectangle [i]) {
         rectanglesCount++;
       } 
     }
-    usedRectangle [usedRectangle.length - 1] =  distanceSqToWallMiddle [0] > 0.0001
-        || distanceSqToWallMiddle [points.length - 1] > 0.0001;
+    usedRectangle [usedRectangle.length - 1] =  distanceSqToWallMiddle [0] > 0.001f
+        || distanceSqToWallMiddle [points.length - 1] > 0.001f;
     if (usedRectangle [usedRectangle.length - 1]) {
       rectanglesCount++;
     }
@@ -499,72 +513,114 @@ public class Wall3D extends Object3DBranch {
       float yMinTextureCoords = yMin / texture.getHeight();
       TexCoord2f firstTextureCoords = new TexCoord2f(0, yMinTextureCoords);
       j = 0;
-      for (int i = 0; i < points.length - 1; i++) {
-        if (usedRectangle [i]) {
-          if (Math.abs(distanceSqToWallMiddle [i] - halfThicknessSq) < 0.0001
-              && Math.abs(distanceSqToWallMiddle [i + 1] - halfThicknessSq) < 0.0001) {
+      // Tolerate more error with round walls since arc points are approximative
+      float epsilon = arcCircleCenter == null 
+          ? 0.0001f 
+          : halfThicknessSq / 4;
+      for (int index = 0; index < points.length; index++) {
+        int nextIndex = (index + 1) % points.length;
+        if (usedRectangle [index]) {
+          if (Math.abs(distanceSqToWallMiddle [index] - halfThicknessSq) < epsilon
+              && Math.abs(distanceSqToWallMiddle [nextIndex] - halfThicknessSq) < epsilon) {
             // Compute texture coordinates of wall part parallel to wall middle
             // according to textureReferencePoint
-            float firstHorizontalTextureCoords = (float)Point2D.distance(textureReferencePoint[0], textureReferencePoint[1], 
-                points[i][0], points[i][1]) / texture.getWidth();
-            float secondHorizontalTextureCoords = (float)Point2D.distance(textureReferencePoint[0], textureReferencePoint[1], 
-                points[i + 1][0], points[i + 1][1]) / texture.getWidth();
+            float firstHorizontalTextureCoords;
+            float secondHorizontalTextureCoords;
+            if (arcCircleCenter == null) {
+              firstHorizontalTextureCoords = (float)Point2D.distance(textureReferencePoint[0], textureReferencePoint[1], 
+                  points [index][0], points [index][1]) / texture.getWidth();
+              secondHorizontalTextureCoords = (float)Point2D.distance(textureReferencePoint[0], textureReferencePoint[1], 
+                  points [nextIndex][0], points [nextIndex][1]) / texture.getWidth();
+            } else {
+              if (pointUCoordinates [index] == null) {
+                float pointAngle = (float)Math.atan2(points [index][1] - arcCircleCenter [1], points [index][0] - arcCircleCenter [0]);
+                pointAngle = adjustAngleOnReferencePointAngle(pointAngle, referencePointAngle, arcExtent);
+                pointUCoordinates [index] = (pointAngle - referencePointAngle) * arcCircleRadius / texture.getWidth();
+              }
+              if (pointUCoordinates [nextIndex] == null) {
+                float pointAngle = (float)Math.atan2(points [nextIndex][1] - arcCircleCenter [1], points [nextIndex][0] - arcCircleCenter [0]);
+                pointAngle = adjustAngleOnReferencePointAngle(pointAngle, referencePointAngle, arcExtent);
+                pointUCoordinates [nextIndex] = (pointAngle - referencePointAngle) * arcCircleRadius / texture.getWidth();
+              }
+              
+              firstHorizontalTextureCoords = pointUCoordinates [index];
+              secondHorizontalTextureCoords = pointUCoordinates [nextIndex];
+            }
             textureCoords [j++] = new TexCoord2f(firstHorizontalTextureCoords, yMinTextureCoords);
             textureCoords [j++] = new TexCoord2f(secondHorizontalTextureCoords, yMinTextureCoords);
-            textureCoords [j++] = new TexCoord2f(secondHorizontalTextureCoords, top [i + 1].y / texture.getHeight());
-            textureCoords [j++] = new TexCoord2f(firstHorizontalTextureCoords, top [i].y / texture.getHeight());
+            textureCoords [j++] = new TexCoord2f(secondHorizontalTextureCoords, top [nextIndex].y / texture.getHeight());
+            textureCoords [j++] = new TexCoord2f(firstHorizontalTextureCoords, top [index].y / texture.getHeight());
           } else {
-            float horizontalTextureCoords = (float)Point2D.distance(points[i][0], points[i][1], 
-                points[i + 1][0], points[i + 1][1]) / texture.getWidth();
+            float horizontalTextureCoords = (float)Point2D.distance(points [index][0], points [index][1], 
+                points [nextIndex][0], points [nextIndex][1]) / texture.getWidth();
             textureCoords [j++] = firstTextureCoords;
             textureCoords [j++] = new TexCoord2f(horizontalTextureCoords, yMinTextureCoords);
-            textureCoords [j++] = new TexCoord2f(horizontalTextureCoords, top [i + 1].y / texture.getHeight());
-            textureCoords [j++] = new TexCoord2f(0, top [i].y / texture.getHeight());
+            textureCoords [j++] = new TexCoord2f(horizontalTextureCoords, top [nextIndex].y / texture.getHeight());
+            textureCoords [j++] = new TexCoord2f(0, top [index].y / texture.getHeight());
           }
         }
       }
-      if (usedRectangle [usedRectangle.length - 1]) {
-        if (Math.abs(distanceSqToWallMiddle [0] - halfThicknessSq) < 0.0001
-            && Math.abs(distanceSqToWallMiddle [points.length - 1] - halfThicknessSq) < 0.0001) {
-          float firstHorizontalTextureCoords = (float)Point2D.distance(textureReferencePoint[0], textureReferencePoint[1], 
-              points[points.length - 1][0], points[points.length - 1][1]) / texture.getWidth();
-          float secondHorizontalTextureCoords = (float)Point2D.distance(textureReferencePoint[0], textureReferencePoint[1], 
-              points[0][0], points[0][1]) / texture.getWidth();
-          textureCoords [j++] = new TexCoord2f(firstHorizontalTextureCoords, yMinTextureCoords);
-          textureCoords [j++] = new TexCoord2f(secondHorizontalTextureCoords, yMinTextureCoords);
-          textureCoords [j++] = new TexCoord2f(secondHorizontalTextureCoords, top [0].y / texture.getHeight());
-          textureCoords [j++] = new TexCoord2f(firstHorizontalTextureCoords, top [top.length - 1].y / texture.getHeight());
-        } else {
-          float horizontalTextureCoords = (float)Point2D.distance(points[0][0], points[0][1], 
-              points[points.length - 1][0], points[points.length - 1][1]) / texture.getWidth();
-          textureCoords [j++] = firstTextureCoords;
-          textureCoords [j++] = new TexCoord2f(horizontalTextureCoords, yMinTextureCoords);
-          textureCoords [j++] = new TexCoord2f(horizontalTextureCoords, top [0].y / texture.getHeight());
-          textureCoords [j++] = new TexCoord2f(0, top [top.length - 1].y / texture.getHeight());
-        }
-      }
       geometryInfo.setTextureCoordinateParams(1, 2);
       geometryInfo.setTextureCoordinates(0, textureCoords);
     }
     
     // Generate normals
-    new NormalGenerator(0).generateNormals(geometryInfo);
+    NormalGenerator normalGenerator = new NormalGenerator();
+    if (arcCircleCenter == null) {
+      normalGenerator.setCreaseAngle(0);
+    }
+    normalGenerator.generateNormals(geometryInfo);
     return geometryInfo.getIndexedGeometryArray();
   }
 
+  /**
+   * Returns <code>pointAngle</code> plus or minus 2 PI to ensure <code>pointAngle</code> value 
+   * will be greater or lower than <code>referencePointAngle</code> depending on <code>arcExtent</code> direction.
+   */
+  private float adjustAngleOnReferencePointAngle(float pointAngle, float referencePointAngle, float arcExtent) {
+    if (arcExtent > 0) {
+      if ((referencePointAngle > 0 
+          && (pointAngle < 0
+              || referencePointAngle > pointAngle))
+        || (referencePointAngle < 0 
+            && pointAngle < 0 
+            && referencePointAngle > pointAngle)) {
+        pointAngle += 2 * (float)Math.PI;
+      }
+    } else {
+      if ((referencePointAngle < 0 
+            && (pointAngle > 0
+                || referencePointAngle < pointAngle))
+          || (referencePointAngle > 0 
+              && pointAngle > 0 
+              && referencePointAngle < pointAngle)) {
+        pointAngle -= 2 * (float)Math.PI;
+      }
+    }
+    return pointAngle;
+  }
+
   /**
    * Returns the geometry of an horizontal part of a wall at <code>y</code>.
    */
-  private Geometry createWallHorizontalPartGeometry(float [][] points, float y) {
+  private Geometry createWallHorizontalPartGeometry(float [][] points, float y, 
+                                                    boolean reverseOrder, boolean roundWall) {
     Point3f [] coords = new Point3f [points.length];
     for (int i = 0; i < points.length; i++) {
-      coords [i] = new Point3f(points[i][0], y, points[i][1]);
+      coords [i] = new Point3f(points [i][0], y, points [i][1]);
     }
     GeometryInfo geometryInfo = new GeometryInfo(GeometryInfo.POLYGON_ARRAY);
     geometryInfo.setCoordinates (coords);
     geometryInfo.setStripCounts(new int [] {coords.length});
+    if (reverseOrder) {
+      geometryInfo.reverse();
+    }
     // Generate normals
-    new NormalGenerator(0).generateNormals(geometryInfo);
+    NormalGenerator normalGenerator = new NormalGenerator();
+    if (roundWall) {
+      normalGenerator.setCreaseAngle(0);
+    }
+    normalGenerator.generateNormals(geometryInfo);
     return geometryInfo.getIndexedGeometryArray ();
   }
   
@@ -573,18 +629,23 @@ public class Wall3D extends Object3DBranch {
    */
   private Geometry createWallTopPartGeometry(float [][] points, 
                                              double cosWallYawAngle, double sinWallYawAngle, 
-                                             double topLineAlpha, double topLineBeta) {
+                                             double topLineAlpha, double topLineBeta, 
+                                             boolean roundWall) {
     Point3f [] coords = new Point3f [points.length];
     for (int i = 0; i < points.length; i++) {
-      double xTopPointWithZeroYaw = cosWallYawAngle * points[i][0] + sinWallYawAngle * points[i][1];
+      double xTopPointWithZeroYaw = cosWallYawAngle * points [i][0] + sinWallYawAngle * points [i][1];
       float topY = (float)(topLineAlpha * xTopPointWithZeroYaw + topLineBeta);
-      coords [i] = new Point3f(points[i][0], topY, points[i][1]);
+      coords [i] = new Point3f(points [i][0], topY, points [i][1]);
     }
     GeometryInfo geometryInfo = new GeometryInfo(GeometryInfo.POLYGON_ARRAY);
     geometryInfo.setCoordinates (coords);
     geometryInfo.setStripCounts(new int [] {coords.length});
     // Generate normals
-    new NormalGenerator(0).generateNormals(geometryInfo);
+    NormalGenerator normalGenerator = new NormalGenerator();
+    if (roundWall) {
+      normalGenerator.setCreaseAngle(0);
+    }
+    normalGenerator.generateNormals(geometryInfo);
     return geometryInfo.getIndexedGeometryArray ();
   }
   
@@ -620,9 +681,9 @@ public class Wall3D extends Object3DBranch {
   private void updateWallAppearance(boolean waitTextureLoadingEnd) {
     Wall wall = (Wall)getUserData();
     updateFilledWallSideAppearance(((Shape3D)getChild(LEFT_WALL_SIDE)).getAppearance(), 
-        wall.getLeftSideTexture(), waitTextureLoadingEnd, wall.getLeftSideColor());
+        wall.getLeftSideTexture(), waitTextureLoadingEnd, wall.getLeftSideColor(), wall.getLeftSideShininess());
     updateFilledWallSideAppearance(((Shape3D)getChild(RIGHT_WALL_SIDE)).getAppearance(), 
-        wall.getRightSideTexture(), waitTextureLoadingEnd, wall.getRightSideColor());
+        wall.getRightSideTexture(), waitTextureLoadingEnd, wall.getRightSideColor(), wall.getRightSideShininess());
     if (numChildren() > 2) {
       updateOutlineWallSideAppearance(((Shape3D)getChild(LEFT_WALL_SIDE + 2)).getAppearance());
       updateOutlineWallSideAppearance(((Shape3D)getChild(RIGHT_WALL_SIDE + 2)).getAppearance());
@@ -631,17 +692,19 @@ public class Wall3D extends Object3DBranch {
   
   /**
    * Sets filled wall side appearance with its color, texture, transparency and visibility.
+   * @param f 
    */
   private void updateFilledWallSideAppearance(final Appearance wallSideAppearance, 
                                               final HomeTexture wallSideTexture,
                                               boolean waitTextureLoadingEnd,
-                                              Integer wallSideColor) {
+                                              Integer wallSideColor, 
+                                              float shininess) {
     if (wallSideTexture == null) {
-      wallSideAppearance.setMaterial(getMaterial(wallSideColor));
+      wallSideAppearance.setMaterial(getMaterial(wallSideColor, wallSideColor, shininess));
       wallSideAppearance.setTexture(null);
     } else {
       // Update material and texture of wall side
-      wallSideAppearance.setMaterial(DEFAULT_MATERIAL);
+      wallSideAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_AMBIENT_COLOR, shininess));
       final TextureManager textureManager = TextureManager.getInstance();
       textureManager.loadTexture(wallSideTexture.getImage(), waitTextureLoadingEnd,
           new TextureManager.TextureObserver() {
@@ -677,24 +740,6 @@ public class Wall3D extends Object3DBranch {
         || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE);
   }
   
-  private Material getMaterial(Integer color) {
-    if (color != null) {
-      Material material = materials.get(color); 
-      if (material == null) {
-        Color3f materialColor = new Color3f(((color >>> 16) & 0xFF) / 256f,
-                                            ((color >>> 8) & 0xFF) / 256f,
-                                                    (color & 0xFF) / 256f);
-        material = new Material(materialColor, new Color3f(), materialColor, new Color3f(), 1);
-        material.setCapability(Material.ALLOW_COMPONENT_READ);
-        // Store created materials in cache
-        materials.put(color, material);
-      }
-      return material;
-    } else {
-      return DEFAULT_MATERIAL;
-    }
-  }
-  
   /**
    * An area used to compute holes in walls. 
    */
