@@ -44,7 +44,7 @@ public class Home implements Serializable, Cloneable {
    * in <code>Home</code> class or in one of the classes that it uses,
    * this number is increased.
    */
-  public static final long CURRENT_VERSION = 2600;
+  public static final long CURRENT_VERSION = 3000;
   
   private static final boolean KEEP_BACKWARD_COMPATIBLITY = true;
   
@@ -54,7 +54,7 @@ public class Home implements Serializable, Cloneable {
    */
   public enum Property {NAME, MODIFIED,
     FURNITURE_SORTED_PROPERTY, FURNITURE_DESCENDING_SORTED, FURNITURE_VISIBLE_PROPERTIES,    
-    BACKGROUND_IMAGE, CAMERA, PRINT, BASE_PLAN_LOCKED};
+    BACKGROUND_IMAGE, CAMERA, PRINT, BASE_PLAN_LOCKED, STORED_CAMERAS, RECOVERED};
   
   private List<HomePieceOfFurniture>                  furniture;
   private transient CollectionChangeSupport<HomePieceOfFurniture> furnitureChangeSupport;
@@ -72,9 +72,11 @@ public class Home implements Serializable, Cloneable {
   private String                                      name;
   private float                                       wallHeight;
   private transient boolean                           modified;
+  private transient boolean                           recovered;
   private BackgroundImage                             backgroundImage;
   private ObserverCamera                              observerCamera;
   private Camera                                      topCamera;
+  private List<Camera>                                storedCameras;  
   private HomeEnvironment                             environment;
   private HomePrint                                   print;
   private String                                      furnitureSortedPropertyName;
@@ -84,6 +86,7 @@ public class Home implements Serializable, Cloneable {
   private transient PropertyChangeSupport             propertyChangeSupport;
   private long                                        version;
   private boolean                                     basePlanLocked; 
+  private Compass                                     compass;
   // The 5 following environment fields are still declared for compatibility reasons
   private int                                         skyColor;
   private int                                         groundColor;
@@ -134,7 +137,7 @@ public class Home implements Serializable, Cloneable {
         HomePieceOfFurniture.SortableProperty.HEIGHT,
         HomePieceOfFurniture.SortableProperty.VISIBLE});
     // Init transient lists and other fields
-    init();
+    init(true);
   }
 
   /**
@@ -142,7 +145,7 @@ public class Home implements Serializable, Cloneable {
    * and reads home from <code>in</code> stream with default reading method.
    */
   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
-    init();
+    init(false);
     in.defaultReadObject();
     
     if (KEEP_BACKWARD_COMPATIBLITY) {
@@ -193,7 +196,7 @@ public class Home implements Serializable, Cloneable {
     }
   }
 
-  private void init() {
+  private void init(boolean newHome) {
     // Initialize transient lists
     this.selectedItems = new ArrayList<Selectable>();
     this.furnitureChangeSupport = new CollectionChangeSupport<HomePieceOfFurniture>(this);
@@ -222,11 +225,15 @@ public class Home implements Serializable, Cloneable {
     // Create a default observer camera (use a 63 field of view equivalent to a 35mm lens for a 24x36 film)
     this.observerCamera = new ObserverCamera(50, 50, 170, 
         7 * (float)Math.PI / 4, (float)Math.PI / 16, (float)Math.PI * 63 / 180);
+    this.storedCameras = Collections.emptyList();
     // Initialize new fields 
     this.environment = new HomeEnvironment();
     this.rooms = new ArrayList<Room>();
     this.dimensionLines = new ArrayList<DimensionLine>();
     this.labels = new ArrayList<Label>();
+    this.compass = new Compass(-100, 50, 100);
+    // Let compass be visible only on new homes
+    this.compass.setVisible(newHome);
     this.visualProperties = new HashMap<String, Object>();
     
     this.version = CURRENT_VERSION;
@@ -770,6 +777,26 @@ public class Home implements Serializable, Cloneable {
     }
   }
   
+  /**
+   * Returns whether this home was recovered or not.
+   * @since 3.0
+   */
+  public boolean isRecovered() {
+    return this.recovered;
+  }
+
+  /**
+   * Sets whether this home was recovered or not and fires a <code>PropertyChangeEvent</code>.
+   * @since 3.0
+   */
+  public void setRecovered(boolean recovered) {
+    if (recovered != this.recovered) {
+      this.recovered = recovered;
+      this.propertyChangeSupport.firePropertyChange(
+          Property.RECOVERED.name(), !recovered, recovered);
+    }
+  }
+  
   /**
    * Returns the furniture property on which home is sorted or <code>null</code> if
    * home furniture isn't sorted.
@@ -891,6 +918,32 @@ public class Home implements Serializable, Cloneable {
     }
     return this.camera;
   }
+  
+  /**
+   * Sets the cameras stored by this home and fires a <code>PropertyChangeEvent</code>.
+   * The list given as parameter is cloned but not the camera instances it contains.
+   * @since 3.0
+   */
+  public void setStoredCameras(List<Camera> storedCameras) {
+    if (!this.storedCameras.equals(storedCameras)) {
+      List<Camera> oldStoredCameras = this.storedCameras;
+      if (storedCameras == null) {
+        this.storedCameras = Collections.emptyList();
+      } else {
+        this.storedCameras = new ArrayList<Camera>(storedCameras);
+      }
+      this.propertyChangeSupport.firePropertyChange(
+          Property.STORED_CAMERAS.name(), Collections.unmodifiableList(oldStoredCameras), Collections.unmodifiableList(storedCameras));
+    }
+  }
+
+  /**
+   * Returns an unmodifiable list of the cameras stored by this home.
+   * @since 3.0
+   */
+  public List<Camera> getStoredCameras() {
+    return Collections.unmodifiableList(this.storedCameras);
+  }
 
   /**
    * Returns the environment attributes of this home.
@@ -899,6 +952,14 @@ public class Home implements Serializable, Cloneable {
     return this.environment;
   }
 
+  /**
+   * Returns the compass associated to this home.
+   * @since 3.0
+   */
+  public Compass getCompass() {
+    return this.compass;
+  }
+  
   /**
    * Returns the print attributes of this home.
    */
@@ -1003,8 +1064,13 @@ public class Home implements Serializable, Cloneable {
       } else {
         clone.camera = clone.topCamera;
       }
+      clone.storedCameras = new ArrayList<Camera>(this.storedCameras.size());
+      for (Camera camera : this.storedCameras) {
+        clone.storedCameras.add(camera.clone());
+      }
       // Clone other mutable objects
       clone.environment = this.environment.clone();
+      clone.compass = this.compass.clone();
       clone.furnitureVisibleProperties = new ArrayList<HomePieceOfFurniture.SortableProperty>(
           this.furnitureVisibleProperties);
       clone.visualProperties = new HashMap<String, Object>(this.visualProperties);
@@ -1049,7 +1115,8 @@ public class Home implements Serializable, Cloneable {
     List<Selectable> list = new ArrayList<Selectable>();
     for (Selectable item : items) {
       if (!(item instanceof Wall)         // Walls are copied further
-          && !(item instanceof Camera)) { // Cameras can't be duplicated
+          && !(item instanceof Camera)    // Cameras and compass can't be duplicated
+          && !(item instanceof Compass)) { 
         list.add(item.clone());
       }
     }
