@@ -23,6 +23,7 @@ import java.awt.Shape;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.PathIterator;
+import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
@@ -50,7 +51,7 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
    * to a piece of furniture will be notified under a property name equal to the string value of one these properties.
    */
   public enum Property {NAME, NAME_VISIBLE, NAME_X_OFFSET, NAME_Y_OFFSET, NAME_STYLE,
-      DESCRIPTION, WIDTH, DEPTH, HEIGHT, COLOR, TEXTURE, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_MIRRORED};
+      DESCRIPTION, WIDTH, DEPTH, HEIGHT, COLOR, TEXTURE, SHININESS, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_MIRRORED, MOVABLE};
   
   /** 
    * The properties on which home furniture may be sorted.  
@@ -216,9 +217,12 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   private boolean                doorOrWindow;
   private Integer                color;
   private HomeTexture            texture;
+  private Float                  shininess;
   private float [][]             modelRotation;
   private boolean                backFaceShown;
   private boolean                resizable;
+  private boolean                deformable;
+  private boolean                texturable;
   private BigDecimal             price;
   private BigDecimal             valueAddedTaxPercentage;
   private boolean                visible;
@@ -230,7 +234,6 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
   private transient Shape shapeCache;
 
-
   /**
    * Creates a home piece of furniture from an existing piece.
    * @param piece the piece from which data are copied
@@ -251,6 +254,8 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     this.modelRotation = piece.getModelRotation();
     this.backFaceShown = piece.isBackFaceShown();
     this.resizable = piece.isResizable();
+    this.deformable = piece.isDeformable();
+    this.texturable = piece.isTexturable();
     this.price = piece.getPrice();
     this.valueAddedTaxPercentage = piece.getValueAddedTaxPercentage();
     if (piece instanceof HomePieceOfFurniture) {
@@ -284,6 +289,8 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
     this.modelRotation = IDENTITY;
     this.resizable = true;
+    this.deformable = true;
+    this.texturable = true;
     this.propertyChangeSupport = new PropertyChangeSupport(this);
     in.defaultReadObject();
   }
@@ -527,6 +534,13 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     return this.movable;
   }
 
+  public void setMovable(boolean movable) {
+    if (movable != this.movable) {
+      this.movable = movable;
+      this.propertyChangeSupport.firePropertyChange(Property.MOVABLE.name(), !movable, movable);
+    }
+  }
+  
   /**
    * Returns <code>true</code> if this piece of furniture is a door or a window.
    * As this method existed before {@linkplain HomeDoorOrWindow HomeDoorOrWindow} class,
@@ -570,13 +584,18 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   /**
    * Sets the color of this piece of furniture or <code>null</code> if piece color is unchanged. 
    * Once this piece is updated, listeners added to this piece will receive a change notification.
+   * @throws IllegalStateException if this piece of furniture isn't texturable
    */
   public void setColor(Integer color) {
-    if (color != this.color
-        || (color != null && !color.equals(this.color))) {
-      Integer oldColor = this.color;
-      this.color = color;
-      this.propertyChangeSupport.firePropertyChange(Property.COLOR.name(), oldColor, color);
+    if (isTexturable()) {
+      if (color != this.color
+          || (color != null && !color.equals(this.color))) {
+        Integer oldColor = this.color;
+        this.color = color;
+        this.propertyChangeSupport.firePropertyChange(Property.COLOR.name(), oldColor, color);
+      }
+    } else {
+      throw new IllegalStateException("Piece isn't texturable");
     }
   }
 
@@ -592,14 +611,47 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   /**
    * Sets the texture of this piece of furniture or <code>null</code> if piece texture is unchanged. 
    * Once this piece is updated, listeners added to this piece will receive a change notification.
+   * @throws IllegalStateException if this piece of furniture isn't texturable
    * @since 2.3
    */
   public void setTexture(HomeTexture texture) {
-    if (texture != this.texture
-        || (texture != null && !texture.equals(this.texture))) {
-      HomeTexture oldTexture = this.texture;
-      this.texture = texture;
-      this.propertyChangeSupport.firePropertyChange(Property.TEXTURE.name(), oldTexture, texture);
+    if (isTexturable()) {
+      if (texture != this.texture
+          || (texture != null && !texture.equals(this.texture))) {
+        HomeTexture oldTexture = this.texture;
+        this.texture = texture;
+        this.propertyChangeSupport.firePropertyChange(Property.TEXTURE.name(), oldTexture, texture);
+      }
+    } else {
+      throw new IllegalStateException("Piece isn't texturable");
+    }
+  }
+
+  /**
+   * Returns the shininess of this piece of furniture.
+   * @return a value between 0 (matt) and 1 (very shiny) or <code>null</code> if piece shininess is unchanged.
+   * @since 3.0
+   */
+  public Float getShininess() {
+    return this.shininess;
+  }
+  
+  /**
+   * Sets the shininess of this piece of furniture or <code>null</code> if piece shininess is unchanged. 
+   * Once this piece is updated, listeners added to this piece will receive a change notification.
+   * @throws IllegalStateException if this piece of furniture isn't texturable
+   * @since 3.0
+   */
+  public void setShininess(Float shininess) {
+    if (isTexturable()) {
+      if (shininess != this.shininess
+          || (shininess != null && !shininess.equals(this.shininess))) {
+        Float oldShininess = this.shininess;
+        this.shininess = shininess;
+        this.propertyChangeSupport.firePropertyChange(Property.SHININESS.name(), oldShininess, shininess);
+      }
+    } else {
+      throw new IllegalStateException("Piece isn't texturable");
     }
   }
 
@@ -610,6 +662,22 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     return this.resizable;    
   }
   
+  /**
+   * Returns <code>true</code> if this piece is deformable.
+   * @since 3.0
+   */
+  public boolean isDeformable() {
+    return this.deformable;    
+  }
+
+  /**
+   * Returns <code>false</code> if this piece should always keep the same color or texture.
+   * @since 3.0
+   */
+  public boolean isTexturable() {
+    return this.texturable;
+  }
+
   /**
    * Returns the price of this piece of furniture or <code>null</code>. 
    */
@@ -824,38 +892,54 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
 
   /**
    * Returns <code>true</code> if the top left point of this piece is 
-   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>.
+   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>, 
+   * and if that point is closer to top left point than to top right and bottom left points.
    */
   public boolean isTopLeftPointAt(float x, float y, float margin) {
     float [][] points = getPoints();
-    return Math.abs(x - points[0][0]) <= margin && Math.abs(y - points[0][1]) <= margin;
+    double distanceSquareToTopLeftPoint = Point2D.distanceSq(x, y, points[0][0], points[0][1]);
+    return distanceSquareToTopLeftPoint <= margin * margin
+        && distanceSquareToTopLeftPoint < Point2D.distanceSq(x, y, points[1][0], points[1][1])
+        && distanceSquareToTopLeftPoint < Point2D.distanceSq(x, y, points[3][0], points[3][1]);
   }
 
   /**
    * Returns <code>true</code> if the top right point of this piece is 
-   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>.
+   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>, 
+   * and if that point is closer to top right point than to top left and bottom right points.
    */
   public boolean isTopRightPointAt(float x, float y, float margin) {
     float [][] points = getPoints();
-    return Math.abs(x - points[1][0]) <= margin && Math.abs(y - points[1][1]) <= margin;
+    double distanceSquareToTopRightPoint = Point2D.distanceSq(x, y, points[1][0], points[1][1]);
+    return distanceSquareToTopRightPoint <= margin * margin
+        && distanceSquareToTopRightPoint < Point2D.distanceSq(x, y, points[0][0], points[0][1])
+        && distanceSquareToTopRightPoint < Point2D.distanceSq(x, y, points[2][0], points[2][1]);
   }
 
   /**
    * Returns <code>true</code> if the bottom left point of this piece is 
-   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>.
+   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>,
+   * and if that point is closer to bottom left point than to top left and bottom right points.
    */
   public boolean isBottomLeftPointAt(float x, float y, float margin) {
     float [][] points = getPoints();
-    return Math.abs(x - points[3][0]) <= margin && Math.abs(y - points[3][1]) <= margin;
+    double distanceSquareToBottomLeftPoint = Point2D.distanceSq(x, y, points[3][0], points[3][1]);
+    return distanceSquareToBottomLeftPoint <= margin * margin
+        && distanceSquareToBottomLeftPoint < Point2D.distanceSq(x, y, points[0][0], points[0][1])
+        && distanceSquareToBottomLeftPoint < Point2D.distanceSq(x, y, points[2][0], points[2][1]);
   }
 
   /**
    * Returns <code>true</code> if the bottom right point of this piece is 
-   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>.
+   * the point at (<code>x</code>, <code>y</code>) with a given <code>margin</code>, 
+   * and if that point is closer to top left point than to top right and bottom left points.
    */
   public boolean isBottomRightPointAt(float x, float y, float margin) {
     float [][] points = getPoints();
-    return Math.abs(x - points[2][0]) <= margin && Math.abs(y - points[2][1]) <= margin;
+    double distanceSquareToBottomRightPoint = Point2D.distanceSq(x, y, points[2][0], points[2][1]);
+    return distanceSquareToBottomRightPoint <= margin * margin
+        && distanceSquareToBottomRightPoint < Point2D.distanceSq(x, y, points[1][0], points[1][1])
+        && distanceSquareToBottomRightPoint < Point2D.distanceSq(x, y, points[3][0], points[3][1]);
   }
 
   /**
