@@ -47,6 +47,7 @@ import java.awt.event.MouseMotionAdapter;
 import java.awt.event.MouseWheelEvent;
 import java.awt.event.MouseWheelListener;
 import java.awt.geom.Area;
+import java.awt.geom.GeneralPath;
 import java.awt.geom.PathIterator;
 import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
@@ -60,6 +61,7 @@ import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -84,7 +86,6 @@ import javax.media.j3d.IllegalRenderingStateException;
 import javax.media.j3d.J3DGraphics2D;
 import javax.media.j3d.Light;
 import javax.media.j3d.Node;
-import javax.media.j3d.OrderedGroup;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.Texture;
 import javax.media.j3d.Transform3D;
@@ -150,7 +151,8 @@ import com.sun.j3d.utils.universe.ViewingPlatform;
 public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d.viewcontroller.View, Printable {
   private enum ActionType {MOVE_CAMERA_FORWARD, MOVE_CAMERA_FAST_FORWARD, MOVE_CAMERA_BACKWARD, MOVE_CAMERA_FAST_BACKWARD,  
       ROTATE_CAMERA_YAW_LEFT, ROTATE_CAMERA_YAW_FAST_LEFT, ROTATE_CAMERA_YAW_RIGHT, ROTATE_CAMERA_YAW_FAST_RIGHT, 
-      ROTATE_CAMERA_PITCH_UP, ROTATE_CAMERA_PITCH_DOWN, ELEVATE_CAMERA_UP, ELEVATE_CAMERA_DOWN}
+      ROTATE_CAMERA_PITCH_UP, ROTATE_CAMERA_PITCH_DOWN, 
+      ELEVATE_CAMERA_UP, ELEVATE_CAMERA_FAST_UP, ELEVATE_CAMERA_DOWN, ELEVATE_CAMERA_FAST_DOWN}
   
   private final Home                               home;
   private final boolean                            displayShadowOnFloor;
@@ -379,7 +381,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
         new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-previous.png"))),
         new GridBagConstraints(0, 1, 1, 2, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 5, 0, 0, 
+    navigationPanel.add(new NavigationButton(controller, 12.5f, 0, 0, 
         new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-up.png"))),
         new GridBagConstraints(1, 0, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 0, 0), 0, 0));
@@ -387,15 +389,15 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
         new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-next.png"))),
         new GridBagConstraints(2, 1, 1, 2, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 2), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, -5, 0, 0, 
+    navigationPanel.add(new NavigationButton(controller, -12.5f, 0, 0, 
         new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-down.png"))),
         new GridBagConstraints(1, 3, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 0, 0, -(float)Math.PI / 72, 
+    navigationPanel.add(new NavigationButton(controller, 0, 0, -(float)Math.PI / 100, 
         new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-up-small.png"))),
         new GridBagConstraints(1, 1, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 0, 0, (float)Math.PI / 72, 
+    navigationPanel.add(new NavigationButton(controller, 0, 0, (float)Math.PI / 100, 
         new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-down-small.png"))),
         new GridBagConstraints(1, 2, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 1, 0), 0, 0));
@@ -451,8 +453,8 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
       // Create a timer that will update camera angles and location
       final Timer timer = new Timer(50, new ActionListener() {
           public void actionPerformed(ActionEvent ev) {
-            controller.moveCamera(shiftDown ? moveDelta / 10 : moveDelta);
-            controller.rotateCameraYaw(shiftDown ? yawDelta / 10 : yawDelta);
+            controller.moveCamera(shiftDown ? moveDelta : moveDelta / 5);
+            controller.rotateCameraYaw(shiftDown ? yawDelta : yawDelta / 5);
             controller.rotateCameraPitch(pitchDelta);
           }
         });
@@ -668,6 +670,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
    * Returns an image of the home viewed by this component at the given size.
    */
   public BufferedImage getOffScreenImage(int width, int height) {
+    List<Selectable> selectedItems = this.home.getSelectedItems();
     SimpleUniverse offScreenImageUniverse = null;
     try {
       View view;
@@ -677,9 +680,14 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
       } else {
         view = this.universe.getViewer().getView();
       }
-   
+      
+      // Empty temporarily selection to create the off screen image
+      List<Selectable> emptySelection = Collections.emptyList();
+      this.home.setSelectedItems(emptySelection);
       return Component3DManager.getInstance().getOffScreenImage(view, width, height);
     } finally {
+      // Restore selection
+      this.home.setSelectedItems(selectedItems);
       if (offScreenImageUniverse != null) {
         offScreenImageUniverse.cleanup();
       } 
@@ -736,19 +744,18 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
     }
     view.setFieldOfView(fieldOfView);
     double frontClipDistance;
-    // Use a variable front clip distance for camera 
-    // depending on the elevation (at an elevation higher than 3 m
-    // back clip distance must be greater than 6000 or a black zone appears 
-    // at the horizon in off screen images)
     if (topCamera) {
-      frontClipDistance = 5;
-      if (camera.getZ() > 750) {
-        frontClipDistance += (camera.getZ() - 750) / 150;
-      }
+      // Use a variable front clip distance for top camera 
+      // depending on the distance to scene center      
+      frontClipDistance = 1 + Math.abs(camera.getZ() / Math.sin(camera.getPitch())) / 50;
     } else {
-      frontClipDistance = 2;
-      if (camera.getZ() > 300) {
-        frontClipDistance += (camera.getZ() - 300) / 150;
+      // Use a variable front clip distance for observer camera 
+      // depending on the elevation (at an elevation higher than 3 m
+      // back clip distance must be greater than 6000 or a black zone appears 
+      // at the horizon in off screen images)
+      frontClipDistance = 2.5;
+      if (camera.getZ() > 200) {
+        frontClipDistance += (camera.getZ() - 200) / 50;
       }
     }
     // Update front and back clip distance to ensure their ratio is less than 3000
@@ -935,19 +942,19 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
             if (isEnabled()) {
               if (ev.isAltDown()) {
                 // Mouse move along Y axis while alt is down changes camera location
-                float delta = 0.5f * (this.yLastMouseMove - ev.getY());
-                // Multiply delta by 10 if shift isn't down
-                if (!ev.isShiftDown()) {
-                  delta *= 10;
+                float delta = 1.25f * (this.yLastMouseMove - ev.getY());
+                // Multiply delta by 10 if shift is down
+                if (ev.isShiftDown()) {
+                  delta *= 5;
                 } 
                 controller.moveCamera(delta);
               } else {
                 final float ANGLE_FACTOR = 0.005f;
                 // Mouse move along X axis changes camera yaw 
                 float yawDelta = ANGLE_FACTOR * (ev.getX() - this.xLastMouseMove);
-                // Multiply yaw delta by 10 if shift isn't down
-                if (!ev.isShiftDown()) {
-                  yawDelta *= 10;
+                // Multiply yaw delta by 10 if shift is down
+                if (ev.isShiftDown()) {
+                  yawDelta *= 5;
                 } 
                 controller.rotateCameraYaw(yawDelta);
                 
@@ -1054,10 +1061,10 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
         public void mouseWheelMoved(MouseWheelEvent ev) {
           if (isEnabled()) {
             // Mouse wheel changes camera location 
-            float delta = -ev.getWheelRotation();
-            // Multiply delta by 10 if shift isn't down
-            if (!ev.isShiftDown()) {
-              delta *= 10;
+            float delta = -2.5f * ev.getWheelRotation();
+            // Multiply delta by 10 if shift is down
+            if (ev.isShiftDown()) {
+              delta *= 5;
             } 
             controller.moveCamera(delta);
           }
@@ -1081,25 +1088,27 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
    */
   private void installKeyboardActions() {
     InputMap inputMap = getInputMap(WHEN_FOCUSED);
-    inputMap.put(KeyStroke.getKeyStroke("shift UP"), ActionType.MOVE_CAMERA_FORWARD);
-    inputMap.put(KeyStroke.getKeyStroke("shift W"), ActionType.MOVE_CAMERA_FORWARD);
-    inputMap.put(KeyStroke.getKeyStroke("UP"), ActionType.MOVE_CAMERA_FAST_FORWARD);
-    inputMap.put(KeyStroke.getKeyStroke("W"), ActionType.MOVE_CAMERA_FAST_FORWARD);
-    inputMap.put(KeyStroke.getKeyStroke("shift DOWN"), ActionType.MOVE_CAMERA_BACKWARD);
-    inputMap.put(KeyStroke.getKeyStroke("shift S"), ActionType.MOVE_CAMERA_BACKWARD);
-    inputMap.put(KeyStroke.getKeyStroke("DOWN"), ActionType.MOVE_CAMERA_FAST_BACKWARD);
-    inputMap.put(KeyStroke.getKeyStroke("S"), ActionType.MOVE_CAMERA_FAST_BACKWARD);
-    inputMap.put(KeyStroke.getKeyStroke("shift LEFT"), ActionType.ROTATE_CAMERA_YAW_LEFT);
-    inputMap.put(KeyStroke.getKeyStroke("shift A"), ActionType.ROTATE_CAMERA_YAW_LEFT);
-    inputMap.put(KeyStroke.getKeyStroke("LEFT"), ActionType.ROTATE_CAMERA_YAW_FAST_LEFT);
-    inputMap.put(KeyStroke.getKeyStroke("A"), ActionType.ROTATE_CAMERA_YAW_FAST_LEFT);
-    inputMap.put(KeyStroke.getKeyStroke("shift RIGHT"), ActionType.ROTATE_CAMERA_YAW_RIGHT);
-    inputMap.put(KeyStroke.getKeyStroke("shift D"), ActionType.ROTATE_CAMERA_YAW_RIGHT);
-    inputMap.put(KeyStroke.getKeyStroke("RIGHT"), ActionType.ROTATE_CAMERA_YAW_FAST_RIGHT);
-    inputMap.put(KeyStroke.getKeyStroke("D"), ActionType.ROTATE_CAMERA_YAW_FAST_RIGHT);
+    inputMap.put(KeyStroke.getKeyStroke("shift UP"), ActionType.MOVE_CAMERA_FAST_FORWARD);
+    inputMap.put(KeyStroke.getKeyStroke("shift W"), ActionType.MOVE_CAMERA_FAST_FORWARD);
+    inputMap.put(KeyStroke.getKeyStroke("UP"), ActionType.MOVE_CAMERA_FORWARD);
+    inputMap.put(KeyStroke.getKeyStroke("W"), ActionType.MOVE_CAMERA_FORWARD);
+    inputMap.put(KeyStroke.getKeyStroke("shift DOWN"), ActionType.MOVE_CAMERA_FAST_BACKWARD);
+    inputMap.put(KeyStroke.getKeyStroke("shift S"), ActionType.MOVE_CAMERA_FAST_BACKWARD);
+    inputMap.put(KeyStroke.getKeyStroke("DOWN"), ActionType.MOVE_CAMERA_BACKWARD);
+    inputMap.put(KeyStroke.getKeyStroke("S"), ActionType.MOVE_CAMERA_BACKWARD);
+    inputMap.put(KeyStroke.getKeyStroke("shift LEFT"), ActionType.ROTATE_CAMERA_YAW_FAST_LEFT);
+    inputMap.put(KeyStroke.getKeyStroke("shift A"), ActionType.ROTATE_CAMERA_YAW_FAST_LEFT);
+    inputMap.put(KeyStroke.getKeyStroke("LEFT"), ActionType.ROTATE_CAMERA_YAW_LEFT);
+    inputMap.put(KeyStroke.getKeyStroke("A"), ActionType.ROTATE_CAMERA_YAW_LEFT);
+    inputMap.put(KeyStroke.getKeyStroke("shift RIGHT"), ActionType.ROTATE_CAMERA_YAW_FAST_RIGHT);
+    inputMap.put(KeyStroke.getKeyStroke("shift D"), ActionType.ROTATE_CAMERA_YAW_FAST_RIGHT);
+    inputMap.put(KeyStroke.getKeyStroke("RIGHT"), ActionType.ROTATE_CAMERA_YAW_RIGHT);
+    inputMap.put(KeyStroke.getKeyStroke("D"), ActionType.ROTATE_CAMERA_YAW_RIGHT);
     inputMap.put(KeyStroke.getKeyStroke("PAGE_UP"), ActionType.ROTATE_CAMERA_PITCH_UP);
     inputMap.put(KeyStroke.getKeyStroke("PAGE_DOWN"), ActionType.ROTATE_CAMERA_PITCH_DOWN);
+    inputMap.put(KeyStroke.getKeyStroke("shift HOME"), ActionType.ELEVATE_CAMERA_FAST_UP);
     inputMap.put(KeyStroke.getKeyStroke("HOME"), ActionType.ELEVATE_CAMERA_UP);
+    inputMap.put(KeyStroke.getKeyStroke("shift END"), ActionType.ELEVATE_CAMERA_FAST_DOWN);
     inputMap.put(KeyStroke.getKeyStroke("END"), ActionType.ELEVATE_CAMERA_DOWN);
   }
  
@@ -1109,9 +1118,9 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   private void createActions(final HomeController3D controller) {
     // Move camera action mapped to arrow keys 
     class MoveCameraAction extends AbstractAction {
-      private final int delta;
+      private final float delta;
       
-      public MoveCameraAction(int delta) {
+      public MoveCameraAction(float delta) {
         this.delta = delta;
       }
 
@@ -1156,18 +1165,20 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
       }
     }
     ActionMap actionMap = getActionMap();
-    actionMap.put(ActionType.MOVE_CAMERA_FORWARD, new MoveCameraAction(1));
-    actionMap.put(ActionType.MOVE_CAMERA_FAST_FORWARD, new MoveCameraAction(10));
-    actionMap.put(ActionType.MOVE_CAMERA_BACKWARD, new MoveCameraAction(-1));
-    actionMap.put(ActionType.MOVE_CAMERA_FAST_BACKWARD, new MoveCameraAction(-10));
+    actionMap.put(ActionType.MOVE_CAMERA_FORWARD, new MoveCameraAction(6.5f));
+    actionMap.put(ActionType.MOVE_CAMERA_FAST_FORWARD, new MoveCameraAction(32.5f));
+    actionMap.put(ActionType.MOVE_CAMERA_BACKWARD, new MoveCameraAction(-6.5f));
+    actionMap.put(ActionType.MOVE_CAMERA_FAST_BACKWARD, new MoveCameraAction(-32.5f));
     actionMap.put(ActionType.ELEVATE_CAMERA_DOWN, new ElevateCameraAction(-2.5f));
+    actionMap.put(ActionType.ELEVATE_CAMERA_FAST_DOWN, new ElevateCameraAction(-10f));
     actionMap.put(ActionType.ELEVATE_CAMERA_UP, new ElevateCameraAction(2.5f));
-    actionMap.put(ActionType.ROTATE_CAMERA_YAW_LEFT, new RotateCameraYawAction(-(float)Math.PI / 180));
-    actionMap.put(ActionType.ROTATE_CAMERA_YAW_FAST_LEFT, new RotateCameraYawAction(-(float)Math.PI / 18));
-    actionMap.put(ActionType.ROTATE_CAMERA_YAW_RIGHT, new RotateCameraYawAction((float)Math.PI / 180));
-    actionMap.put(ActionType.ROTATE_CAMERA_YAW_FAST_RIGHT, new RotateCameraYawAction((float)Math.PI / 18));
-    actionMap.put(ActionType.ROTATE_CAMERA_PITCH_UP, new RotateCameraPitchAction(-(float)Math.PI / 180));
-    actionMap.put(ActionType.ROTATE_CAMERA_PITCH_DOWN, new RotateCameraPitchAction((float)Math.PI / 180));
+    actionMap.put(ActionType.ELEVATE_CAMERA_FAST_UP, new ElevateCameraAction(10f));
+    actionMap.put(ActionType.ROTATE_CAMERA_YAW_LEFT, new RotateCameraYawAction(-(float)Math.PI / 60));
+    actionMap.put(ActionType.ROTATE_CAMERA_YAW_FAST_LEFT, new RotateCameraYawAction(-(float)Math.PI / 12));
+    actionMap.put(ActionType.ROTATE_CAMERA_YAW_RIGHT, new RotateCameraYawAction((float)Math.PI / 60));
+    actionMap.put(ActionType.ROTATE_CAMERA_YAW_FAST_RIGHT, new RotateCameraYawAction((float)Math.PI / 12));
+    actionMap.put(ActionType.ROTATE_CAMERA_PITCH_UP, new RotateCameraPitchAction(-(float)Math.PI / 120));
+    actionMap.put(ActionType.ROTATE_CAMERA_PITCH_DOWN, new RotateCameraPitchAction((float)Math.PI / 120));
   }
 
   /**
@@ -1177,49 +1188,16 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
                                       boolean listenToHomeUpdates, 
                                       boolean waitForLoading) {
     BranchGroup root = new BranchGroup();
-
     // Build scene tree
+    root.addChild(createHomeTree(displayShadowOnFloor, listenToHomeUpdates, waitForLoading));
     root.addChild(createBackgroundNode(listenToHomeUpdates));
-    
-    Group groundAndRoomsGroup = createExtensibleGroup(true);
-    // Add first ground then home rooms to an ordered group to ensure ground is always drawn behind floors   
-    groundAndRoomsGroup.addChild(createGroundNode(-1E5f / 2, -1E5f / 2, 1E5f, 1E5f, listenToHomeUpdates, waitForLoading));
-    for (Room room : this.home.getRooms()) {
-      addObject(groundAndRoomsGroup, room, waitForLoading);
-    }    
-    root.addChild(groundAndRoomsGroup);
-    
-    Group wallsAndFurnitureGroup = createExtensibleGroup(false);
-    // Add walls and pieces already available in a normal group 
-    // (transparent shapes don't work well in ordered groups) 
-    for (Wall wall : this.home.getWalls()) {
-      addObject(wallsAndFurnitureGroup, wall, waitForLoading);
-    }
-    for (HomePieceOfFurniture piece : this.home.getFurniture()) {
-      addObject(wallsAndFurnitureGroup, piece, waitForLoading);
-    }
-    if (displayShadowOnFloor) {
-      addShadowOnFloor(groundAndRoomsGroup);
-    }
-    root.addChild(wallsAndFurnitureGroup);
-    
+    root.addChild(createGroundNode(-1E5f / 2, -1E5f / 2, 1E5f, 1E5f, listenToHomeUpdates, waitForLoading));
+
     // Add default lights
     for (Light light : createLights(listenToHomeUpdates)) {
       root.addChild(light);
     }
     
-    if (listenToHomeUpdates) {
-      // Add wall, furniture, room listeners to home for further update    
-      addWallListener(wallsAndFurnitureGroup);
-      addFurnitureListener(wallsAndFurnitureGroup);
-      addRoomListener(groundAndRoomsGroup, 1);
-      // Add environment listeners
-      addEnvironmentListeners();
-      // Should update shadow on floor too but in the facts 
-      // User Interface doesn't propose to modify the furniture of a home
-      // that displays shadow on floor yet
-    }
-
     return root;
   }
 
@@ -1412,18 +1390,54 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   private void updateLightColor(Light light) {
     Color3f defaultColor = (Color3f)light.getUserData();
     int lightColor = this.home.getEnvironment().getLightColor();
-    light.setColor(new Color3f(((lightColor >>> 16) & 0xFF) / 256f * defaultColor.x,
-                                ((lightColor >>> 8) & 0xFF) / 256f * defaultColor.y,
-                                        (lightColor & 0xFF) / 256f * defaultColor.z));
+    light.setColor(new Color3f(((lightColor >>> 16) & 0xFF) / 255f * defaultColor.x,
+                                ((lightColor >>> 8) & 0xFF) / 255f * defaultColor.y,
+                                        (lightColor & 0xFF) / 255f * defaultColor.z));
     clearPrintedImageCache();
   }
+
   /**
-   * Returns a new extensible group.  
+   * Returns a <code>home</code> new tree node, with branches for each wall 
+   * and piece of furniture of <code>home</code>. 
    */
-  private Group createExtensibleGroup(boolean ordered) {
-    Group homeGroup = ordered
-        ? new OrderedGroup()
-        : new Group();    
+  private Node createHomeTree(boolean displayShadowOnFloor, 
+                              boolean listenToHomeUpdates, 
+                              boolean waitForLoading) {
+    Group homeRoot = createHomeRoot();
+    // Add walls, pieces and rooms already available 
+    for (Room room : this.home.getRooms()) {
+      addObject(homeRoot, room, waitForLoading);
+    }    
+    for (Wall wall : this.home.getWalls()) {
+      addObject(homeRoot, wall, waitForLoading);
+    }
+    for (HomePieceOfFurniture piece : this.home.getFurniture()) {
+      addObject(homeRoot, piece, waitForLoading);
+    }
+    
+    if (displayShadowOnFloor) {
+      addShadowOnFloor(homeRoot);
+    }
+    
+    if (listenToHomeUpdates) {
+      // Add wall, furniture, room listeners to home for further update    
+      addWallListener(homeRoot);
+      addFurnitureListener(homeRoot);
+      addRoomListener(homeRoot);
+      // Add environment listeners
+      addEnvironmentListeners();
+      // Should update shadow on floor too but in the facts 
+      // User Interface doesn't propose to modify the furniture of a home
+      // that displays shadow on floor yet
+    }
+    return homeRoot;
+  }
+
+  /**
+   * Returns a new group at home subtree root.
+   */
+  private Group createHomeRoot() {
+    Group homeGroup = new Group();    
     //  Allow group to have new children
     homeGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);
     homeGroup.setCapability(Group.ALLOW_CHILDREN_EXTEND);
@@ -1480,6 +1494,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
             || HomePieceOfFurniture.Property.ELEVATION.name().equals(ev.getPropertyName())
             || HomePieceOfFurniture.Property.COLOR.name().equals(ev.getPropertyName())
             || HomePieceOfFurniture.Property.TEXTURE.name().equals(ev.getPropertyName())
+            || HomePieceOfFurniture.Property.SHININESS.name().equals(ev.getPropertyName())
             || HomePieceOfFurniture.Property.MODEL_MIRRORED.name().equals(ev.getPropertyName())
             || HomePieceOfFurniture.Property.VISIBLE.name().equals(ev.getPropertyName())) {
               HomePieceOfFurniture piece = (HomePieceOfFurniture)ev.getSource();
@@ -1536,11 +1551,57 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
    * Adds a room listener to home rooms that updates the children of the given 
    * <code>group</code>, each time a room is added, updated or deleted. 
    */
-  private void addRoomListener(final Group group, final int indexOffset) {
+  private void addRoomListener(final Group group) {
     this.roomChangeListener = new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent ev) {
-          updateObjects(Arrays.asList(new Room [] {(Room)ev.getSource()}));
-          groundColorAndTextureListener.propertyChange(null);
+          if (Room.Property.FLOOR_COLOR.name().equals(ev.getPropertyName())
+              || Room.Property.FLOOR_TEXTURE.name().equals(ev.getPropertyName())
+              || Room.Property.FLOOR_SHININESS.name().equals(ev.getPropertyName())
+              || Room.Property.CEILING_COLOR.name().equals(ev.getPropertyName())
+              || Room.Property.CEILING_TEXTURE.name().equals(ev.getPropertyName())
+              || Room.Property.CEILING_SHININESS.name().equals(ev.getPropertyName())) {
+            updateObjects(Arrays.asList(new Room [] {(Room)ev.getSource()}));
+          } else if (Room.Property.FLOOR_VISIBLE.name().equals(ev.getPropertyName())
+              || Room.Property.CEILING_VISIBLE.name().equals(ev.getPropertyName())) {   
+            updateObjects(home.getRooms());
+            groundColorAndTextureListener.propertyChange(null);
+          } else if (Room.Property.POINTS.name().equals(ev.getPropertyName())) {   
+            if (homeObjectsToUpdate != null) {
+              // Don't try to optimize if more than one room to update
+              updateObjects(home.getRooms());
+            } else {
+              updateObjects(Arrays.asList(new Room [] {(Room)ev.getSource()}));
+              // Search the rooms that overlap the updated one
+              Area oldArea = getArea((float [][])ev.getOldValue());
+              Area newArea = getArea((float [][])ev.getNewValue());
+              for (Room room : home.getRooms()) {
+                if (room != ev.getSource()) {
+                  Area roomAreaIntersectionWithOldArea = getArea(room.getPoints());
+                  Area roomAreaIntersectionWithNewArea = new Area(roomAreaIntersectionWithOldArea);
+                  roomAreaIntersectionWithNewArea.intersect(newArea);                  
+                  if (!roomAreaIntersectionWithNewArea.isEmpty()) {
+                    updateObjects(Arrays.asList(new Room [] {room}));
+                  } else {
+                    roomAreaIntersectionWithOldArea.intersect(oldArea);
+                    if (!roomAreaIntersectionWithOldArea.isEmpty()) {
+                      updateObjects(Arrays.asList(new Room [] {room}));
+                    }
+                  }
+                }
+              }              
+            }
+            groundColorAndTextureListener.propertyChange(null);
+          }            
+        }
+        
+        private Area getArea(float [][] points) {
+          GeneralPath path = new GeneralPath();
+          path.moveTo(points [0][0], points [0][1]);
+          for (int i = 1; i < points.length; i++) {
+            path.lineTo(points [i][0], points [i][1]);
+          }
+          path.closePath();
+          return new Area(path);
         }
       };
     for (Room room : this.home.getRooms()) {
@@ -1553,7 +1614,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
             case ADD :
               // Add room to its group at the index indicated by the event 
               // to ensure the 3D rooms are drawn in the same order as in the plan  
-              addObject(group, room, ev.getIndex() + indexOffset, false);
+              addObject(group, room, ev.getIndex(), false);
               room.addPropertyChangeListener(roomChangeListener);
               break;
             case DELETE :
@@ -1561,6 +1622,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
               room.removePropertyChangeListener(roomChangeListener);
               break;
           }
+          updateObjects(home.getRooms());
           groundColorAndTextureListener.propertyChange(null);
         }
       };
