@@ -22,10 +22,13 @@ package com.eteks.sweethome3d.swing;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
 import java.awt.Insets;
+import java.awt.event.ItemEvent;
+import java.awt.event.ItemListener;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 
 import javax.swing.ButtonGroup;
+import javax.swing.JCheckBox;
 import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JOptionPane;
@@ -63,19 +66,26 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
   private JSpinner                elevationSpinner;
   private JLabel                  angleLabel;
   private JSpinner                angleSpinner;
+  private NullableCheckBox        basePlanItemCheckBox;
   private JLabel                  widthLabel;
   private JSpinner                widthSpinner;
   private JLabel                  depthLabel;
   private JSpinner                depthSpinner;
   private JLabel                  heightLabel;
   private JSpinner                heightSpinner;
-  private JRadioButton            defaultRadioButton;
+  private JCheckBox               keepProportionsCheckBox;
+  private NullableCheckBox        mirroredModelCheckBox;
+  private JRadioButton            defaultColorAndTextureRadioButton;
   private JRadioButton            colorRadioButton;
   private ColorButton             colorButton;
   private JRadioButton            textureRadioButton;
   private JComponent              textureComponent;
+  private JRadioButton            defaultShininessRadioButton;
+  private JRadioButton            mattRadioButton;
+  private JRadioButton            shinyRadioButton;
   private NullableCheckBox        visibleCheckBox;
-  private NullableCheckBox        mirroredModelCheckBox;
+  private JLabel                  lightPowerLabel;
+  private JSpinner                lightPowerSpinner;
   private String                  dialogTitle;
 
   /**
@@ -90,7 +100,7 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
     this.controller = controller;
     createComponents(preferences, controller);
     setMnemonics(preferences);
-    layoutComponents();
+    layoutComponents(preferences, controller);
   }
 
   /**
@@ -103,7 +113,7 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
     
     // Create name label and its text field bound to NAME controller property
     this.nameLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, HomeFurniturePanel.class, "nameLabel.text"));
-    this.nameTextField = new JTextField(controller.getName(), 10);
+    this.nameTextField = new JTextField(controller.getName(), 15);
     if (!OperatingSystem.isMacOSXLeopardOrSuperior()) {
       SwingTools.addAutoSelectionOnFocusGain(this.nameTextField);
     }
@@ -256,6 +266,32 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
         }
       });
    
+    // Create base plan item check box bound to BASE_PLAN_ITEM controller property
+    this.basePlanItemCheckBox = new NullableCheckBox(SwingTools.getLocalizedLabelText(preferences, 
+        HomeFurniturePanel.class, "basePlanItemCheckBox.text"));
+    String basePlanItemToolTip = preferences.getLocalizedString(
+        HomeFurniturePanel.class, "basePlanItemCheckBox.tooltip");
+    if (basePlanItemToolTip.length() > 0) {
+      this.basePlanItemCheckBox.setToolTipText(basePlanItemToolTip);
+    }
+    this.basePlanItemCheckBox.setNullable(controller.getBasePlanItem() == null);
+    this.basePlanItemCheckBox.setValue(controller.getBasePlanItem());
+    final PropertyChangeListener basePlanItemModelChangeListener = new PropertyChangeListener() {
+      public void propertyChange(PropertyChangeEvent ev) {
+        basePlanItemCheckBox.setNullable(ev.getNewValue() == null);
+        basePlanItemCheckBox.setValue((Boolean)ev.getNewValue());
+      }
+    };
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.BASE_PLAN_ITEM, basePlanItemModelChangeListener);
+    this.basePlanItemCheckBox.addChangeListener(new ChangeListener() {
+        public void stateChanged(ChangeEvent ev) {
+          controller.removePropertyChangeListener(HomeFurnitureController.Property.BASE_PLAN_ITEM, basePlanItemModelChangeListener);
+          controller.setBasePlanItem(basePlanItemCheckBox.getValue());
+          controller.addPropertyChangeListener(HomeFurnitureController.Property.BASE_PLAN_ITEM, basePlanItemModelChangeListener);
+        }
+      }); 
+    this.basePlanItemCheckBox.setEnabled(controller.isBasePlanItemEditable());
+
     // Create width label and its spinner bound to WIDTH controller property
     this.widthLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, 
         HomeFurniturePanel.class, "widthLabel.text", unitName));
@@ -325,12 +361,54 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
         }
       });
     
-    // Create radio buttons bound to COLOR and TEXTURE controller properties
-    this.defaultRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
-        HomeFurniturePanel.class, "defaultRadioButton.text"));
-    this.defaultRadioButton.addChangeListener(new ChangeListener() {
+    // Create keep proportions check box bound to PROPORTIONAL controller property
+    this.keepProportionsCheckBox = new JCheckBox(SwingTools.getLocalizedLabelText(preferences, 
+        ImportedFurnitureWizardStepsPanel.class, "keepProportionsCheckBox.text"));
+    this.keepProportionsCheckBox.addItemListener(new ItemListener() {
+        public void itemStateChanged(ItemEvent ev) {
+          controller.setProportional(keepProportionsCheckBox.isSelected());
+        }
+      });
+    this.keepProportionsCheckBox.setSelected(controller.isProportional());
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.PROPORTIONAL,
+        new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            // If proportional property changes update keep proportions check box
+            keepProportionsCheckBox.setSelected(controller.isProportional());
+          }
+        });
+
+    // Create mirror check box bound to MODEL_MIRRORED controller property
+    this.mirroredModelCheckBox = new NullableCheckBox(SwingTools.getLocalizedLabelText(preferences, 
+        HomeFurniturePanel.class, "mirroredModelCheckBox.text"));
+    String mirroredModelToolTip = preferences.getLocalizedString(
+        HomeFurniturePanel.class, "mirroredModelCheckBox.tooltip");
+    if (mirroredModelToolTip.length() > 0) {
+      this.mirroredModelCheckBox.setToolTipText(mirroredModelToolTip);
+    }
+    this.mirroredModelCheckBox.setNullable(controller.getModelMirrored() == null);
+    this.mirroredModelCheckBox.setValue(controller.getModelMirrored());
+    final PropertyChangeListener mirroredModelChangeListener = new PropertyChangeListener() {
+      public void propertyChange(PropertyChangeEvent ev) {
+        mirroredModelCheckBox.setNullable(ev.getNewValue() == null);
+        mirroredModelCheckBox.setValue((Boolean)ev.getNewValue());
+      }
+    };
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.MODEL_MIRRORED, mirroredModelChangeListener);
+    this.mirroredModelCheckBox.addChangeListener(new ChangeListener() {
         public void stateChanged(ChangeEvent ev) {
-          if (defaultRadioButton.isSelected()) {
+          controller.removePropertyChangeListener(HomeFurnitureController.Property.MODEL_MIRRORED, mirroredModelChangeListener);
+          controller.setModelMirrored(mirroredModelCheckBox.getValue());
+          controller.addPropertyChangeListener(HomeFurnitureController.Property.MODEL_MIRRORED, mirroredModelChangeListener);
+        }
+      });
+    
+    // Create radio buttons bound to COLOR and TEXTURE controller properties
+    this.defaultColorAndTextureRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
+        HomeFurniturePanel.class, "defaultColorAndTextureRadioButton.text"));
+    this.defaultColorAndTextureRadioButton.addChangeListener(new ChangeListener() {
+        public void stateChanged(ChangeEvent ev) {
+          if (defaultColorAndTextureRadioButton.isSelected()) {
             controller.setPaint(HomeFurnitureController.FurniturePaint.DEFAULT);
           }
         }
@@ -338,7 +416,7 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
     controller.addPropertyChangeListener(HomeFurnitureController.Property.PAINT, 
         new PropertyChangeListener() {
           public void propertyChange(PropertyChangeEvent ev) {
-            updateRadioButtons(controller);
+            updateColorAndTextureRadioButtons(controller);
           }
         });
     this.colorRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
@@ -385,10 +463,51 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
     }
 
     ButtonGroup buttonGroup = new ButtonGroup();
-    buttonGroup.add(this.defaultRadioButton);
+    buttonGroup.add(this.defaultColorAndTextureRadioButton);
     buttonGroup.add(this.colorRadioButton);
     buttonGroup.add(this.textureRadioButton);
-    updateRadioButtons(controller);
+    updateColorAndTextureRadioButtons(controller);
+
+    // Create radio buttons bound to SHININESS controller properties
+    this.defaultShininessRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
+        HomeFurniturePanel.class, "defaultShininessRadioButton.text"));
+    this.defaultShininessRadioButton.addChangeListener(new ChangeListener() {
+        public void stateChanged(ChangeEvent ev) {
+          if (defaultShininessRadioButton.isSelected()) {
+            controller.setShininess(HomeFurnitureController.FurnitureShininess.DEFAULT);
+          }
+        }
+      });
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.SHININESS, 
+        new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            updateShininessRadioButtons(controller);
+          }
+        });
+    this.mattRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
+        HomeFurniturePanel.class, "mattRadioButton.text"));
+    this.mattRadioButton.addChangeListener(new ChangeListener() {
+        public void stateChanged(ChangeEvent ev) {
+          if (mattRadioButton.isSelected()) {
+            controller.setShininess(HomeFurnitureController.FurnitureShininess.MATT);
+          }
+        }
+      });
+    this.shinyRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
+        HomeFurniturePanel.class, "shinyRadioButton.text"));
+    this.shinyRadioButton.addChangeListener(new ChangeListener() {
+        public void stateChanged(ChangeEvent ev) {
+          if (shinyRadioButton.isSelected()) {
+            controller.setShininess(HomeFurnitureController.FurnitureShininess.SHINY);
+          }
+        }
+      });
+    
+    buttonGroup = new ButtonGroup();
+    buttonGroup.add(this.defaultShininessRadioButton);
+    buttonGroup.add(this.mattRadioButton);
+    buttonGroup.add(this.shinyRadioButton);
+    updateShininessRadioButtons(controller);
 
     // Create visible check box bound to VISIBLE controller property
     this.visibleCheckBox = new NullableCheckBox(SwingTools.getLocalizedLabelText(preferences, 
@@ -410,50 +529,72 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
         }
       });
     
-    // Create mirror check box bound to MODEL_MIRRORED controller property
-    this.mirroredModelCheckBox = new NullableCheckBox(SwingTools.getLocalizedLabelText(preferences, 
-        HomeFurniturePanel.class, "mirroredModelCheckBox.text"));
-    this.mirroredModelCheckBox.setNullable(controller.getModelMirrored() == null);
-    this.mirroredModelCheckBox.setValue(controller.getModelMirrored());
-    final PropertyChangeListener mirroredModelChangeListener = new PropertyChangeListener() {
-      public void propertyChange(PropertyChangeEvent ev) {
-        mirroredModelCheckBox.setNullable(ev.getNewValue() == null);
-        mirroredModelCheckBox.setValue((Boolean)ev.getNewValue());
-      }
-    };
-    controller.addPropertyChangeListener(HomeFurnitureController.Property.MODEL_MIRRORED, mirroredModelChangeListener);
-    this.mirroredModelCheckBox.addChangeListener(new ChangeListener() {
-        public void stateChanged(ChangeEvent ev) {
-          controller.removePropertyChangeListener(HomeFurnitureController.Property.MODEL_MIRRORED, mirroredModelChangeListener);
-          controller.setModelMirrored(mirroredModelCheckBox.getValue());
-          controller.addPropertyChangeListener(HomeFurnitureController.Property.MODEL_MIRRORED, mirroredModelChangeListener);
-        }
-      });
-    
-    updateSizeComponents(controller);     
-    // Add a listener that enables / disables size fields depending on furniture resizable
-    controller.addPropertyChangeListener(HomeFurnitureController.Property.RESIZABLE, 
-        new PropertyChangeListener() {
+    // Create power label and its spinner bound to POWER controller property
+    if (controller.isLightPowerEditable()) {
+      this.lightPowerLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, 
+          HomeFurniturePanel.class, "lightPowerLabel.text", unitName));
+      final NullableSpinner.NullableSpinnerNumberModel lightPowerSpinnerModel = 
+          new NullableSpinner.NullableSpinnerNumberModel(0, 0, 100, 5);
+      this.lightPowerSpinner = new NullableSpinner(lightPowerSpinnerModel);
+      lightPowerSpinnerModel.setNullable(controller.getLightPower() == null);
+      lightPowerSpinnerModel.setValue(controller.getLightPower() != null ? Math.round(controller.getLightPower() * 100) : null);
+      final PropertyChangeListener lightPowerChangeListener = new PropertyChangeListener() {
           public void propertyChange(PropertyChangeEvent ev) {
-            updateSizeComponents(controller);     
+            Float lightPower = (Float)ev.getNewValue();
+            lightPowerSpinnerModel.setNullable(lightPower == null);
+            lightPowerSpinnerModel.setValue(lightPower != null ? Math.round((Float)ev.getNewValue() * 100) : null);
+          }
+        };
+      controller.addPropertyChangeListener(HomeFurnitureController.Property.LIGHT_POWER, lightPowerChangeListener);
+      lightPowerSpinnerModel.addChangeListener(new ChangeListener() {
+          public void stateChanged(ChangeEvent ev) {
+            controller.removePropertyChangeListener(HomeFurnitureController.Property.LIGHT_POWER, lightPowerChangeListener);
+            controller.setLightPower(((Number)lightPowerSpinnerModel.getValue()).floatValue() / 100f);
+            controller.addPropertyChangeListener(HomeFurnitureController.Property.LIGHT_POWER, lightPowerChangeListener);
           }
         });
+    }
+    
+    updateSizeComponents(controller);     
+    // Add a listener that enables / disables size fields depending on furniture resizable and deformable
+    PropertyChangeListener sizeListener = new PropertyChangeListener() {
+      public void propertyChange(PropertyChangeEvent ev) {
+        updateSizeComponents(controller);     
+      }
+    };
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.RESIZABLE, sizeListener);
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.DEFORMABLE, sizeListener);
     
     this.dialogTitle = preferences.getLocalizedString(HomeFurniturePanel.class, "homeFurniture.title");
   }
   
   /**
-   * Updates radio buttons. 
+   * Updates color and texture radio buttons. 
    */
-  private void updateRadioButtons(HomeFurnitureController controller) {
+  private void updateColorAndTextureRadioButtons(HomeFurnitureController controller) {
     if (controller.getPaint() == HomeFurnitureController.FurniturePaint.DEFAULT) {
-      this.defaultRadioButton.setSelected(true);
+      this.defaultColorAndTextureRadioButton.setSelected(true);
     } else if (controller.getPaint() == HomeFurnitureController.FurniturePaint.COLORED) {
       this.colorRadioButton.setSelected(true);
     } else if (controller.getPaint() == HomeFurnitureController.FurniturePaint.TEXTURED) {
       this.textureRadioButton.setSelected(true);
     } else { // null
-      SwingTools.deselectAllRadioButtons(this.defaultRadioButton, this.colorRadioButton, this.textureRadioButton);
+      SwingTools.deselectAllRadioButtons(this.defaultColorAndTextureRadioButton, this.colorRadioButton, this.textureRadioButton);
+    }
+  }
+
+  /**
+   * Updates shininess radio buttons. 
+   */
+  private void updateShininessRadioButtons(HomeFurnitureController controller) {
+    if (controller.getShininess() == HomeFurnitureController.FurnitureShininess.DEFAULT) {
+      this.defaultShininessRadioButton.setSelected(true);
+    } else if (controller.getShininess() == HomeFurnitureController.FurnitureShininess.MATT) {
+      this.mattRadioButton.setSelected(true);
+    } else if (controller.getShininess() == HomeFurnitureController.FurnitureShininess.SHINY) {
+      this.shinyRadioButton.setSelected(true);
+    } else { // null
+      SwingTools.deselectAllRadioButtons(this.defaultShininessRadioButton, this.mattRadioButton, this.shinyRadioButton);
     }
   }
 
@@ -468,6 +609,7 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
     this.depthSpinner.setEnabled(editableSize);
     this.heightLabel.setEnabled(editableSize);
     this.heightSpinner.setEnabled(editableSize);
+    this.keepProportionsCheckBox.setEnabled(editableSize && controller.isDeformable());
     this.mirroredModelCheckBox.setEnabled(editableSize);
   }
   
@@ -493,6 +635,8 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
       this.angleLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
           HomeFurniturePanel.class, "angleLabel.mnemonic")).getKeyCode());
       this.angleLabel.setLabelFor(this.angleSpinner);
+      this.keepProportionsCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+          HomeFurniturePanel.class, "keepProportionsCheckBox.mnemonic")).getKeyCode());
       this.widthLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
           HomeFurniturePanel.class, "widthLabel.mnemonic")).getKeyCode());
       this.widthLabel.setLabelFor(this.widthSpinner);
@@ -502,111 +646,182 @@ public class HomeFurniturePanel extends JPanel implements DialogView {
       this.heightLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
           HomeFurniturePanel.class, "heightLabel.mnemonic")).getKeyCode());
       this.heightLabel.setLabelFor(this.heightSpinner);
-      this.defaultRadioButton.setMnemonic(KeyStroke.getKeyStroke(
-          preferences.getLocalizedString(HomeFurniturePanel.class, "defaultRadioButton.mnemonic")).getKeyCode());
+      this.basePlanItemCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+          HomeFurniturePanel.class, "basePlanItemCheckBox.mnemonic")).getKeyCode());;
+      this.mirroredModelCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+          HomeFurniturePanel.class, "mirroredModelCheckBox.mnemonic")).getKeyCode());
+      this.defaultColorAndTextureRadioButton.setMnemonic(KeyStroke.getKeyStroke(
+          preferences.getLocalizedString(HomeFurniturePanel.class, "defaultColorAndTextureRadioButton.mnemonic")).getKeyCode());
       this.colorRadioButton.setMnemonic(KeyStroke.getKeyStroke(
           preferences.getLocalizedString(HomeFurniturePanel.class, "colorRadioButton.mnemonic")).getKeyCode());
       this.textureRadioButton.setMnemonic(KeyStroke.getKeyStroke(
           preferences.getLocalizedString(HomeFurniturePanel.class, "textureRadioButton.mnemonic")).getKeyCode());
+      this.defaultShininessRadioButton.setMnemonic(KeyStroke.getKeyStroke(
+          preferences.getLocalizedString(HomeFurniturePanel.class, "defaultShininessRadioButton.mnemonic")).getKeyCode());
+      this.mattRadioButton.setMnemonic(KeyStroke.getKeyStroke(
+          preferences.getLocalizedString(HomeFurniturePanel.class, "mattRadioButton.mnemonic")).getKeyCode());
+      this.shinyRadioButton.setMnemonic(KeyStroke.getKeyStroke(
+          preferences.getLocalizedString(HomeFurniturePanel.class, "shinyRadioButton.mnemonic")).getKeyCode());
       this.visibleCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
           HomeFurniturePanel.class, "visibleCheckBox.mnemonic")).getKeyCode());
-      this.mirroredModelCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
-          HomeFurniturePanel.class, "mirroredModelCheckBox.mnemonic")).getKeyCode());
+      if (this.lightPowerLabel != null) {
+        this.lightPowerLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+            HomeFurniturePanel.class, "lightPowerLabel.mnemonic")).getKeyCode());
+        this.lightPowerLabel.setLabelFor(this.lightPowerSpinner);
+      }
     }
   }
   
   /**
    * Layouts panel components in panel with their labels. 
    */
-  private void layoutComponents() {
+  private void layoutComponents(UserPreferences preferences, 
+                                final HomeFurnitureController controller) {
     int labelAlignment = OperatingSystem.isMacOSX() 
         ? GridBagConstraints.LINE_END
         : GridBagConstraints.LINE_START;
-    // First row
-    Insets labelInsets = new Insets(0, 0, 10, 5);
-    add(this.nameLabel, new GridBagConstraints(
+    // First row    
+    JPanel namePanel = SwingTools.createTitledPanel(preferences.getLocalizedString(
+        HomeFurniturePanel.class, "namePanel.title"));
+    int rowGap = OperatingSystem.isMacOSXLeopardOrSuperior() ? 0 : 5;
+    namePanel.add(this.nameLabel, new GridBagConstraints(
+        0, 0, 1, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
+    namePanel.add(this.nameTextField, new GridBagConstraints(
+        1, 0, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 10), 0, 0));
+    namePanel.add(this.nameVisibleCheckBox, new GridBagConstraints(
+        3, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    add(namePanel, new GridBagConstraints(
+        0, 0, 3, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, rowGap, 0), 0, 0));
+    // Location panel
+    JPanel locationPanel = SwingTools.createTitledPanel(preferences.getLocalizedString(
+        HomeFurniturePanel.class, "locationPanel.title"));
+    Insets labelInsets = new Insets(0, 0, 5, 5);
+    Insets rightComponentInsets = new Insets(0, 0, 5, 0);
+    locationPanel.add(this.xLabel, new GridBagConstraints(
         0, 0, 1, 1, 0, 0, labelAlignment, 
         GridBagConstraints.NONE, labelInsets, 0, 0));
-    Insets componentInsets = new Insets(0, 0, 10, 15);
-    add(this.nameTextField, new GridBagConstraints(
-        1, 0, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 5), 0, 0));
-    Insets rightComponentInsets = new Insets(0, 0, 10, 0);
-    add(this.nameVisibleCheckBox, new GridBagConstraints(
-        3, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, labelInsets, 0, 0));
-    // Second row
-    add(this.xLabel, new GridBagConstraints(
+    locationPanel.add(this.xSpinner, new GridBagConstraints(
+        1, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.HORIZONTAL, rightComponentInsets, -15, 0));
+    locationPanel.add(this.yLabel, new GridBagConstraints(
         0, 1, 1, 1, 0, 0, labelAlignment, 
         GridBagConstraints.NONE, labelInsets, 0, 0));
-    add(this.xSpinner, new GridBagConstraints(
+    locationPanel.add(this.ySpinner, new GridBagConstraints(
         1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, componentInsets, -15, 0));
-    add(this.widthLabel, new GridBagConstraints(
-        2, 1, 1, 1, 0, 0, labelAlignment, 
-        GridBagConstraints.NONE, labelInsets, 0, 0));
-    add(this.widthSpinner, new GridBagConstraints(
-        3, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, rightComponentInsets, -10, 0));
-    // Third row
-    add(this.yLabel, new GridBagConstraints(
+        GridBagConstraints.HORIZONTAL, rightComponentInsets, -15, 0));
+    locationPanel.add(this.elevationLabel, new GridBagConstraints(
         0, 2, 1, 1, 0, 0, labelAlignment, 
         GridBagConstraints.NONE, labelInsets, 0, 0));
-    add(this.ySpinner, new GridBagConstraints(
+    locationPanel.add(this.elevationSpinner, new GridBagConstraints(
         1, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, componentInsets, -15, 0));
-    add(this.depthLabel, new GridBagConstraints(
-        2, 2, 1, 1, 0, 0, labelAlignment, 
-        GridBagConstraints.NONE, labelInsets, 0, 0));
-    add(this.depthSpinner, new GridBagConstraints(
-        3, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, rightComponentInsets, -10, 0));
-    // Forth row
-    add(this.elevationLabel, new GridBagConstraints(
+        GridBagConstraints.HORIZONTAL, rightComponentInsets, -15, 0));
+    locationPanel.add(this.angleLabel, new GridBagConstraints(
         0, 3, 1, 1, 0, 0, labelAlignment, 
         GridBagConstraints.NONE, labelInsets, 0, 0));
-    add(this.elevationSpinner, new GridBagConstraints(
-        1, 3, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, componentInsets, -10, 0));
-    add(this.heightLabel, new GridBagConstraints(
-        2, 3, 1, 1, 0, 0, labelAlignment, 
+    locationPanel.add(this.angleSpinner, new GridBagConstraints(
+        1, 3, 1, 1, 0, 1, GridBagConstraints.LINE_START, 
+        GridBagConstraints.HORIZONTAL, rightComponentInsets, -15, 0));
+    locationPanel.add(this.basePlanItemCheckBox, new GridBagConstraints(
+        0, 4, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    add(locationPanel, new GridBagConstraints(
+        0, 1, 1, 1, 1, 0, labelAlignment, 
+        GridBagConstraints.BOTH, new Insets(0, 0, rowGap, 0), 0, 0));
+    // Size panel
+    JPanel sizePanel = SwingTools.createTitledPanel(preferences.getLocalizedString(
+        HomeFurniturePanel.class, "sizePanel.title"));
+    sizePanel.add(this.widthLabel, new GridBagConstraints(
+        0, 0, 1, 1, 0, 0, labelAlignment, 
         GridBagConstraints.NONE, labelInsets, 0, 0));
-    add(this.heightSpinner, new GridBagConstraints(
-        3, 3, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+    sizePanel.add(this.widthSpinner, new GridBagConstraints(
+        1, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.NONE, rightComponentInsets, -10, 0));
-    // Fifth row
-    add(this.angleLabel, new GridBagConstraints(
-        0, 4, 1, 1, 0, 0, labelAlignment, 
-        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
-    add(this.angleSpinner, new GridBagConstraints(
-        1, 4, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 15), -15, 0));
-    add(this.defaultRadioButton, new GridBagConstraints(
-        2, 4, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
-    // Sixth row
-    add(this.mirroredModelCheckBox, new GridBagConstraints(
-        1, 5, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 5, 15), 0, 0));
-    add(this.colorRadioButton, new GridBagConstraints(
-        2, 5, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
-    add(this.colorButton, new GridBagConstraints(
-        3, 5, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
-    // Last row
-    add(this.visibleCheckBox, new GridBagConstraints(
-        1, 6, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 0, 15), 0, 0));
+    sizePanel.add(this.depthLabel, new GridBagConstraints(
+        0, 1, 1, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.NONE, labelInsets, 0, 0));
+    sizePanel.add(this.depthSpinner, new GridBagConstraints(
+        1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, rightComponentInsets, -10, 0));
+    sizePanel.add(this.heightLabel, new GridBagConstraints(
+        0, 2, 1, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.NONE, labelInsets, 0, 0));
+    sizePanel.add(this.heightSpinner, new GridBagConstraints(
+        1, 2, 1, 1, 0, 1, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, rightComponentInsets, -10, 0));
+    sizePanel.add(this.keepProportionsCheckBox, new GridBagConstraints(
+        0, 3, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, rightComponentInsets, 0, 0));
+    sizePanel.add(this.mirroredModelCheckBox, new GridBagConstraints(
+        0, 4, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    add(sizePanel, new GridBagConstraints(
+        1, 1, 2, 1, 1, 0, labelAlignment, 
+        GridBagConstraints.BOTH, new Insets(0, 0, rowGap, 0), 0, 0));
+    // Color and Texture panel
+    final JPanel colorAndTexturePanel = SwingTools.createTitledPanel(preferences.getLocalizedString(
+        HomeFurniturePanel.class, "colorAndTexturePanel.title"));
+    colorAndTexturePanel.add(this.defaultColorAndTextureRadioButton, new GridBagConstraints(
+        0, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, labelInsets, 0, 0));
+    colorAndTexturePanel.add(this.colorRadioButton, new GridBagConstraints(
+        0, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
+    colorAndTexturePanel.add(this.colorButton, new GridBagConstraints(
+        1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
     if (this.textureComponent != null) {
-      add(this.textureRadioButton, new GridBagConstraints(
-          2, 6, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
-      add(this.textureComponent, new GridBagConstraints(
-          3, 6, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+      colorAndTexturePanel.add(this.textureRadioButton, new GridBagConstraints(
+          0, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
+      colorAndTexturePanel.add(this.textureComponent, new GridBagConstraints(
+          1, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, new Insets(5, 0, 0, 0), 0, 0));
       this.textureComponent.setPreferredSize(this.colorButton.getPreferredSize());
     }
+    add(colorAndTexturePanel, new GridBagConstraints(
+        0, 2, 1, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.BOTH, new Insets(0, 0, rowGap, 0), 0, 0));
+    // Shininess panel
+    final JPanel shininessPanel = SwingTools.createTitledPanel(preferences.getLocalizedString(
+        HomeFurniturePanel.class, "shininessPanel.title"));
+    shininessPanel.add(this.defaultShininessRadioButton, new GridBagConstraints(
+        0, 0, 1, 1, 0, 1, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
+    shininessPanel.add(this.mattRadioButton, new GridBagConstraints(
+        0, 1, 1, 1, 0, 1, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    shininessPanel.add(this.shinyRadioButton, new GridBagConstraints(
+        0, 2, 1, 1, 0, 1, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(5, 0, 0, 0), 0, 0));
+    add(shininessPanel, new GridBagConstraints(
+        1, 2, 2, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.BOTH, new Insets(0, 0, rowGap, 0), 0, 0));
+    // Last row
+    add(this.visibleCheckBox, new GridBagConstraints(
+        0, 3, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.NONE, new Insets(0, 10, 0, 0), 0, 0));
+    if (this.lightPowerLabel != null) {
+      add(this.lightPowerLabel, new GridBagConstraints(
+          1, 3, 1, 1, 0, 0, labelAlignment, 
+          GridBagConstraints.NONE, new Insets(0, 10, 0, 5), 0, 0));
+      add(this.lightPowerSpinner, new GridBagConstraints(
+          2, 3, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
+    }
+    
+    controller.addPropertyChangeListener(HomeFurnitureController.Property.TEXTURABLE, 
+        new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            colorAndTexturePanel.setVisible(controller.isTexturable());
+            shininessPanel.setVisible(controller.isTexturable());
+          }
+        });
+    colorAndTexturePanel.setVisible(controller.isTexturable());
+    shininessPanel.setVisible(controller.isTexturable());
   }
 
   /**
