@@ -25,6 +25,7 @@ import java.awt.EventQueue;
 import java.awt.Frame;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.GridBagLayout;
 import java.awt.Insets;
 import java.awt.Rectangle;
 import java.awt.Toolkit;
@@ -55,6 +56,7 @@ import javax.swing.ButtonGroup;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
 import javax.swing.JOptionPane;
+import javax.swing.JPanel;
 import javax.swing.JRadioButton;
 import javax.swing.JScrollPane;
 import javax.swing.JToggleButton;
@@ -205,26 +207,35 @@ public class SwingTools {
   }
 
   /**
-   * Updates the Swing resource bundle in use from the current Locale. 
+   * Updates the Swing resource bundles in use from the current Locale. 
    */
   public static void updateSwingResourceLanguage() {
     // Read Swing localized properties because Swing doesn't update its internal strings automatically
     // when default Locale is updated (see bug http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4884480)
-    String [] swingResources = {"com.sun.swing.internal.plaf.basic.resources.basic",
-                                "com.sun.swing.internal.plaf.metal.resources.metal"};
-    for (String swingResource : swingResources) {
-      ResourceBundle resource;
-      try {
-        resource = ResourceBundle.getBundle(swingResource);
-      } catch (MissingResourceException ex) {
-        resource = ResourceBundle.getBundle(swingResource, Locale.ENGLISH);
-      }
-      // Update UIManager properties
-      for (Enumeration<?> it = resource.getKeys(); it.hasMoreElements(); ) {
-        String property = (String)it.nextElement();
-        UIManager.put(property, resource.getString(property));
-      }      
-    };
+    updateSwingResourceBundle("com.sun.swing.internal.plaf.metal.resources.metal");
+    updateSwingResourceBundle("com.sun.swing.internal.plaf.basic.resources.basic");
+    if (UIManager.getLookAndFeel().getClass().getName().equals("com.sun.java.swing.plaf.gtk.GTKLookAndFeel")) {
+      updateSwingResourceBundle("com.sun.java.swing.plaf.gtk.resources.gtk");
+    } else if (UIManager.getLookAndFeel().getClass().getName().equals("com.sun.java.swing.plaf.motif.MotifLookAndFeel")) {
+      updateSwingResourceBundle("com.sun.java.swing.plaf.motif.resources.motif");
+    } 
+  }
+
+  /**
+   * Updates a Swing resource bundle in use from the current Locale. 
+   */
+  private static void updateSwingResourceBundle(String swingResource) {
+    ResourceBundle resource;
+    try {
+      resource = ResourceBundle.getBundle(swingResource);
+    } catch (MissingResourceException ex) {
+      resource = ResourceBundle.getBundle(swingResource, Locale.ENGLISH);
+    }
+    // Update UIManager properties
+    for (Enumeration<?> it = resource.getKeys(); it.hasMoreElements(); ) {
+      String property = (String)it.nextElement();
+      UIManager.put(property, resource.getString(property));
+    }
   }
   
   /**
@@ -491,4 +502,19 @@ public class SwingTools {
       // Ignore splash screen
     }
   }
+  
+  /**
+   * Returns a new panel with a border and the given <code>title</code>
+   */
+  public static JPanel createTitledPanel(String title) {
+    JPanel titledPanel = new JPanel(new GridBagLayout());
+    Border panelBorder = BorderFactory.createTitledBorder(title);
+    // For systems different from Mac OS X 10.5, add an empty border 
+    if (!OperatingSystem.isMacOSXLeopardOrSuperior()) {
+      panelBorder = BorderFactory.createCompoundBorder(
+          panelBorder, BorderFactory.createEmptyBorder(0, 2, 2, 2));
+    }    
+    titledPanel.setBorder(panelBorder);    
+    return titledPanel;
+  }
 }
