@@ -25,6 +25,7 @@ import java.awt.BorderLayout;
 import java.awt.CardLayout;
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.ComponentOrientation;
 import java.awt.Composite;
 import java.awt.Cursor;
 import java.awt.Dimension;
@@ -43,6 +44,8 @@ import java.awt.event.ComponentAdapter;
 import java.awt.event.ComponentEvent;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
+import java.awt.event.MouseAdapter;
+import java.awt.event.MouseEvent;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.awt.geom.AffineTransform;
@@ -62,13 +65,19 @@ import java.io.InputStream;
 import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.lang.ref.WeakReference;
+import java.text.DateFormat;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Calendar;
 import java.util.Collections;
+import java.util.Date;
 import java.util.Dictionary;
+import java.util.GregorianCalendar;
 import java.util.Hashtable;
 import java.util.List;
 import java.util.ListIterator;
+import java.util.TimeZone;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 
@@ -86,6 +95,7 @@ import javax.swing.BorderFactory;
 import javax.swing.BoundedRangeModel;
 import javax.swing.DefaultListCellRenderer;
 import javax.swing.JButton;
+import javax.swing.JCheckBox;
 import javax.swing.JComboBox;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
@@ -94,11 +104,15 @@ import javax.swing.JList;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JProgressBar;
+import javax.swing.JSeparator;
 import javax.swing.JSlider;
+import javax.swing.JSpinner;
 import javax.swing.JToolBar;
 import javax.swing.KeyStroke;
+import javax.swing.SpinnerDateModel;
 import javax.swing.SwingUtilities;
 import javax.swing.Timer;
+import javax.swing.ToolTipManager;
 import javax.swing.UIManager;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
@@ -126,6 +140,11 @@ import com.eteks.sweethome3d.viewcontroller.View;
 public class VideoPanel extends JPanel implements DialogView {
   private enum ActionType {START_VIDEO_CREATION, STOP_VIDEO_CREATION, SAVE_VIDEO, CLOSE, 
       DELETE_CAMERA_PATH, PLAYBACK, PAUSE, RECORD, SEEK_BACKWARD, SEEK_FORWARD, SKIP_BACKWARD, SKIP_FORWARD, DELETE_LAST_RECORD}
+
+  private static final String VIDEO_DIALOG_X_VISUAL_PROPERTY = "com.eteks.sweethome3d.swing.VideoPanel.VideoDialogX";
+  private static final String VIDEO_DIALOG_Y_VISUAL_PROPERTY = "com.eteks.sweethome3d.swing.VideoPanel.VideoDialogY";
+
+  private static final int MINIMUM_DELAY_BEFORE_DISCARDING_WITHOUT_WARNING = 30000;
   
   private static final VideoFormat [] VIDEO_FORMATS = {
       new VideoFormat(VideoFormat.JPEG, new Dimension(176, 132), Format.NOT_SPECIFIED, Format.byteArray, 12), // 4/3
@@ -160,16 +179,21 @@ public class VideoPanel extends JPanel implements DialogView {
   private String                videoFormatComboBoxFormat;
   private JLabel                qualityLabel;
   private JSlider               qualitySlider;
-  private JPanel                qualityDescriptionPanel;
-  private JLabel []             qualityDescriptionLabels;
+  private Component             advancedComponentsSeparator;
+  private JLabel                dateLabel;
+  private JSpinner              dateSpinner;
+  private JLabel                timeLabel;
+  private JSpinner              timeSpinner;
+  private JCheckBox             ceilingLightEnabledCheckBox;
   private String                dialogTitle;
   private ExecutorService       videoCreationExecutor;
+  private long                  videoCreationStartTime;
   private File                  videoFile;
   private JButton               createButton;
   private JButton               saveButton;
   private JButton               closeButton;
 
-  private static VideoPanel     currentVideoPanel; // There can be only one video panel opened at a time
+  private static VideoPanel     currentVideoPanel; // Support only one video panel opened at a time
 
   /**
    * Creates a video panel.
@@ -269,7 +293,7 @@ public class VideoPanel extends JPanel implements DialogView {
         new ResourceAction(preferences, VideoPanel.class, ActionType.STOP_VIDEO_CREATION.name(), true) {
           @Override
           public void actionPerformed(ActionEvent ev) {
-            stopVideoCreation();
+            stopVideoCreation(true);
           }
         });
     actions.put(ActionType.SAVE_VIDEO, 
@@ -294,6 +318,7 @@ public class VideoPanel extends JPanel implements DialogView {
   private void createComponents(Home home, 
                                 final UserPreferences preferences,
                                 final VideoController controller) {
+    final Dimension preferredSize = new Dimension(getToolkit().getScreenSize().width <= 1024 ? 324 : 404, 404);
     this.planComponent = new PlanComponent(home, preferences, null) {
         private void updateScale() {
           if (getWidth() > 0 && getHeight() > 0) {
@@ -308,10 +333,15 @@ public class VideoPanel extends JPanel implements DialogView {
           }
         }
         
+        @Override
+        public Dimension getPreferredSize() {
+          return preferredSize;
+        }
+        
         @Override
         public void revalidate() {
-          updateScale();
           super.revalidate();
+          updateScale();
         }
 
         @Override
@@ -320,11 +350,6 @@ public class VideoPanel extends JPanel implements DialogView {
           updateScale();
         }
         
-        @Override
-        public Dimension getPreferredSize() {
-          return new Dimension(404, 404);
-        }
-        
         @Override
         protected List<Selectable> getPaintedItems() {
           List<Selectable> paintedItems = super.getPaintedItems();
@@ -543,20 +568,43 @@ public class VideoPanel extends JPanel implements DialogView {
     controller.addPropertyChangeListener(VideoController.Property.ASPECT_RATIO, propertyChangeListener);
     controller.addPropertyChangeListener(VideoController.Property.FRAME_RATE, propertyChangeListener);
 
-    // Quality panel displaying explanations about quality level
-    final CardLayout qualityDescriptionLayout = new CardLayout();
-    this.qualityDescriptionPanel = new JPanel(qualityDescriptionLayout);
-    this.qualityDescriptionLabels = new JLabel [controller.getQualityLevelCount()];
-    Font font = toolTipFont;
-    for (int i = 0; i < this.qualityDescriptionLabels.length; i++) {
-      this.qualityDescriptionLabels [i] = new JLabel();
-      this.qualityDescriptionLabels [i].setFont(font);
-      this.qualityDescriptionPanel.add(String.valueOf(i), this.qualityDescriptionLabels [i]);
-    }
-
     // Quality label and slider bound to QUALITY controller property
     this.qualityLabel = new JLabel();
-    this.qualitySlider = new JSlider(0, qualityDescriptionLabels.length - 1);
+    this.qualitySlider = new JSlider(0, controller.getQualityLevelCount() - 1) {
+        @Override
+        public String getToolTipText(MouseEvent ev) {
+          float valueUnderMouse = getSliderValueAt(this, ev.getX(), preferences);
+          float valueToTick = valueUnderMouse + 1 - (float)Math.floor(valueUnderMouse + 1);
+          if (valueToTick < 0.25f || valueToTick > 0.75f) {
+            // Display a tooltip that explains the different quality levels
+            return "<html><table><tr valign='middle'>"
+                + "<td><img border='1' src='" 
+                + new ResourceURLContent(PhotoPanel.class, "resources/quality" + Math.round(valueUnderMouse) + ".jpg").getURL() + "'></td>"
+                + "<td>" + preferences.getLocalizedString(VideoPanel.class, "quality" + Math.round(valueUnderMouse) + "DescriptionLabel.text") + "</td>"
+                + "</tr></table>";
+          } else {
+            return null;
+          }
+        }
+      };
+    // Add a listener that displays also the tool tip when user clicks on the slider
+    this.qualitySlider.addMouseListener(new MouseAdapter() {
+        @Override
+        public void mousePressed(final MouseEvent ev) {
+          EventQueue.invokeLater(new Runnable() {
+              public void run() {
+                float valueUnderMouse = getSliderValueAt(qualitySlider, ev.getX(), preferences);
+                if (qualitySlider.getValue() == Math.round(valueUnderMouse)) {
+                  ToolTipManager toolTipManager = ToolTipManager.sharedInstance();
+                  int initialDelay = toolTipManager.getInitialDelay();
+                  toolTipManager.setInitialDelay(Math.min(initialDelay, 150));
+                  toolTipManager.mouseMoved(ev);
+                  toolTipManager.setInitialDelay(initialDelay);
+                }
+              }
+            });
+        }
+      });
     this.qualitySlider.setPaintLabels(true);
     this.qualitySlider.setPaintTicks(true);    
     this.qualitySlider.setMajorTickSpacing(1);
@@ -575,23 +623,81 @@ public class VideoPanel extends JPanel implements DialogView {
         new PropertyChangeListener() {
           public void propertyChange(PropertyChangeEvent ev) {
             qualitySlider.setValue(controller.getQuality());
-            qualityDescriptionLayout.show(qualityDescriptionPanel, String.valueOf(controller.getQuality()));
+            updateAdvancedComponents();
           }
         });
     this.qualitySlider.setValue(controller.getQuality());
-    qualityDescriptionLayout.show(this.qualityDescriptionPanel, String.valueOf(this.qualitySlider.getValue()));
     
-    this.createButton = new JButton(actionMap.get(ActionType.START_VIDEO_CREATION));
-    this.createButton.addActionListener(new ActionListener() {
-        public void actionPerformed(ActionEvent ev) {
-          // Swap Start / Stop action
-          if (createButton.getAction() == actionMap.get(ActionType.START_VIDEO_CREATION)) {
-            createButton.setAction(actionMap.get(ActionType.STOP_VIDEO_CREATION));
-          } else {
-            createButton.setAction(actionMap.get(ActionType.START_VIDEO_CREATION));
+    this.advancedComponentsSeparator = new JSeparator();
+
+    // Create date and time labels and spinners bound to TIME controller property
+    Date time = new Date(Camera.convertTimeToTimeZone(controller.getTime(), TimeZone.getDefault().getID()));
+    this.dateLabel = new JLabel();
+    final SpinnerDateModel dateSpinnerModel = new SpinnerDateModel();
+    dateSpinnerModel.setValue(time);
+    this.dateSpinner = new JSpinner(dateSpinnerModel);
+    String datePattern = ((SimpleDateFormat)DateFormat.getDateInstance(DateFormat.SHORT)).toPattern();
+    if (datePattern.indexOf("yyyy") == -1) {
+      datePattern = datePattern.replace("yy", "yyyy");
+    }
+    JSpinner.DateEditor dateEditor = new JSpinner.DateEditor(this.dateSpinner, datePattern);
+    this.dateSpinner.setEditor(dateEditor);
+    SwingTools.addAutoSelectionOnFocusGain(dateEditor.getTextField());
+    
+    this.timeLabel = new JLabel();
+    final SpinnerDateModel timeSpinnerModel = new SpinnerDateModel();
+    timeSpinnerModel.setValue(time);
+    this.timeSpinner = new JSpinner(timeSpinnerModel);
+    JSpinner.DateEditor timeEditor = new JSpinner.DateEditor(this.timeSpinner, 
+        ((SimpleDateFormat)DateFormat.getTimeInstance(DateFormat.SHORT)).toPattern());
+    this.timeSpinner.setEditor(timeEditor);
+    SwingTools.addAutoSelectionOnFocusGain(timeEditor.getTextField());
+
+    final PropertyChangeListener timeChangeListener = new PropertyChangeListener() {
+      public void propertyChange(PropertyChangeEvent ev) {
+        Date date = new Date(Camera.convertTimeToTimeZone(controller.getTime(), TimeZone.getDefault().getID()));
+        dateSpinnerModel.setValue(date);
+        timeSpinnerModel.setValue(date);
+      }
+    };
+    controller.addPropertyChangeListener(VideoController.Property.TIME, timeChangeListener);
+    final ChangeListener dateTimeChangeListener = new ChangeListener() {
+        public void stateChanged(ChangeEvent ev) {
+          controller.removePropertyChangeListener(VideoController.Property.TIME, timeChangeListener);
+          // Merge date and time
+          GregorianCalendar dateCalendar = new GregorianCalendar();
+          dateCalendar.setTime((Date)dateSpinnerModel.getValue());
+          GregorianCalendar timeCalendar = new GregorianCalendar();
+          timeCalendar.setTime((Date)timeSpinnerModel.getValue());
+          Calendar utcCalendar = new GregorianCalendar(TimeZone.getTimeZone("UTC"));
+          utcCalendar.set(GregorianCalendar.YEAR, dateCalendar.get(GregorianCalendar.YEAR));
+          utcCalendar.set(GregorianCalendar.MONTH, dateCalendar.get(GregorianCalendar.MONTH));
+          utcCalendar.set(GregorianCalendar.DAY_OF_MONTH, dateCalendar.get(GregorianCalendar.DAY_OF_MONTH));
+          utcCalendar.set(GregorianCalendar.HOUR_OF_DAY, timeCalendar.get(GregorianCalendar.HOUR_OF_DAY));
+          utcCalendar.set(GregorianCalendar.MINUTE, timeCalendar.get(GregorianCalendar.MINUTE));
+          utcCalendar.set(GregorianCalendar.SECOND, timeCalendar.get(GregorianCalendar.SECOND));
+          controller.setTime(utcCalendar.getTimeInMillis());
+          controller.addPropertyChangeListener(VideoController.Property.TIME, timeChangeListener);
+        }
+      };
+    dateSpinnerModel.addChangeListener(dateTimeChangeListener);
+    timeSpinnerModel.addChangeListener(dateTimeChangeListener);
+
+    this.ceilingLightEnabledCheckBox = new JCheckBox();
+    this.ceilingLightEnabledCheckBox.setSelected(controller.getCeilingLightColor() > 0);
+    controller.addPropertyChangeListener(VideoController.Property.CEILING_LIGHT_COLOR, 
+        new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            ceilingLightEnabledCheckBox.setSelected(controller.getCeilingLightColor() > 0);
           }
+        });
+    this.ceilingLightEnabledCheckBox.addItemListener(new ItemListener() {
+        public void itemStateChanged(ItemEvent ev) {
+          controller.setCeilingLightColor(ceilingLightEnabledCheckBox.isSelected() ? 0xD0D0D0 : 0);
         }
       });
+
+    this.createButton = new JButton(actionMap.get(ActionType.START_VIDEO_CREATION));
     this.saveButton = new JButton(actionMap.get(ActionType.SAVE_VIDEO));
     this.closeButton = new JButton(actionMap.get(ActionType.CLOSE));
 
@@ -602,6 +708,25 @@ public class VideoPanel extends JPanel implements DialogView {
         new Dimension(controller.getWidth(), controller.getHeight()), Format.NOT_SPECIFIED, Format.byteArray, controller.getFrameRate()));
   }
 
+  /**
+   * Returns the slider value matching a given x.
+   */
+  private float getSliderValueAt(JSlider qualitySlider, int x, UserPreferences preferences) {
+    int fastLabelOffset = OperatingSystem.isLinux() 
+        ? 0
+        : new JLabel(SwingTools.getLocalizedLabelText(preferences,
+              VideoPanel.class, "fastLabel.text")).getPreferredSize().width / 2;
+    int bestLabelOffset = OperatingSystem.isLinux() 
+        ? 0
+        : new JLabel(SwingTools.getLocalizedLabelText(preferences,
+              VideoPanel.class, "bestLabel.text")).getPreferredSize().width / 2;
+    int sliderWidth = qualitySlider.getWidth() - fastLabelOffset - bestLabelOffset;
+    return (float)(x - (qualitySlider.getComponentOrientation() == ComponentOrientation.LEFT_TO_RIGHT 
+                          ? fastLabelOffset 
+                          : bestLabelOffset))
+        / sliderWidth * qualitySlider.getMaximum();
+  }
+  
   /**
    * Sets the texts of the components.
    */
@@ -613,6 +738,12 @@ public class VideoPanel extends JPanel implements DialogView {
         VideoPanel.class, "videoFormatComboBox.format");
     this.qualityLabel.setText(SwingTools.getLocalizedLabelText(preferences, 
         VideoPanel.class, "qualityLabel.text"));
+    this.dateLabel.setText(SwingTools.getLocalizedLabelText(preferences, 
+        VideoPanel.class, "dateLabel.text"));
+    this.timeLabel.setText(SwingTools.getLocalizedLabelText(preferences, 
+        VideoPanel.class, "timeLabel.text"));
+    this.ceilingLightEnabledCheckBox.setText(SwingTools.getLocalizedLabelText(preferences, 
+        VideoPanel.class, "ceilingLightEnabledCheckBox.text"));
     JLabel fastLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, 
         VideoPanel.class, "fastLabel.text"));
     if (!Component3DManager.getInstance().isOffScreenImageSupported()) {
@@ -624,13 +755,6 @@ public class VideoPanel extends JPanel implements DialogView {
     qualitySliderLabelTable.put(this.qualitySlider.getMinimum(), fastLabel);
     qualitySliderLabelTable.put(this.qualitySlider.getMaximum(), bestLabel);
     this.qualitySlider.setLabelTable(qualitySliderLabelTable);
-    for (int i = 0; i < qualityDescriptionLabels.length; i++) {
-      this.qualityDescriptionLabels [i].setText("<html><table><tr valign='middle'>"
-         + "<td><img border='1' src='" 
-         + new ResourceURLContent(VideoPanel.class, "resources/quality" + i + ".jpg").getURL() + "'></td>"
-         + "<td>" + preferences.getLocalizedString(VideoPanel.class, "quality" + i + "DescriptionLabel.text") + "</td>"
-         + "</tr></table>");
-    }
     this.dialogTitle = preferences.getLocalizedString(VideoPanel.class, "createVideo.title");
     Window window = SwingUtilities.getWindowAncestor(this);  
     if (window != null) {
@@ -652,6 +776,14 @@ public class VideoPanel extends JPanel implements DialogView {
           KeyStroke.getKeyStroke(preferences.getLocalizedString(
               VideoPanel.class, "qualityLabel.mnemonic")).getKeyCode());
       this.qualityLabel.setLabelFor(this.qualitySlider);
+      this.dateLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+          VideoPanel.class, "dateLabel.mnemonic")).getKeyCode());
+      this.dateLabel.setLabelFor(this.dateSpinner);
+      this.timeLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+          VideoPanel.class, "timeLabel.mnemonic")).getKeyCode());
+      this.timeLabel.setLabelFor(this.timeSpinner);
+      this.ceilingLightEnabledCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString( 
+          VideoPanel.class, "ceilingLightEnabledCheckBox.mnemonic")).getKeyCode());
     }
   }
 
@@ -710,10 +842,10 @@ public class VideoPanel extends JPanel implements DialogView {
     // Fourth row
     // Add a dummy label at left and right
     add(new JLabel(), new GridBagConstraints(
-        0, 3, 1, 3, 0.5f, 0, GridBagConstraints.CENTER, 
+        0, 3, 1, 6, 0.5f, 0, GridBagConstraints.CENTER, 
         GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
     add(new JLabel(), new GridBagConstraints(
-        3, 3, 1, 3, 0.5f, 0, GridBagConstraints.CENTER, 
+        3, 3, 1, 6, 0.5f, 0, GridBagConstraints.CENTER, 
         GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
     Insets labelInsets = new Insets(0, 0, 0, 5);
     add(this.videoFormatLabel, new GridBagConstraints(
@@ -730,14 +862,55 @@ public class VideoPanel extends JPanel implements DialogView {
     add(this.qualitySlider, new GridBagConstraints(
         2, 4, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
+    // Sixth row
+    add(this.advancedComponentsSeparator, new GridBagConstraints(
+        1, 6, 4, 1, 0, 0, GridBagConstraints.CENTER, 
+        GridBagConstraints.HORIZONTAL, new Insets(3, 0, 3, 0), 0, 0));
+    // Seventh row
+    JPanel advancedPanel = new JPanel(new GridBagLayout());
+    advancedPanel.add(this.dateLabel, new GridBagConstraints(
+        1, 7, 1, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
+    advancedPanel.add(this.dateSpinner, new GridBagConstraints(
+        2, 7, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 10), 1, 0));
+    advancedPanel.add(this.timeLabel, new GridBagConstraints(
+        3, 7, 1, 1, 0, 0, labelAlignment, 
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
+    advancedPanel.add(this.timeSpinner, new GridBagConstraints(
+        4, 7, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     // Last row
-    // Force minimum size to avoid resizing effect
-    this.qualityDescriptionPanel.setMinimumSize(this.qualityDescriptionPanel.getPreferredSize());
-    add(this.qualityDescriptionPanel, new GridBagConstraints(
-        1, 5, 4, 1, 0, 0, GridBagConstraints.CENTER, 
+    advancedPanel.add(this.ceilingLightEnabledCheckBox, new GridBagConstraints(
+        1, 8, 4, 1, 0, 0, GridBagConstraints.CENTER, 
+        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    
+    add(advancedPanel, new GridBagConstraints(
+        0, 7, 4, 1, 0, 0, GridBagConstraints.CENTER, 
         GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
   }
   
+  private void updateAdvancedComponents() {
+    Component root = SwingUtilities.getRoot(this);
+    if (root != null) {
+      boolean highQuality = controller.getQuality() >= 2;
+      boolean advancedComponentsVisible = this.advancedComponentsSeparator.isVisible();
+      if (advancedComponentsVisible != highQuality) {
+        int componentsHeight = this.advancedComponentsSeparator.getPreferredSize().height + 6
+            + this.dateSpinner.getPreferredSize().height + 5
+            + this.ceilingLightEnabledCheckBox.getPreferredSize().height;
+        this.advancedComponentsSeparator.setVisible(highQuality);
+        this.dateLabel.setVisible(highQuality);
+        this.dateSpinner.setVisible(highQuality);
+        this.timeLabel.setVisible(highQuality);
+        this.timeSpinner.setVisible(highQuality);
+        this.ceilingLightEnabledCheckBox.setVisible(highQuality);
+        root.setSize(root.getWidth(), 
+            root.getHeight() + (advancedComponentsVisible ? -componentsHeight : componentsHeight));
+      }
+    }   
+  }
+
   /**
    * Displays this panel in a non modal dialog.
    */
@@ -753,37 +926,41 @@ public class VideoPanel extends JPanel implements DialogView {
           null, new Object [] {this.createButton, this.saveButton, this.closeButton}, this.createButton);
       final JDialog dialog = optionPane.createDialog(SwingUtilities.getRootPane((Component)parentView), this.dialogTitle);
       dialog.setModal(false);
-      dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
-      dialog.addComponentListener(new ComponentAdapter() {
-          @Override
-          public void componentHidden(ComponentEvent ev) {
-            if (optionPane.getValue() != null
-                && optionPane.getValue() != JOptionPane.UNINITIALIZED_VALUE) {
-              close();
-            }
-          }
-        });
       
       Component homeRoot = SwingUtilities.getRoot((Component)parentView);
       if (homeRoot != null) {
+        // Restore location if it exists
+        Integer x = (Integer)this.home.getVisualProperty(VIDEO_DIALOG_X_VISUAL_PROPERTY);
+        Integer y = (Integer)this.home.getVisualProperty(VIDEO_DIALOG_Y_VISUAL_PROPERTY);      
+
         int windowRightBorder = homeRoot.getX() + homeRoot.getWidth();
         Dimension screenSize = getToolkit().getScreenSize();
         Insets screenInsets = getToolkit().getScreenInsets(getGraphicsConfiguration());
         int screenRightBorder = screenSize.width - screenInsets.right;
         // Check dialog isn't too high
         int screenHeight = screenSize.height - screenInsets.top - screenInsets.bottom;
-        if (dialog.getHeight() > screenHeight) {
-          dialog.setSize(dialog.getWidth(), screenHeight);
+        if (OperatingSystem.isLinux() && screenHeight == screenSize.height) {
+          // Let's consider that under Linux at least an horizontal bar exists 
+          screenHeight -= 30;
         }
+        int screenBottomBorder = screenSize.height - screenInsets.bottom;
         int dialogWidth = dialog.getWidth();
-        // If there some space left at the right of the window
-        if (screenRightBorder - windowRightBorder > dialogWidth / 2
-            || dialog.getHeight() == screenHeight) {
-          // Move the dialog to the right of window
+        if (dialog.getHeight() > screenHeight) {
+          dialog.setSize(dialogWidth, screenHeight);
+        }
+        int dialogHeight = dialog.getHeight();
+        if (x != null && y != null 
+            && x + dialogWidth <= screenRightBorder
+            && y + dialogHeight <= screenBottomBorder) {
+          dialog.setLocation(x, y);
+        } else if (screenRightBorder - windowRightBorder > dialogWidth / 2
+            || dialogHeight == screenHeight) {
+          // If there some space left at the right of the window
+          // move the dialog to the right of window
           dialog.setLocationByPlatform(false);
           dialog.setLocation(Math.min(windowRightBorder + 5, screenRightBorder - dialogWidth), 
               Math.max(Math.min(homeRoot.getY() + dialog.getInsets().top, 
-                  screenSize.height - dialog.getHeight() - screenInsets.bottom), screenInsets.top));
+                  screenSize.height - dialogHeight - screenInsets.bottom), screenInsets.top));
         } else {
           dialog.setLocationByPlatform(true);
         }
@@ -793,7 +970,7 @@ public class VideoPanel extends JPanel implements DialogView {
       
       dialog.addWindowListener(new WindowAdapter() {
         public void windowClosed(WindowEvent ev) {
-          stopVideoCreation();
+          stopVideoCreation(false);
           if (playbackTimer != null) {
             pausePlayback();
           }
@@ -804,6 +981,24 @@ public class VideoPanel extends JPanel implements DialogView {
         }
       });
       
+      updateAdvancedComponents();
+      ToolTipManager.sharedInstance().registerComponent(this.qualitySlider);
+      dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
+      dialog.addComponentListener(new ComponentAdapter() {
+          @Override
+          public void componentHidden(ComponentEvent ev) {
+            if (optionPane.getValue() != null
+                && optionPane.getValue() != JOptionPane.UNINITIALIZED_VALUE) {
+              close();
+            }
+          }
+
+          @Override
+          public void componentMoved(ComponentEvent ev) {
+            controller.setVisualProperty(VIDEO_DIALOG_X_VISUAL_PROPERTY, dialog.getX());
+            controller.setVisualProperty(VIDEO_DIALOG_Y_VISUAL_PROPERTY, dialog.getY());
+          }
+        });
       dialog.setVisible(true);
       currentVideoPanel = this;
     }
@@ -823,7 +1018,10 @@ public class VideoPanel extends JPanel implements DialogView {
         || !compareCameraLocation(lastCamera, camera)) {
       // Record only new locations
       cameraPath = new ArrayList<Camera>(cameraPath);
-      cameraPath.add(camera.clone());
+      Camera recordedCamera = camera.clone();
+      recordedCamera.setLens(Camera.Lens.NORMAL);
+      recordedCamera.setTime(this.controller.getTime());
+      cameraPath.add(recordedCamera);
       this.controller.setCameraPath(cameraPath);
     }
   }
@@ -837,7 +1035,8 @@ public class VideoPanel extends JPanel implements DialogView {
         && camera1.getZ() == camera2.getZ() 
         && camera1.getYaw() == camera2.getYaw()
         && camera1.getPitch() == camera2.getPitch() 
-        && camera1.getFieldOfView() == camera2.getFieldOfView();
+        && camera1.getFieldOfView() == camera2.getFieldOfView()
+        && camera1.getTime() == camera2.getTime();
   }
 
   /**
@@ -864,13 +1063,17 @@ public class VideoPanel extends JPanel implements DialogView {
           public void actionPerformed(ActionEvent ev) {
             if ("backward".equals(ev.getActionCommand())) {
               if (cameraPathIterator.hasPrevious()) {
-                home.getCamera().setCamera(cameraPathIterator.previous());
+                Camera camera = cameraPathIterator.previous();
+                home.getCamera().setCamera(camera);
+                controller.setTime(camera.getTime());
               } else {
                 pausePlayback();
               }
             } else {
               if (cameraPathIterator.hasNext()) {
-                home.getCamera().setCamera(cameraPathIterator.next());
+                Camera camera = cameraPathIterator.next();
+                home.getCamera().setCamera(camera);
+                controller.setTime(camera.getTime());
               } else {
                 pausePlayback();
               }
@@ -988,7 +1191,8 @@ public class VideoPanel extends JPanel implements DialogView {
   private Camera [] getVideoFramesPath(int frameRate) {
     List<Camera> videoFramesPath = new ArrayList<Camera>();
     final float moveDistancePerFrame = 240000f / 3600 / frameRate;  // 3 cm/frame = 1800 m / 3600 s / 25 frame/s = 2.4 km/h
-    final float moveAnglePerFrame = (float)(Math.PI / 180 * 30 / frameRate); 
+    final float moveAnglePerFrame = (float)(Math.PI / 180 * 30 / frameRate);
+    final float elapsedTimePerFrame = 345600 / frameRate * 25; // 250 frame/day at 25 frame/second
     
     List<Camera> cameraPath = this.controller.getCameraPath();
     Camera camera = cameraPath.get(0);
@@ -998,6 +1202,7 @@ public class VideoPanel extends JPanel implements DialogView {
     float yaw = camera.getYaw(); 
     float pitch = camera.getPitch(); 
     float fieldOfView = camera.getFieldOfView();
+    long  time = camera.getTime();
     videoFramesPath.add(camera.clone());
     
     for (int i = 1; i < cameraPath.size(); i++) {
@@ -1008,15 +1213,17 @@ public class VideoPanel extends JPanel implements DialogView {
       float newYaw = camera.getYaw(); 
       float newPitch = camera.getPitch(); 
       float newFieldOfView = camera.getFieldOfView();
+      long  newTime = camera.getTime();
       
       float distance = new Point3f(x, y, z).distance(new Point3f(newX, newY, newZ));
       float moveCount = distance / moveDistancePerFrame;
       float yawAngleCount = Math.abs(newYaw - yaw) / moveAnglePerFrame;
       float pitchAngleCount = Math.abs(newPitch - pitch) / moveAnglePerFrame;
       float fieldOfViewAngleCount = Math.abs(newFieldOfView - fieldOfView) / moveAnglePerFrame;
+      float timeCount = Math.abs(newTime - time) / elapsedTimePerFrame;
 
       int frameCount = (int)Math.max(moveCount, Math.max(yawAngleCount, 
-          Math.max(pitchAngleCount, fieldOfViewAngleCount)));
+          Math.max(pitchAngleCount, Math.max(fieldOfViewAngleCount, timeCount))));
       
       float deltaX = (newX - x) / frameCount;
       float deltaY = (newY - y) / frameCount;
@@ -1024,12 +1231,15 @@ public class VideoPanel extends JPanel implements DialogView {
       float deltaYawAngle = (newYaw - yaw) / frameCount;
       float deltaPitchAngle = (newPitch - pitch) / frameCount;
       float deltaFieldOfViewAngle = (newFieldOfView - fieldOfView) / frameCount;
+      long deltaTime = Math.round(((double)newTime - time) / frameCount);
       
       for (int j = 1; j <= frameCount; j++) {
         videoFramesPath.add(new Camera(
             x + deltaX * j, y + deltaY * j, z + deltaZ * j, 
             yaw + deltaYawAngle * j, pitch + deltaPitchAngle * j, 
-            fieldOfView + deltaFieldOfViewAngle * j));                    
+            fieldOfView + deltaFieldOfViewAngle * j,
+            time + deltaTime * j,
+            Camera.Lens.NORMAL));
       }
       
       x = newX;
@@ -1038,6 +1248,7 @@ public class VideoPanel extends JPanel implements DialogView {
       yaw = newYaw;
       pitch = newPitch;
       fieldOfView = newFieldOfView;
+      time = newTime;
     }
 
     return videoFramesPath.toArray(new Camera [videoFramesPath.size()]);
@@ -1049,11 +1260,16 @@ public class VideoPanel extends JPanel implements DialogView {
   private void startVideoCreation() {
     ActionMap actionMap = getActionMap();
     actionMap.get(ActionType.SAVE_VIDEO).setEnabled(false);
+    this.createButton.setAction(getActionMap().get(ActionType.STOP_VIDEO_CREATION));
     actionMap.get(ActionType.RECORD).setEnabled(false);
     actionMap.get(ActionType.DELETE_CAMERA_PATH).setEnabled(false);
     actionMap.get(ActionType.DELETE_LAST_RECORD).setEnabled(false);
+    getRootPane().setDefaultButton(this.createButton);
     this.videoFormatComboBox.setEnabled(false);
     this.qualitySlider.setEnabled(false);
+    this.dateSpinner.setEnabled(false);
+    this.timeSpinner.setEnabled(false);
+    this.ceilingLightEnabledCheckBox.setEnabled(false);
     this.statusLayout.show(this.statusPanel, PROGRESS_CARD);
     this.progressBar.setIndeterminate(true);
     this.progressLabel.setText("");
@@ -1074,6 +1290,7 @@ public class VideoPanel extends JPanel implements DialogView {
    * Caution : this method must be thread safe because it's called from an executor. 
    */
   private void computeVideo(Home home) {
+    this.videoCreationStartTime = System.currentTimeMillis();
     int frameRate = this.controller.getFrameRate();
     int quality = this.controller.getQuality();
     int width = this.controller.getWidth();
@@ -1140,8 +1357,14 @@ public class VideoPanel extends JPanel implements DialogView {
             actionMap.get(ActionType.RECORD).setEnabled(true);
             actionMap.get(ActionType.DELETE_CAMERA_PATH).setEnabled(true);
             actionMap.get(ActionType.DELETE_LAST_RECORD).setEnabled(true);
+            if (videoFile != null) {
+              getRootPane().setDefaultButton(saveButton);
+            }
             videoFormatComboBox.setEnabled(true);
             qualitySlider.setEnabled(true);
+            dateSpinner.setEnabled(true);
+            timeSpinner.setEnabled(true);
+            ceilingLightEnabledCheckBox.setEnabled(true);
             statusLayout.show(statusPanel, TIP_CARD);
             videoCreationExecutor = null;
           }
@@ -1166,11 +1389,21 @@ public class VideoPanel extends JPanel implements DialogView {
   /**
    * Stops video creation.
    */
-  private void stopVideoCreation() {
-    if (this.videoCreationExecutor != null) {
-      // Interrupt executor thread
-      this.videoCreationExecutor.shutdownNow();
-      this.videoCreationExecutor = null;
+  private void stopVideoCreation(boolean confirmStop) {
+    if (this.videoCreationExecutor != null
+        // Confirm the stop if a rendering has been running for more than 30 s 
+        && (!confirmStop
+            || System.currentTimeMillis() - this.videoCreationStartTime < MINIMUM_DELAY_BEFORE_DISCARDING_WITHOUT_WARNING
+            || JOptionPane.showConfirmDialog(getRootPane(), 
+                  this.preferences.getLocalizedString(VideoPanel.class, "confirmStopCreation.message"),
+                  this.preferences.getLocalizedString(VideoPanel.class, "confirmStopCreation.title"), 
+                  JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.YES_OPTION)) {
+      if (this.videoCreationExecutor != null) { // Check a second time in case rendering stopped meanwhile
+        // Interrupt executor thread
+        this.videoCreationExecutor.shutdownNow();
+        this.videoCreationExecutor = null;
+        this.createButton.setAction(getActionMap().get(ActionType.START_VIDEO_CREATION));
+      }
     }
   }
 
@@ -1262,6 +1495,7 @@ public class VideoPanel extends JPanel implements DialogView {
   private void close() {
     Window window = SwingUtilities.getWindowAncestor(this);
     if (window.isDisplayable()) {
+      ToolTipManager.sharedInstance().unregisterComponent(this.qualitySlider);
       window.dispose();
     }    
   }
