@@ -20,9 +20,12 @@
 package com.eteks.sweethome3d.tools;
 
 import java.io.File;
+import java.io.FileFilter;
 import java.io.IOException;
 import java.util.MissingResourceException;
 import java.util.ResourceBundle;
+import java.util.Timer;
+import java.util.TimerTask;
 import java.util.UUID;
 
 import com.apple.eio.FileManager;
@@ -62,7 +65,7 @@ public class OperatingSystem {
       temporarySubFolder = "work";
     }
     temporarySubFolder = System.getProperty(
-        "com.eteks.sweethome3d.tools.OperatingSystem.TemporarySubFolder", temporarySubFolder);
+        "com.eteks.sweethome3d.tools.temporarySubFolder", temporarySubFolder);
     TEMPORARY_SUB_FOLDER = temporarySubFolder;
     TEMPORARY_SESSION_SUB_FOLDER = UUID.randomUUID().toString();
   }
@@ -108,7 +111,13 @@ public class OperatingSystem {
    * @throws IOException if the file couldn't be created
    */
   public static File createTemporaryFile(String prefix, String suffix) throws IOException {
-    File temporaryFolder = getDefaultTemporaryFolder(true);
+    File temporaryFolder;
+    try {
+      temporaryFolder = getDefaultTemporaryFolder(true);
+    } catch (IOException ex) {
+      // In case creating default temporary folder failed, use default temporary files folder
+      temporaryFolder = null;
+    }
     File temporaryFile = File.createTempFile(prefix, suffix, temporaryFolder);
     temporaryFile.deleteOnExit();
     return temporaryFile;
@@ -135,24 +144,59 @@ public class OperatingSystem {
    * Returns the default folder used to store temporary files created in the program.
    */
   private synchronized static File getDefaultTemporaryFolder(boolean create) throws IOException {
-    if (TEMPORARY_SUB_FOLDER != null
-        && !isWindows()) {
-      // Under Windows, always use system temporary folder until the bug #6962458 is fixed 
-      // See http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6962458 for more information
+    if (TEMPORARY_SUB_FOLDER != null) {
       File temporaryFolder = new File(getDefaultApplicationFolder(), TEMPORARY_SUB_FOLDER);
-      String versionPrefix = Home.CURRENT_VERSION + "-";
-      File sessionTemporaryFolder = new File(temporaryFolder, 
+      final String versionPrefix = Home.CURRENT_VERSION + "-";
+      final File sessionTemporaryFolder = new File(temporaryFolder, 
           versionPrefix + TEMPORARY_SESSION_SUB_FOLDER);      
       if (!sessionTemporaryFolder.exists()) {
-        if (create) {
-          // Create temporary folder  
-          if (!sessionTemporaryFolder.mkdirs()) {
-            throw new IOException("Can't create temporary folder " + sessionTemporaryFolder);
-          } else {
-            sessionTemporaryFolder.deleteOnExit();
-          }
+        // Retrieve existing folders working with same Sweet Home 3D version in temporary folder
+        final File [] siblingTemporaryFolders = temporaryFolder.listFiles(new FileFilter() {
+            public boolean accept(File file) {
+              return file.isDirectory() 
+                  && file.getName().startsWith(versionPrefix);
+            }
+          });
+        
+        // Create temporary folder  
+        if (!sessionTemporaryFolder.mkdirs()) {
+          throw new IOException("Can't create temporary folder " + sessionTemporaryFolder);
         } else {
-          return null;
+          sessionTemporaryFolder.deleteOnExit();
+        }
+        
+        // Launch a timer that updates modification date of the temporary folder each minute
+        final long updateDelay = 60000;
+        new Timer(true).schedule(new TimerTask() {
+            @Override
+            public void run() {
+              // Ensure modification date is always growing in case system time was adjusted
+              sessionTemporaryFolder.setLastModified(Math.max(System.currentTimeMillis(),
+                  sessionTemporaryFolder.lastModified() + updateDelay));
+            }
+          }, updateDelay, updateDelay);
+        
+        if (siblingTemporaryFolders != null
+            && siblingTemporaryFolders.length > 0) {
+          // Launch a timer that will delete in 10 min temporary folders older than a week 
+          final long deleteDelay = 10 * 60000;
+          final long age = 7 * 24 * 3600000;
+          new Timer(true).schedule(new TimerTask() {
+              @Override
+              public void run() {
+                long now = System.currentTimeMillis();
+                for (File siblingTemporaryFolder : siblingTemporaryFolders) {
+                  if (siblingTemporaryFolder.exists()
+                      && now - siblingTemporaryFolder.lastModified() > age) {
+                    File [] temporaryFiles = siblingTemporaryFolder.listFiles();
+                    for (File temporaryFile : temporaryFiles) {
+                      temporaryFile.delete();
+                    }
+                    siblingTemporaryFolder.delete();
+                  }
+                }
+              }
+            }, deleteDelay);
         }
       }
       return sessionTemporaryFolder;
