@@ -34,6 +34,7 @@ import javax.swing.undo.UndoableEditSupport;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeDoorOrWindow;
 import com.eteks.sweethome3d.model.HomeFurnitureGroup;
+import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
 import com.eteks.sweethome3d.model.Selectable;
@@ -47,13 +48,20 @@ public class HomeFurnitureController implements Controller {
   /**
    * The properties that may be edited by the view associated to this controller. 
    */
-  public enum Property {NAME, NAME_VISIBLE, X, Y, ELEVATION, ANGLE_IN_DEGREES, 
-      WIDTH, DEPTH,  HEIGHT, COLOR, PAINT, VISIBLE, MODEL_MIRRORED, RESIZABLE}
+  public enum Property {NAME, NAME_VISIBLE, X, Y, ELEVATION, ANGLE_IN_DEGREES, BASE_PLAN_ITEM, 
+      WIDTH, DEPTH,  HEIGHT, PROPORTIONAL, COLOR, PAINT, SHININESS, VISIBLE, MODEL_MIRRORED, LIGHT_POWER, 
+      RESIZABLE, DEFORMABLE, TEXTURABLE}
+  
   /**
    * The possible values for {@linkplain #getPaint() paint type}.
    */
   public enum FurniturePaint {DEFAULT, COLORED, TEXTURED} 
 
+  /**
+   * The possible values for {@linkplain #getShininess() shininess type}.
+   */
+  public enum FurnitureShininess {DEFAULT, MATT, SHINY} 
+
   private final Home                  home;
   private final UserPreferences       preferences;
   private final ViewFactory           viewFactory;
@@ -72,11 +80,19 @@ public class HomeFurnitureController implements Controller {
   private Float   width;
   private Float   depth;
   private Float   height;
+  private boolean proportional;
   private Integer color;
-  private FurniturePaint   furniturePaint;
+  private FurniturePaint     paint;
+  private FurnitureShininess shininess;
   private Boolean visible;
   private Boolean modelMirrored;
+  private Boolean basePlanItem;
+  private boolean basePlanItemEditable;
+  private boolean lightPowerEditable;
+  private Float   lightPower;
   private boolean resizable;
+  private boolean deformable;
+  private boolean texturable;
   
   /**
    * Creates the controller of home furniture view with undo support.
@@ -173,16 +189,24 @@ public class HomeFurnitureController implements Controller {
       setX(null);
       setY(null);
       setElevation(null);
-      setWidth(null);
-      setDepth(null);
-      setHeight(null);
+      this.basePlanItemEditable = false;
+      setWidth(null, false);
+      setDepth(null, false);
+      setHeight(null, false);
       setColor(null);
       if (textureController != null) {
         textureController.setTexture(null);
       }
       setPaint(null);
+      setShininess(null);
       setVisible(null);
       setModelMirrored(null);
+      this.lightPowerEditable = false;
+      setLightPower(null);
+      setResizable(true);
+      setDeformable(true);
+      setTexturable(true);
+      setProportional(false);
     } else {
       // Search the common properties among selected furniture
       HomePieceOfFurniture firstPiece = selectedFurniture.get(0);
@@ -246,6 +270,23 @@ public class HomeFurnitureController implements Controller {
       }
       setElevation(elevation);
 
+      boolean basePlanItemEditable = !firstPiece.isDoorOrWindow();
+      for (int i = 1; !basePlanItemEditable && i < selectedFurniture.size(); i++) {
+        if (!selectedFurniture.get(i).isDoorOrWindow()) {
+          basePlanItemEditable = true;
+        }
+      }
+      this.basePlanItemEditable = basePlanItemEditable;
+
+      Boolean basePlanItem = !firstPiece.isMovable();
+      for (int i = 1; i < selectedFurniture.size(); i++) {
+        if (basePlanItem.booleanValue() != !selectedFurniture.get(i).isMovable()) {
+          basePlanItem = null;
+          break;
+        }
+      }
+      setBasePlanItem(basePlanItem);
+
       Float width = firstPiece.getWidth();
       for (int i = 1; i < selectedFurniture.size(); i++) {
         if (width.floatValue() != selectedFurniture.get(i).getWidth()) {
@@ -253,7 +294,7 @@ public class HomeFurnitureController implements Controller {
           break;
         }
       }
-      setWidth(width);
+      setWidth(width, false);
 
       Float depth = firstPiece.getDepth();
       for (int i = 1; i < selectedFurniture.size(); i++) {
@@ -262,7 +303,7 @@ public class HomeFurnitureController implements Controller {
           break;
         }
       }
-      setDepth(depth);
+      setDepth(depth, false);
 
       Float height = firstPiece.getHeight();
       for (int i = 1; i < selectedFurniture.size(); i++) {
@@ -271,7 +312,7 @@ public class HomeFurnitureController implements Controller {
           break;
         }
       }
-      setHeight(height);
+      setHeight(height, false);
 
       Integer color = firstPiece.getColor();
       if (color != null) {
@@ -319,6 +360,22 @@ public class HomeFurnitureController implements Controller {
         setPaint(null);
       }
 
+      Float firstPieceShininess = firstPiece.getShininess();
+      FurnitureShininess shininess = firstPieceShininess == null 
+          ? FurnitureShininess.DEFAULT
+          : (firstPieceShininess.floatValue() == 0
+              ? FurnitureShininess.MATT
+              : FurnitureShininess.SHINY);
+      for (int i = 1; i < selectedFurniture.size(); i++) {
+        HomePieceOfFurniture piece = selectedFurniture.get(i);
+        if (firstPieceShininess != piece.getShininess()
+            || (firstPieceShininess != null && !firstPieceShininess.equals(piece.getShininess()))) {
+          shininess = null;
+          break;
+        }
+      }
+      setShininess(shininess);
+
       Boolean visible = firstPiece.isVisible();
       for (int i = 1; i < selectedFurniture.size(); i++) {
         if (visible != selectedFurniture.get(i).isVisible()) {
@@ -336,6 +393,27 @@ public class HomeFurnitureController implements Controller {
         }
       }
       setModelMirrored(modelMirrored);     
+
+      boolean lightPowerEditable = firstPiece instanceof HomeLight;
+      for (int i = 1; lightPowerEditable && i < selectedFurniture.size(); i++) {
+        if (!(selectedFurniture.get(i) instanceof HomeLight)) {
+          lightPowerEditable = false;
+        }
+      }
+      this.lightPowerEditable = lightPowerEditable;
+
+      if (lightPowerEditable) {
+        Float lightPower = ((HomeLight)firstPiece).getPower();
+        for (int i = 1; i < selectedFurniture.size(); i++) {
+          if (lightPower.floatValue() != ((HomeLight)selectedFurniture.get(i)).getPower()) {
+            lightPower = null;
+            break;
+          }
+        }
+        setLightPower(lightPower);
+      } else {
+        setLightPower(null);
+      }
       
       // Enable size components only if all pieces are resizable
       Boolean resizable = firstPiece.isResizable();
@@ -346,6 +424,27 @@ public class HomeFurnitureController implements Controller {
         }
       }
       setResizable(resizable != null && resizable.booleanValue());
+
+      Boolean deformable = firstPiece.isDeformable();
+      for (int i = 1; i < selectedFurniture.size(); i++) {
+        if (deformable.booleanValue() != selectedFurniture.get(i).isDeformable()) {
+          deformable = null;
+          break;
+        }
+      }
+      setDeformable(deformable != null && deformable.booleanValue());
+      if (!isDeformable()) {
+        setProportional(true);
+      }
+
+      Boolean texturable = firstPiece.isTexturable();
+      for (int i = 1; i < selectedFurniture.size(); i++) {
+        if (texturable.booleanValue() != selectedFurniture.get(i).isTexturable()) {
+          texturable = null;
+          break;
+        }
+      }
+      setTexturable(texturable == null || texturable.booleanValue());
     }
   }  
   
@@ -457,14 +556,52 @@ public class HomeFurnitureController implements Controller {
     return this.angleInDegrees;
   }
   
+  /**
+   * Returns <code>true</code> if base plan item is an editable property.
+   */
+  public boolean isBasePlanItemEditable() {
+    return this.basePlanItemEditable;
+  }
+  
+  /**
+   * Sets whether furniture is a base plan item or not.
+   */
+  public void setBasePlanItem(Boolean basePlanItem) {
+    if (basePlanItem != this.basePlanItem) {
+      Boolean oldMovable = this.basePlanItem;
+      this.basePlanItem = basePlanItem;
+      this.propertyChangeSupport.firePropertyChange(Property.BASE_PLAN_ITEM.name(), oldMovable, basePlanItem);
+    }
+  }
+
+  /**
+   * Returns whether furniture is a base plan item or not.
+   */
+  public Boolean getBasePlanItem() {
+    return this.basePlanItem;
+  }
+  
   /**
    * Sets the edited width.
    */
   public void setWidth(Float width) {
+    setWidth(width, isProportional());
+  }
+
+  private void setWidth(Float width, boolean updateDepthAndHeight) {
     if (width != this.width) {
       Float oldWidth = this.width;
       this.width = width;
       this.propertyChangeSupport.firePropertyChange(Property.WIDTH.name(), oldWidth, width);
+      if (oldWidth != null && width != null && updateDepthAndHeight) {
+        float ratio = width / oldWidth;
+        if (getDepth() != null) {
+          setDepth(getDepth() * ratio, false);
+        }
+        if (getHeight() != null) {
+          setHeight(getHeight() * ratio, false);
+        }
+      }
     }
   }
 
@@ -479,10 +616,23 @@ public class HomeFurnitureController implements Controller {
    * Sets the edited depth.
    */
   public void setDepth(Float depth) {
+    setDepth(depth, isProportional());
+  }
+
+  private void setDepth(Float depth, boolean updateWidthAndHeight) {
     if (depth != this.depth) {
       Float oldDepth = this.depth;
       this.depth = depth;
       this.propertyChangeSupport.firePropertyChange(Property.DEPTH.name(), oldDepth, depth);
+      if (oldDepth != null && depth != null && updateWidthAndHeight) {
+        float ratio = depth / oldDepth;
+        if (getWidth() != null) {
+          setWidth(getWidth() * ratio, false);
+        }
+        if (getHeight() != null) {
+          setHeight(getHeight() * ratio, false);
+        }
+      }
     }
   }
 
@@ -497,10 +647,23 @@ public class HomeFurnitureController implements Controller {
    * Sets the edited height.
    */
   public void setHeight(Float height) {
+    setHeight(height, isProportional());
+  }
+
+  public void setHeight(Float height, boolean updateWidthAndDepth) {
     if (height != this.height) {
       Float oldHeight = this.height;
       this.height = height;
       this.propertyChangeSupport.firePropertyChange(Property.HEIGHT.name(), oldHeight, height);
+      if (oldHeight != null && height != null && updateWidthAndDepth) {
+        float ratio = height / oldHeight;
+        if (getWidth() != null) {
+          setWidth(getWidth() * ratio, false);
+        }
+        if (getDepth() != null) {
+          setDepth(getDepth() * ratio, false);
+        }
+      }
     }
   }
 
@@ -511,6 +674,24 @@ public class HomeFurnitureController implements Controller {
     return this.height;
   }
   
+  /**
+   * Sets whether furniture proportions should be kept.
+   */
+  public void setProportional(boolean proportional) {
+    if (proportional != this.proportional) {
+      boolean oldProportional = this.proportional;
+      this.proportional = proportional;
+      this.propertyChangeSupport.firePropertyChange(Property.PROPORTIONAL.name(), oldProportional, proportional);
+    }
+  }
+
+  /**
+   * Returns whether furniture proportions should be kept or not.
+   */
+  public boolean isProportional() {
+    return this.proportional;
+  }
+  
   /**
    * Sets the edited color.
    */
@@ -532,11 +713,11 @@ public class HomeFurnitureController implements Controller {
   /**
    * Sets whether the piece is colored, textured or unknown painted.
    */
-  public void setPaint(FurniturePaint furniturePaint) {
-    if (furniturePaint != this.furniturePaint) {
-      FurniturePaint oldPaint = this.furniturePaint;
-      this.furniturePaint = furniturePaint;
-      this.propertyChangeSupport.firePropertyChange(Property.PAINT.name(), oldPaint, furniturePaint);
+  public void setPaint(FurniturePaint paint) {
+    if (paint != this.paint) {
+      FurniturePaint oldPaint = this.paint;
+      this.paint = paint;
+      this.propertyChangeSupport.firePropertyChange(Property.PAINT.name(), oldPaint, paint);
     }
   }
   
@@ -544,7 +725,25 @@ public class HomeFurnitureController implements Controller {
    * Returns whether the piece is colored, textured or unknown painted.
    */
   public FurniturePaint getPaint() {
-    return this.furniturePaint;
+    return this.paint;
+  }
+
+  /**
+   * Sets whether the piece shininess is the default one, matt, shiny or unknown.
+   */
+  public void setShininess(FurnitureShininess shininess) {
+    if (shininess != this.shininess) {
+      FurnitureShininess oldShininess = this.shininess;
+      this.shininess = shininess;
+      this.propertyChangeSupport.firePropertyChange(Property.SHININESS.name(), oldShininess, shininess);
+    }
+  }
+  
+  /**
+   * Returns whether the piece is shininess is the default one, matt, shiny or unknown.
+   */
+  public FurnitureShininess getShininess() {
+    return this.shininess;
   }
 
   /**
@@ -583,10 +782,35 @@ public class HomeFurnitureController implements Controller {
     return this.modelMirrored;
   }
   
+  /**
+   * Returns <code>true</code> if light power is an editable property.
+   */
+  public boolean isLightPowerEditable() {
+    return this.lightPowerEditable;
+  }
+  
+  /**
+   * Returns the edited light power.
+   */
+  public Float getLightPower() {
+    return this.lightPower;
+  }
+  
+  /**
+   * Sets the edited light power.
+   */
+  public void setLightPower(Float lightPower) {
+    if (lightPower != this.lightPower) {
+      Float oldLightPower = this.lightPower;
+      this.lightPower = lightPower;
+      this.propertyChangeSupport.firePropertyChange(Property.LIGHT_POWER.name(), oldLightPower, lightPower);
+    }
+  }
+
   /**
    * Sets whether furniture model can be resized or not.
    */
-  public void setResizable(boolean resizable) {
+  private void setResizable(boolean resizable) {
     if (resizable != this.resizable) {
       boolean oldResizable = this.resizable;
       this.resizable = resizable;
@@ -600,14 +824,49 @@ public class HomeFurnitureController implements Controller {
   public boolean isResizable() {
     return this.resizable;
   }
+
+  /**
+   * Sets whether furniture model can be deformed or not.
+   */
+  private void setDeformable(boolean deformable) {
+    if (deformable != this.deformable) {
+      boolean oldDeformable = this.deformable;
+      this.deformable = deformable;
+      this.propertyChangeSupport.firePropertyChange(Property.DEFORMABLE.name(), oldDeformable, deformable);
+    }
+  }
+
+  /**
+   * Returns whether furniture model can be deformed or not.
+   */
+  public boolean isDeformable() {
+    return this.deformable;
+  }
+  
+  /**
+   * Sets whether the color or the texture of the furniture model can be changed or not.
+   */
+  private void setTexturable(boolean texturable) {
+    if (texturable != this.texturable) {
+      boolean oldTexturable = this.texturable;
+      this.texturable = texturable;
+      this.propertyChangeSupport.firePropertyChange(Property.TEXTURABLE.name(), oldTexturable, texturable);
+    }
+  }
+
+  /**
+   * Returns whether the color or the texture of the furniture model can be changed or not.
+   */
+  public boolean isTexturable() {
+    return this.texturable;
+  }
   
   /**
    * Controls the modification of selected furniture in the edited home.
    */
   public void modifyFurniture() {
     List<Selectable> oldSelection = this.home.getSelectedItems(); 
-    List<HomePieceOfFurniture> selectedFurniture = 
-        Home.getFurnitureSubList(oldSelection);
+    List<HomePieceOfFurniture> selectedFurniture = Home.getFurnitureSubList(oldSelection);
     if (!selectedFurniture.isEmpty()) {
       String name = getName();
       Boolean nameVisible = getNameVisible();
@@ -616,6 +875,7 @@ public class HomeFurnitureController implements Controller {
       Float elevation = getElevation();
       Float angle = getAngleInDegrees() != null
           ? (float)Math.toRadians(getAngleInDegrees())  : null;
+      Boolean basePlanItem = getBasePlanItem();
       Float width = getWidth();
       Float depth = getDepth();
       Float height = getHeight();
@@ -629,28 +889,39 @@ public class HomeFurnitureController implements Controller {
       } else {
         texture = null;
       }
+      boolean defaultShininess = getShininess() == FurnitureShininess.DEFAULT;
+      Float shininess = getShininess() == FurnitureShininess.SHINY
+          ? new Float(0.5f)
+          : (getShininess() == FurnitureShininess.MATT
+              ? new Float(0) : null);
       Boolean visible = getVisible();
       Boolean modelMirrored = getModelMirrored();
+      Float lightPower = getLightPower();
       
       // Create an array of modified furniture with their current properties values
       ModifiedPieceOfFurniture [] modifiedFurniture = 
           new ModifiedPieceOfFurniture [selectedFurniture.size()]; 
       for (int i = 0; i < modifiedFurniture.length; i++) {
         HomePieceOfFurniture piece = selectedFurniture.get(i);
-        if (piece instanceof HomeDoorOrWindow) {
+        if (piece instanceof HomeLight) {
+          modifiedFurniture [i] = new ModifiedLight((HomeLight)piece);
+        } else if (piece instanceof HomeDoorOrWindow) {
           modifiedFurniture [i] = new ModifiedDoorOrWindow((HomeDoorOrWindow)piece);
+        } else if (piece instanceof HomeFurnitureGroup) {
+          modifiedFurniture [i] = new ModifiedFurnitureGroup((HomeFurnitureGroup)piece);
         } else {
           modifiedFurniture [i] = new ModifiedPieceOfFurniture(piece);
         }
       }
       // Apply modification
-      doModifyFurniture(modifiedFurniture, 
-          name, nameVisible, x, y, width, depth, height, elevation, angle, defaultColorsAndTextures, color, texture, visible, modelMirrored); 
+      doModifyFurniture(modifiedFurniture, name, nameVisible, x, y, elevation, angle, basePlanItem, width, depth, height, 
+          defaultColorsAndTextures, color, texture, defaultShininess, shininess, visible, modelMirrored, lightPower);
+      List<Selectable> newSelection = this.home.getSelectedItems(); 
       if (this.undoSupport != null) {
         UndoableEdit undoableEdit = new FurnitureModificationUndoableEdit(
-            this.home, this.preferences, oldSelection, modifiedFurniture, 
-            name, nameVisible, x, y, width, depth, height, elevation, angle, 
-            defaultColorsAndTextures, color, texture, modelMirrored, visible);
+            this.home, this.preferences, oldSelection, newSelection, modifiedFurniture, 
+            name, nameVisible, x, y, elevation, angle, basePlanItem, width, depth, height, 
+            defaultColorsAndTextures, color, texture, defaultShininess, shininess, visible, modelMirrored, lightPower);
         this.undoSupport.postEdit(undoableEdit);
       }
     }
@@ -665,57 +936,68 @@ public class HomeFurnitureController implements Controller {
     private final UserPreferences             preferences;
     private final ModifiedPieceOfFurniture [] modifiedFurniture;
     private final List<Selectable>            oldSelection;
+    private final List<Selectable>            newSelection;
     private final String                      name;
     private final Boolean                     nameVisible;
     private final Float                       x;
     private final Float                       y;
+    private final Float                       elevation;
+    private final Float                       angle;
+    private final Boolean                     basePlanItem;
     private final Float                       width;
     private final Float                       depth;
     private final Float                       height;
-    private final Float                       elevation;
-    private final Float                       angle;
     private final boolean                     defaultColorsAndTextures;
     private final Integer                     color;
     private final HomeTexture                 texture;
-    private final Boolean                     modelMirrored;
+    private final boolean                     defaultShininess;
+    private final Float                       shininess;
     private final Boolean                     visible;
+    private final Boolean                     modelMirrored;
+    private final Float                       lightPower;
 
     private FurnitureModificationUndoableEdit(Home home,
                                               UserPreferences preferences, 
                                               List<Selectable> oldSelection,
+                                              List<Selectable> newSelection,
                                               ModifiedPieceOfFurniture [] modifiedFurniture,
                                               String name, Boolean nameVisible, 
-                                              Float x, Float y,
+                                              Float x, Float y, Float elevation, Float angle, Boolean basePlanItem, 
                                               Float width, Float depth, Float height,
-                                              Float elevation, Float angle,
-                                              boolean defaultColorsAndTextures,
-                                              Integer color, HomeTexture texture,
+                                              boolean defaultColorsAndTextures, Integer color, HomeTexture texture,
+                                              boolean defaultShininess, Float shininess,
+                                              Boolean visible,
                                               Boolean modelMirrored,
-                                              Boolean visible) {
+                                              Float lightPower) {
       this.home = home;
       this.preferences = preferences;
       this.oldSelection = oldSelection;
+      this.newSelection = newSelection;
       this.modifiedFurniture = modifiedFurniture;
       this.name = name;
       this.nameVisible = nameVisible;
       this.x = x;
       this.y = y;
+      this.elevation = elevation;
+      this.angle = angle;
+      this.basePlanItem = basePlanItem;
       this.width = width;
       this.depth = depth;
       this.height = height;
-      this.elevation = elevation;
-      this.angle = angle;
       this.defaultColorsAndTextures = defaultColorsAndTextures;
       this.color = color;
       this.texture = texture;
-      this.modelMirrored = modelMirrored;
+      this.defaultShininess = defaultShininess;
+      this.shininess = shininess;
       this.visible = visible;
+      this.modelMirrored = modelMirrored;
+      this.lightPower = lightPower;
     }
 
     @Override
     public void undo() throws CannotUndoException {
       super.undo();
-      undoModifyFurniture(this.modifiedFurniture); 
+      undoModifyFurniture(this.modifiedFurniture);
       home.setSelectedItems(this.oldSelection); 
     }
 
@@ -723,11 +1005,12 @@ public class HomeFurnitureController implements Controller {
     public void redo() throws CannotRedoException {
       super.redo();
       doModifyFurniture(this.modifiedFurniture, 
-          this.name, this.nameVisible, this.x, this.y, this.width, 
-          this.depth, this.height, this.elevation, this.angle, 
+          this.name, this.nameVisible, this.x, this.y, this.elevation, 
+          this.angle, this.basePlanItem, this.width, this.depth, this.height, 
           this.defaultColorsAndTextures, this.color, this.texture, 
-          this.visible, this.modelMirrored); 
-      home.setSelectedItems(this.oldSelection); 
+          this.defaultShininess, this.shininess,
+          this.visible, this.modelMirrored, this.lightPower); 
+      home.setSelectedItems(this.newSelection); 
     }
 
     @Override
@@ -742,46 +1025,72 @@ public class HomeFurnitureController implements Controller {
    */
   private static void doModifyFurniture(ModifiedPieceOfFurniture [] modifiedFurniture, 
                                         String name, Boolean nameVisible, 
-                                        Float x, Float y, Float width, 
-                                        Float depth, Float height, Float elevation, 
-                                        Float angle, boolean defaultColorsAndTextures, 
-                                        Integer color, HomeTexture texture,
-                                        Boolean visible, Boolean modelMirrored) {
+                                        Float x, Float y, Float elevation, Float angle, Boolean basePlanItem, 
+                                        Float width, Float depth, Float height, 
+                                        boolean defaultColorsAndTextures, Integer color, HomeTexture texture,
+                                        boolean defaultShininess, Float shininess,
+                                        Boolean visible, Boolean modelMirrored, 
+                                        Float lightPower) {
     for (ModifiedPieceOfFurniture modifiedPiece : modifiedFurniture) {
       HomePieceOfFurniture piece = modifiedPiece.getPieceOfFurniture();
-      piece.setName(name != null 
-          ? name : piece.getName());
-      piece.setNameVisible(nameVisible != null 
-          ? nameVisible : piece.isNameVisible());
-      piece.setX(x != null 
-          ? x.floatValue() : piece.getX());
-      piece.setY(y != null 
-          ? y.floatValue() : piece.getY());
-      piece.setAngle(angle != null ? angle.floatValue() : piece.getAngle());
+      if (name != null) {
+        piece.setName(name);
+      }
+      if (nameVisible != null) {
+        piece.setNameVisible(nameVisible);
+      }
+      if (x != null) {
+        piece.setX(x);
+      }
+      if (y != null) {
+        piece.setY(y);
+      }
+      if (elevation != null) {
+        piece.setElevation(elevation); 
+      }
+      if (angle != null) {
+        piece.setAngle(angle);
+      }
+      if (basePlanItem != null && !piece.isDoorOrWindow()) {
+        piece.setMovable(!basePlanItem);
+      }
       if (piece.isResizable()) {
-        piece.setWidth(width != null 
-            ? width.floatValue() : piece.getWidth());
-        piece.setDepth(depth != null  
-            ? depth.floatValue() : piece.getDepth());
-        piece.setHeight(height != null  
-            ? height.floatValue() : piece.getHeight());
-        piece.setModelMirrored(modelMirrored != null  
-            ? modelMirrored.booleanValue() : piece.isModelMirrored());
+        if (width != null) {
+          piece.setWidth(width);
+        }
+        if (depth != null) {
+          piece.setDepth(depth);
+        }
+        if (height != null) {
+          piece.setHeight(height);
+        }
+        if (modelMirrored != null) {
+          piece.setModelMirrored(modelMirrored);
+        }
       }
-      piece.setElevation(elevation != null 
-          ? elevation.floatValue() : piece.getElevation());
-      if (defaultColorsAndTextures) {
-        piece.setColor(null);
-        piece.setTexture(null);
-      } else if (texture != null) {
-        piece.setColor(null);
-        piece.setTexture(texture);
-      } else if (color != null) {
-        piece.setTexture(null);
-        piece.setColor(color);
+      if (piece.isTexturable()) {
+        if (defaultColorsAndTextures) {
+          piece.setColor(null);
+          piece.setTexture(null);
+        } else if (texture != null) {
+          piece.setColor(null);
+          piece.setTexture(texture);
+        } else if (color != null) {
+          piece.setTexture(null);
+          piece.setColor(color);
+        }
+        if (defaultShininess) {
+          piece.setShininess(null);
+        } else if (shininess != null) {
+          piece.setShininess(shininess);
+        }
+      }
+      if (visible != null) {
+        piece.setVisible(visible);
+      }
+      if (lightPower != null) {
+        ((HomeLight)piece).setPower(lightPower);
       }
-      piece.setVisible(visible != null 
-          ? visible.booleanValue() : piece.isVisible());
     }
   }
 
@@ -805,17 +1114,15 @@ public class HomeFurnitureController implements Controller {
     private final float                y;
     private final float                elevation;
     private final float                angle;
+    private final boolean              movable;
     private final float                width;
     private final float                depth;
     private final float                height;
     private final Integer              color;
     private final HomeTexture          texture;
+    private final Float                shininess;
     private final boolean              visible;
     private final boolean              modelMirrored;
-    private final float []             groupFurnitureX;
-    private final float []             groupFurnitureY;
-    private final float []             groupFurnitureWidth;
-    private final float []             groupFurnitureDepth;
 
     public ModifiedPieceOfFurniture(HomePieceOfFurniture piece) {
       this.piece = piece;
@@ -825,32 +1132,15 @@ public class HomeFurnitureController implements Controller {
       this.y = piece.getY();
       this.elevation = piece.getElevation();
       this.angle = piece.getAngle();
+      this.movable = piece.isMovable();
       this.width = piece.getWidth();
       this.depth = piece.getDepth();
       this.height = piece.getHeight();
       this.color = piece.getColor();
       this.texture = piece.getTexture();
+      this.shininess = piece.getShininess();
       this.visible = piece.isVisible();
       this.modelMirrored = piece.isModelMirrored();
-      if (piece instanceof HomeFurnitureGroup) {
-        List<HomePieceOfFurniture> groupFurniture = getGroupFurniture((HomeFurnitureGroup)piece);
-        this.groupFurnitureX = new float [groupFurniture.size()];
-        this.groupFurnitureY = new float [groupFurniture.size()];
-        this.groupFurnitureWidth = new float [groupFurniture.size()];
-        this.groupFurnitureDepth = new float [groupFurniture.size()];
-        for (int i = 0; i < groupFurniture.size(); i++) {
-          HomePieceOfFurniture groupPiece = groupFurniture.get(i);
-          this.groupFurnitureX [i] = groupPiece.getX();
-          this.groupFurnitureY [i] = groupPiece.getY();
-          this.groupFurnitureWidth [i] = groupPiece.getWidth();
-          this.groupFurnitureDepth [i] = groupPiece.getDepth();
-        }
-      } else {
-        this.groupFurnitureX = null;
-        this.groupFurnitureY = null;
-        this.groupFurnitureWidth = null;
-        this.groupFurnitureDepth = null;
-      }
     }
 
     public HomePieceOfFurniture getPieceOfFurniture() {
@@ -864,42 +1154,19 @@ public class HomeFurnitureController implements Controller {
       this.piece.setY(this.y);
       this.piece.setElevation(this.elevation);
       this.piece.setAngle(this.angle);
+      this.piece.setMovable(this.movable);
       if (this.piece.isResizable()) {
         this.piece.setWidth(this.width);
         this.piece.setDepth(this.depth);
         this.piece.setHeight(this.height);
-        this.piece.setModelMirrored(modelMirrored);
+        this.piece.setModelMirrored(this.modelMirrored);
+      }
+      if (this.piece.isTexturable()) {
+        this.piece.setColor(this.color);
+        this.piece.setTexture(this.texture);
+        this.piece.setShininess(this.shininess);
       }
-      this.piece.setColor(this.color);
-      this.piece.setTexture(this.texture);
       this.piece.setVisible(this.visible);
-      if (this.piece instanceof HomeFurnitureGroup) {
-        List<HomePieceOfFurniture> groupFurniture = getGroupFurniture((HomeFurnitureGroup)this.piece);
-        for (int i = 0; i < groupFurniture.size(); i++) {
-          HomePieceOfFurniture groupPiece = groupFurniture.get(i);
-          if (this.piece.isResizable()) {
-            // Restore group furniture location and size because resizing a group isn't reversible 
-            groupPiece.setX(this.groupFurnitureX [i]);
-            groupPiece.setY(this.groupFurnitureY [i]);
-            groupPiece.setWidth(this.groupFurnitureWidth [i]);
-            groupPiece.setDepth(this.groupFurnitureDepth [i]);
-          }
-        }
-      }
-    }
-    
-    /**
-     * Returns all the children of the given <code>furnitureGroup</code>.  
-     */
-    private List<HomePieceOfFurniture> getGroupFurniture(HomeFurnitureGroup furnitureGroup) {
-      List<HomePieceOfFurniture> pieces = new ArrayList<HomePieceOfFurniture>();
-      for (HomePieceOfFurniture piece : furnitureGroup.getFurniture()) {
-        pieces.add(piece);
-        if (piece instanceof HomeFurnitureGroup) {
-          pieces.addAll(getGroupFurniture((HomeFurnitureGroup)piece));
-        } 
-      }
-      return pieces;
     }
   }
   
@@ -919,4 +1186,77 @@ public class HomeFurnitureController implements Controller {
       ((HomeDoorOrWindow)getPieceOfFurniture()).setBoundToWall(this.boundToWall);
     }
   }
+
+  /**
+   * Stores the current properties values of a modified light.
+   */
+  private static class ModifiedLight extends ModifiedPieceOfFurniture {
+    private final float power;
+    
+    public ModifiedLight(HomeLight light) {
+      super(light);
+      this.power = light.getPower();
+    }
+
+    public void reset() {
+      super.reset();
+      ((HomeLight)getPieceOfFurniture()).setPower(this.power);
+    }
+  }
+
+  /**
+   * Stores the current properties values of a modified group.
+   */
+  private static class ModifiedFurnitureGroup extends ModifiedPieceOfFurniture {
+    private final float [] groupFurnitureX;
+    private final float [] groupFurnitureY;
+    private final float [] groupFurnitureWidth;
+    private final float [] groupFurnitureDepth;
+    
+    public ModifiedFurnitureGroup(HomeFurnitureGroup group) {
+      super(group);
+      List<HomePieceOfFurniture> groupFurniture = getGroupFurniture((HomeFurnitureGroup)group);
+      this.groupFurnitureX = new float [groupFurniture.size()];
+      this.groupFurnitureY = new float [groupFurniture.size()];
+      this.groupFurnitureWidth = new float [groupFurniture.size()];
+      this.groupFurnitureDepth = new float [groupFurniture.size()];
+      for (int i = 0; i < groupFurniture.size(); i++) {
+        HomePieceOfFurniture groupPiece = groupFurniture.get(i);
+        this.groupFurnitureX [i] = groupPiece.getX();
+        this.groupFurnitureY [i] = groupPiece.getY();
+        this.groupFurnitureWidth [i] = groupPiece.getWidth();
+        this.groupFurnitureDepth [i] = groupPiece.getDepth();
+      }
+    }
+
+    public void reset() {
+      super.reset();
+      HomeFurnitureGroup group = (HomeFurnitureGroup)getPieceOfFurniture();
+      List<HomePieceOfFurniture> groupFurniture = getGroupFurniture(group);
+      for (int i = 0; i < groupFurniture.size(); i++) {
+        HomePieceOfFurniture groupPiece = groupFurniture.get(i);
+        if (group.isResizable()) {
+          // Restore group furniture location and size because resizing a group isn't reversible 
+          groupPiece.setX(this.groupFurnitureX [i]);
+          groupPiece.setY(this.groupFurnitureY [i]);
+          groupPiece.setWidth(this.groupFurnitureWidth [i]);
+          groupPiece.setDepth(this.groupFurnitureDepth [i]);
+        }
+      }
+    }
+    
+    /**
+     * Returns all the children of the given <code>furnitureGroup</code>.  
+     */
+    private List<HomePieceOfFurniture> getGroupFurniture(HomeFurnitureGroup furnitureGroup) {
+      List<HomePieceOfFurniture> pieces = new ArrayList<HomePieceOfFurniture>();
+      for (HomePieceOfFurniture piece : furnitureGroup.getFurniture()) {
+        pieces.add(piece);
+        if (piece instanceof HomeFurnitureGroup) {
+          pieces.addAll(getGroupFurniture((HomeFurnitureGroup)piece));
+        } 
+      }
+      return pieces;
+    }
+  }
 }
